@@ -1,6 +1,6 @@
 # The LearningOnline Network
 #
-# $Id: Lond.pm,v 1.6 2014/01/01 17:41:37 raeburn Exp $
+# $Id: Lond.pm,v 1.8 2015/05/22 21:14:59 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -333,7 +333,7 @@ sub dump_course_id_handler {
     my ($udom,$since,$description,$instcodefilter,$ownerfilter,$coursefilter,
         $typefilter,$regexp_ok,$rtn_as_hash,$selfenrollonly,$catfilter,$showhidden,
         $caller,$cloner,$cc_clone_list,$cloneonly,$createdbefore,$createdafter,
-        $creationcontext,$domcloner,$hasuniquecode) = split(/:/,$tail);
+        $creationcontext,$domcloner,$hasuniquecode,$reqcrsdom,$reqinstcode) = split(/:/,$tail);
     my $now = time;
     my ($cloneruname,$clonerudom,%cc_clone);
     if (defined($description)) {
@@ -409,12 +409,16 @@ sub dump_course_id_handler {
     unless ($hasuniquecode) {
         $hasuniquecode = '.';
     }
+    if ($reqinstcode ne '') {
+        $reqinstcode = &unescape($reqinstcode);
+    }
     my $unpack = 1;
     if ($description eq '.' && $instcodefilter eq '.' && $ownerfilter eq '.' && 
         $typefilter eq '.') {
         $unpack = 0;
     }
     if (!defined($since)) { $since=0; }
+    my (%gotcodedefaults,%otcodedefaults);
     my $qresult='';
 
     my $hashref = &tie_domain_hash($udom, "nohist_courseids", &GDBM_WRCREAT())
@@ -434,12 +438,15 @@ sub dump_course_id_handler {
                 $lasttime = $hashref->{$lasttime_key};
                 next if ($lasttime<$since);
             }
-            my ($canclone,$valchange);
+            my ($canclone,$valchange,$clonefromcode);
             my $items = &Apache::lonnet::thaw_unescape($value);
             if (ref($items) eq 'HASH') {
                 if ($hashref->{$lasttime_key} eq '') {
                     next if ($since > 1);
                 }
+                if ($items->{'inst_code'}) {
+                    $clonefromcode = $items->{'inst_code'};
+                }
                 $is_hash =  1;
                 if ($domcloner) {
                     $canclone = 1;
@@ -465,6 +472,41 @@ sub dump_course_id_handler {
                                 }
                             }
                         }
+                        unless ($canclone) {
+                            if (($reqcrsdom eq $udom) && ($reqinstcode) && ($clonefromcode)) {
+                                if (grep(/\=/,@cloneable))  {
+                                    foreach my $cloner (@cloneable) {
+                                        if (($cloner ne '*') && ($cloner !~ /^\*\:$LONCAPA::match_domain$/) &&
+                                            ($cloner !~ /^$LONCAPA::match_username\:$LONCAPA::match_domain$/) && ($cloner ne '')) {
+                                            if ($cloner =~ /=/) {
+                                                my (%codedefaults,@code_order);
+                                                if (ref($gotcodedefaults{$udom}) eq 'HASH') {
+                                                    if (ref($gotcodedefaults{$udom}{'defaults'}) eq 'HASH') {
+                                                        %codedefaults = %{$gotcodedefaults{$udom}{'defaults'}};
+                                                    }
+                                                    if (ref($gotcodedefaults{$udom}{'order'}) eq 'ARRAY') {
+                                                        @code_order = @{$gotcodedefaults{$udom}{'order'}};
+                                                    }
+                                                } else {
+                                                    &Apache::lonnet::auto_instcode_defaults($udom,
+                                                                                            \%codedefaults,
+                                                                                            \@code_order);
+                                                    $gotcodedefaults{$udom}{'defaults'} = \%codedefaults;
+                                                    $gotcodedefaults{$udom}{'order'} = \@code_order;
+                                                }
+                                                if (@code_order > 0) {
+                                                    if (&Apache::lonnet::check_instcode_cloning(\%codedefaults,\@code_order,
+                                                                                                $cloner,$clonefromcode,$reqinstcode)) {
+                                                        $canclone = 1;
+                                                        last;
+                                                    }
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                        }
                     } elsif (defined($cloneruname)) {
                         if ($cc_clone{$unesc_key}) {
                             $canclone = 1;
@@ -485,6 +527,24 @@ sub dump_course_id_handler {
                             }
                         }
                     }
+                    unless (($canclone) || ($items->{'cloners'})) {
+                        my %domdefs = &Apache::lonnet::get_domain_defaults($udom);
+                        if ($domdefs{'canclone'}) {
+                            unless ($domdefs{'canclone'} eq 'none') {
+                                if ($domdefs{'canclone'} eq 'domain') {
+                                    if ($clonerudom eq $udom) {
+                                        $canclone = 1;
+                                    }
+                                } elsif (($clonefromcode) && ($reqinstcode) &&
+                                         ($udom eq $reqcrsdom)) {
+                                    if (&Apache::lonnet::default_instcode_cloning($udom,$domdefs{'canclone'},
+                                                                                  $clonefromcode,$reqinstcode)) {
+                                        $canclone = 1;
+                                    }
+                                }
+                            }
+                        }
+                    }
                 }
                 if ($unpack || !$rtn_as_hash) {
                     $unesc_val{'descr'} = $items->{'description'};
