@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA 
 # edit mode helpers
 #
-# $Id: edit.pm,v 1.135 2011/08/09 16:12:21 bisitz Exp $
+# $Id: edit.pm,v 1.152 2014/11/28 18:23:03 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -161,7 +161,7 @@ sub tag_start {
     if ($target eq "edit") {
 	my $tag=$token->[1];
 	if (!$description) {
-	    $description=&Apache::lonxml::description($token);
+	    $description=&mt(&Apache::lonxml::description($token));
 	    if (!$description) { $description="&lt;$tag&gt;"; }
 	}
 	$result.= &start_table($token)."<tr><td>$description</td>
@@ -175,7 +175,7 @@ sub tag_start {
 #    "</tr><tr><td colspan=\"3\">\n";
 	my @help = Apache::lonxml::helpinfo($token);
 	if ($help[0]) {
-	    $result .= '</td><td align="right" valign="top">' .
+	    $result .= '</td><td class="LC_edit_problem_latexhelper">'.
 		Apache::loncommon::help_open_topic(@help);
 	} else { $result .= "</td><td>&nbsp;"; }
 	$result .= &end_row().&start_spanning_row();
@@ -203,7 +203,7 @@ sub start_table {
     }
     $Apache::edit::colordepth++;
     push(@Apache::edit::inserttag,$token->[1]);
-    my $result='<div align="right">';
+    my $result='<div>';
     $result.='<table bgcolor="'.$color.'" width="97%" border="0" cellspacing="3" cellpadding="2">';
     return $result;
 }
@@ -211,7 +211,7 @@ sub start_table {
 sub end_table {
     $Apache::edit::colordepth--;
     my $result='</table></div>';
-    $result.='<div align="left"><table><tr><td>';
+    $result.='<div><table><tr><td>';
 
     my ($tagname,$closingtag);
     if (defined($Apache::edit::inserttag[-2])) {
@@ -296,7 +296,7 @@ sub get_insert_list {
 	    my $descrip=$Apache::lonxml::insertlist{"$tag.description"};
 	    my $tagnum =$Apache::lonxml::insertlist{"$tag.num"};
 	    $options{$descrip} ="<option value=\"$tagnum\">".
-		$descrip."</option>\n";
+		&mt($descrip)."</option>\n";
 	}
 	foreach my $option (sort(keys(%options))) {$result.=$options{$option};}
 	if ($result) { $result='<option selected="selected"></option>'.$result; }
@@ -401,10 +401,8 @@ sub insert_functionplotresponse {
     return '
 <functionplotresponse>
 <functionplotelements>
-<spline />
 </functionplotelements>
 <functionplotruleset>
-<functionplotrule />
 </functionplotruleset>
 </functionplotresponse>';
 }
@@ -419,11 +417,51 @@ sub insert_backgroundplot {
 <backgroundplot />';
 }
 
+sub insert_plotobject {
+    return '
+<plotobject />';
+}
+
+sub insert_plotvector {
+    return '
+<plotvector />';
+}
+
+sub insert_drawvectorsum {
+    return '
+<drawvectorsum />';
+}
+
+
 sub insert_functionplotrule {
     return '
 <functionplotrule />';
 }
 
+sub insert_functionplotvectorrule {
+    return '
+<functionplotvectorrule />';
+}
+
+sub insert_functionplotvectorsumrule {
+    return '
+<functionplotvectorsumrule />';
+}
+
+sub insert_functionplotcustomrule {
+    return '
+<functionplotcustomrule>
+<answer type="loncapa/perl">
+# &fpr_val("label"), &fpr_f($x), &fpr_dfdx($x), &fpr_d2fdx2($x)
+# ($xs,$xe,$ys,$ye)=&fpr_vectorcoords("Name"), ($x,$y)=&fpr_objectcoords("Name")
+# &fpr_vectorlength("Name"), &fpr_vectorangle("Name")
+ 
+# Return 0 or 1
+return 1;
+</answer>
+</functionplotcustomrule>';
+}
+
 sub insert_functionplotruleset {
     return '
 <functionplotruleset>
@@ -450,6 +488,13 @@ sub insert_numericalresponse {
 </numericalresponse>';
 }
 
+sub insert_externalresponse {
+    return '
+<externalresponse url="" answer="" answerdisplay="" form="">
+    <textfield spellcheck="none" />
+</externalresponse>';
+}
+
 sub insert_customresponse {
     return '
 <customresponse>
@@ -622,6 +667,9 @@ sub insert_matchresponse {
 <matchresponse max="10">
     <foilgroup options="">
       <itemgroup>
+        <item>
+          <startouttext /><endouttext />
+        </item>
       </itemgroup>
       <foil>
          <startouttext /><endouttext />
@@ -706,6 +754,7 @@ sub js_change_detection {
     my $unsaved=&mt("There are unsaved changes");
     return (<<SCRIPT);
 <script type="text/javascript">
+// <![CDATA[
 var clean = true;
 var is_submit = false;
 var still_ask = false;
@@ -724,6 +773,7 @@ function unClean() {
      clean=false;
 }
 window.onbeforeunload = compareForm;
+// ]]>
 </script>
 SCRIPT
 }
@@ -746,6 +796,49 @@ sub submit_dont_ask {
     return ' onclick="is_submit=true;'.$extra_action.'" ';
 }
 
+sub js_update_linknum {
+    return (<<SCRIPT);
+<script type="text/javascript">
+// <![CDATA[
+function updateNumber(name,index,caller,textprompt) {
+    var pickitem = document.getElementById(name+'_'+index);
+    var picknumtext = document.getElementById(name+'_numtext_'+index);
+    if (pickitem.checked) {
+        var showval = '';
+        if (pickitem.value != 'nochoice') {
+            showval = pickitem.value;
+        } 
+        var picknum=prompt(textprompt,showval);
+        if (picknum == '' || picknum == null) {
+            if (caller == 'check') {
+                pickitem.checked=false;
+                pickitem.value='nochoice';
+            }
+        } else {
+            picknum.toString();
+            var regexdigit=/^\\d+\$/;
+            if (regexdigit.test(picknum)) {
+                pickitem.value = picknum;
+                picknumtext.innerHTML = '&nbsp;<a href="javascript:updateNumber(\\''+name+'\\',\\''+index+'\\',\\'link\\',\\''+textprompt+'\\');">'+picknum+'</a>';
+            } else {
+                if (caller == 'check') {
+                    pickitem.checked=false;
+                    pickitem.value='nochoice';
+                }
+                return;
+            }
+        }
+    } else {
+        pickitem.value = '';
+        picknumtext.innerHTML = '';
+    }
+}
+
+// ]]>
+</script>
+SCRIPT
+
+}
 
 sub textarea_sizes {
     my ($data)=@_;
@@ -788,7 +881,7 @@ sub editfield {
     if ($cols > 80) { $cols = 80; }
     if ($cols < $minwidth ) { $cols = $minwidth; }
     if ($rows < $minheight) { $rows = $minheight; }
-    if ($description) { $description=$description."<br />"; }
+    if ($description) { $description='<br />'.&mt($description).'<br />'; }
 
     # remove typesetting whitespace from between data and the end tag
     # to make the edit look prettier
@@ -800,8 +893,8 @@ sub editfield {
 	$Apache::lonxml::curdepth.'" '.&element_change_detection().
         $textareaclass.'>'.
 	&HTML::Entities::encode($data,'<>&"').'</textarea>'.
-	($usehtmlarea?&Apache::lonhtmlcommon::spelllink('lonhomework',
-			 'homework_edit_'.$Apache::lonxml::curdepth):'')."\n";
+        ($usehtmlarea?'<br />'.&Apache::lonhtmlcommon::spelllink('lonhomework',
+                                   'homework_edit_'.$Apache::lonxml::curdepth):'')."\n";
 }
 
 sub modifiedfield {
@@ -891,7 +984,7 @@ sub hidden_arg {
 }
 
 sub checked_arg {
-    my ($description,$name,$list,$token) = @_;
+    my ($description,$name,$list,$token,$onclick,$useid) = @_;
     my $result;
     my $optionlist="";
     my $allselected=$token->[2]{$name};
@@ -900,13 +993,13 @@ sub checked_arg {
 	my ($value,$text);
 	if ( ref($option) eq 'ARRAY') {
 	    $value='value="'.$$option[0].'"';
-	    $text=$$option[1];
+	    $text=&mt($$option[1]);
 	    $option=$$option[0];
 	} else {
-	    $text=$option;
+	    $text=&mt($option);
 	    $value='value="'.$option.'"';
 	}
-	$result.='<span class="LC_nobreak"><label><input type="checkbox" '.$value.' name="'.
+        $result.=' <span class="LC_edit_opt"><label><input type="checkbox" '.$value.' name="'.
 	    &html_element_name($name).'"';
 	foreach my $selected (split(/,/,$allselected)) {
 	    if ( $selected eq $option ) {
@@ -914,20 +1007,26 @@ sub checked_arg {
 		last;
 	    }
 	}
-	$result.=&element_change_detection().' />'.$text.'</label></span>'."\n";
+        if ($useid) {
+            $result .= ' id="'.&html_element_name($name).'" ';
+        }
+	$result.=&element_change_detection().$onclick.' />'.$text.'</label></span>'."\n";
     }
     return $result;
 }
 
 sub text_arg {
-    my ($description,$name,$token,$size) = @_;
+    my ($description,$name,$token,$size, $class) = @_;
     my $result;
     if (!defined $size) { $size=20; }
     my $arg=$token->[2]{$name};
     $result=&mt($description).'&nbsp;<input name="'.&html_element_name($name).
-	'" type="text" value="'.$arg.'" size="'.$size.'" '.
-	&element_change_detection().'/>';
-    return '<span class="LC_nobreak">'.$result.'</span>';
+	'" type="text" value="'.$arg.'" size="'.$size.'" ';
+    if (defined $class) {
+	$result .= 'class="' . $class . '" ';
+    }
+    $result .=	&element_change_detection().'/>';
+    return ' <span class="LC_edit_opt">'.$result.'</span>';
 }
 
 sub select_arg {
@@ -953,7 +1052,7 @@ sub select_arg {
 	    }
         }
     }
-    $result.='<span class="LC_nobreak">'.&mt($description).'&nbsp;<select name="'.
+    $result.=' <span class="LC_edit_opt">'.&mt($description).'&nbsp;<select name="'.
 	&html_element_name($name).'" '.&element_change_detection().' >
        '.$optionlist.'
       </select></span>';
@@ -1004,16 +1103,16 @@ sub select_or_text_arg {
     $description=&mt($description);
 #
     return (<<ENDSELECTORTYPE);
-<span class="LC_nobreak">
+ <span class="LC_edit_opt">
 $description
 &nbsp;<select name="$selectelement"
-onChange="if ($selectedvalue!='TYPEDINVALUE') { $hiddenvalue=$selectedvalue; $typedinvalue=''; }" >
+onchange="if ($selectedvalue!='TYPEDINVALUE') { $hiddenvalue=$selectedvalue; $typedinvalue=''; }" >
 $optionlist
 </select>
 <input type="text" size="$size" name="$typeinelement"
        value="$typeinvalue" 
-onChange="$hiddenvalue=$typedinvalue;"
-onFocus="$selectedindex=$selecttypeinindex-1;" />
+onchange="$hiddenvalue=$typedinvalue;"
+onfocus="$selectedindex=$selecttypeinindex-1;" />
 <input type="hidden" name="$element" value="$selected" $change_code />
 </span>
 ENDSELECTORTYPE
