@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # input  definitons
 #
-# $Id: inputtags.pm,v 1.324 2014/06/19 19:37:35 raeburn Exp $
+# $Id: inputtags.pm,v 1.333.2.2 2015/04/21 22:52:44 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -235,6 +235,7 @@ sub start_textfield {
 	    my $addchars=&Apache::lonxml::get_param('addchars',$parstack,$safeeval);
 	    $result='';
 	    my $tagident = 'HWVAL_' . $resid;
+            my $itemid = 'HWVAL_'.$partid.'_'.$resid;
 	    if ($addchars) {
 		$result.=&addchars($tagident, $addchars);
 	    }
@@ -243,7 +244,7 @@ sub start_textfield {
                                                     'externalresponse')) {
                 $textareaclass = 'class="LC_richDetectHtml spellchecked"';
             }
-	    $result.= '<textarea wrap="hard" name="'.$tagident.'" id="'.$tagident.'" ' .
+	    $result.= '<textarea wrap="hard" name="'.$tagident.'" id="'.$itemid.'" ' .
 		      'rows="'.$rows.'" cols="'.$cols.'" '.$textareaclass
 		      .'>'.
                       &HTML::Entities::encode($oldresponse,'<>&"');
@@ -325,10 +326,11 @@ sub end_textfield {
 	my $spellcheck = &Apache::lonxml::get_param('spellcheck', $parstack, $safeeval);
 	$Apache::lonxml::evaluate++;
 	if ($Apache::inputtags::status[-1] eq 'CAN_ANSWER') {
+            my $partid=$Apache::inputtags::part;
 	    my $resid = $Apache::inputtags::response[-1];
-	    my $tagident = 'HWVAL_' . $resid;
+	    my $itemid = 'HWVAL_' . $partid . '_' . $resid;
 	    my $result =  "</textarea>";
-	    $result .= &spellcheck_onblur($tagident, $spellcheck);
+	    $result .= &spellcheck_onblur($itemid, $spellcheck);
 	    return $result;
 	}
     } elsif ($target eq 'edit') {
@@ -457,18 +459,18 @@ sub start_textline {
 		    $readonly='';
 		}
 		my $name = 'HWVAL_'.$id;
+                my $itemid = 'HWVAL_'.$partid.'_'.$id;
 		if ($Apache::inputtags::status[-1] eq 'CANNOT_ANSWER') {
 		    $name = "none";
 		}
 		$result.= '<input onkeydown="javascript:setSubmittedPart(\''.$partid.'\');"'
-		     . ' type="text" '
-		     . $readonly.' name="'. $name . '"'
-		     . ' id="' . $name . '"'
+		     . ' onfocus="javascript:disableAutoComplete(\''.$itemid.'\');"'
+		     . ' type="text" '.$readonly.' name="'. $name . '"'
+		     . ' id="' . $itemid . '"'
 		     . ' value="'.  $oldresponse.'"'
-		     . ' class="spellchecked"  size="'.$size.'"'.$maxlength
-		     . '/>';
+		     . ' class="LC_textline spellchecked"  size="'.$size.'"'.$maxlength.' />';
 
-		$result .= &spellcheck_onblur($name, $spellcheck);
+		$result .= &spellcheck_onblur($itemid, $spellcheck);
 	    }
 	    if ($Apache::lonhomework::type eq 'exam'
 		&& &needs_exam_box($tagstack)) {
@@ -875,7 +877,7 @@ END
 sub valid_award {
     my ($award) =@_;
     foreach my $possibleaward ('EXTRA_ANSWER','MISSING_ANSWER', 'ERROR',
-			       'NO_RESPONSE',
+			       'NO_RESPONSE','WRONG_NUMBOXESCHECKED',
 			       'TOO_LONG', 'UNIT_INVALID_INSTRUCTOR',
 			       'UNIT_INVALID_STUDENT', 'UNIT_IRRECONCIBLE',
 			       'UNIT_FAIL', 'NO_UNIT',
@@ -895,7 +897,7 @@ sub valid_award {
 
 {
     my @awards = ('EXTRA_ANSWER', 'MISSING_ANSWER', 'ERROR', 'NO_RESPONSE',
-		  'TOO_LONG',
+		  'WRONG_NUMBOXESCHECKED','TOO_LONG',
 		  'UNIT_INVALID_INSTRUCTOR', 'UNIT_INVALID_STUDENT',
 		  'UNIT_IRRECONCIBLE', 'UNIT_FAIL', 'NO_UNIT',
 		  'UNIT_NOTNEEDED', 'WANTED_NUMERIC', 'BAD_FORMULA',  'NOT_FUNCTION', 
@@ -985,6 +987,23 @@ sub finalizeawards {
 	$j++;
     }
 
+    # if at least one response item is set to include lenient grading
+    # and that item is partially correct then overall award reflects
+    # that, unless an award for one of the other response items does
+    # not fall within the basic awards for correct or incorrect.
+    if ($Apache::inputtags::leniency) {
+        if (($$awardref[$which] eq 'INCORRECT')
+            && (grep { $_ eq 'EXACT_ANS' ||
+                       $_ eq 'APPROX_ANS' ||
+                       $_ eq 'ASSIGNED_SCORE' } (@$awardref))
+            && !((grep { $_ ne 'INCORRECT' &&
+                         $_ ne 'EXACT_ANS' &&
+                         $_ ne 'APPROX_ANS' &&
+                         $_ ne 'ASSIGNED_SCORE' } (@$awardref)))) {
+            return ('ASSIGNED_SCORE');
+        }
+    }
+
     if (defined($which)) {
 	if (ref($nameref)) {
 	    return ($$awardref[$which],$$msgref[$which],$$nameref[$which]);
@@ -997,7 +1016,7 @@ sub finalizeawards {
 }
 
 sub decideoutput {
-    my ($award,$awarded,$awardmsg,$solved,$previous,$target,$nocorrect)=@_;
+    my ($award,$awarded,$awardmsg,$solved,$previous,$target,$nocorrect,$tdclass)=@_;
 
     my $message='';
     my $button=0;
@@ -1122,6 +1141,13 @@ sub decideoutput {
         }
 	$css_class=$possible_class{'not_charged_try'};
 	$button = 1;
+    } elsif ($award eq 'WRONG_NUMBOXESCHECKED') {
+        $message = &mt('Number of boxes checked outside permissible range (either too few or too many).');
+        if ($target ne 'tex') {
+           $message .= &Apache::loncommon::help_open_topic('Wrong_Num_Boxes_Checked');
+        }
+        $css_class=$possible_class{'not_charged_try'};
+        $button = 1;
     } elsif ($award eq 'ERROR') {
 	$message = &mt('An error occurred while grading your answer.');
 	$css_class=$possible_class{'not_charged_try'};
@@ -1493,7 +1519,8 @@ sub setgradedata {
     $Apache::lonhomework::results{"resource.$id.type"} = $Apache::lonhomework::type;
     $Apache::lonhomework::results{"resource.$id.duedate"} = &Apache::lonnet::EXT("resource.$id.duedate");
     $Apache::lonhomework::results{"resource.$id.hinttries"} = &Apache::lonnet::EXT("resource.$id.hinttries");
-    $Apache::lonhomework::results{"resourse.$id.version"} = &Apache::lonnet::usedversion(); 
+    $Apache::lonhomework::results{"resource.$id.version"} = &Apache::lonnet::usedversion();
+    $Apache::lonhomework::results{"resource.$id.maxtries"} = &Apache::lonnet::EXT("resource.$id.maxtries");
 }
 
 sub find_which_previous {
@@ -1573,7 +1600,7 @@ sub grade {
 }
 
 sub get_grade_messages {
-    my ($id,$prefix,$target,$status,$nocorrect) = @_;
+    my ($id,$prefix,$target,$status,$nocorrect,$tdclass) = @_;
 # nocorrect suppresses "Computer's answer now shown above"
     my ($message,$latemessage,$trystr,$previousmsg);
     my $showbutton = 1;
@@ -1588,13 +1615,13 @@ sub get_grade_messages {
 	&Apache::lonxml::debug('Getting message');
 	($showbutton,my $css_class,$message,$previousmsg) =
 	    &decideoutput($award,$awarded,$awardmsg,$solved,$previous,
-			  $target,(($status eq 'CAN_ANSWER') || $nocorrect));
+			  $target,(($status eq 'CAN_ANSWER') || $nocorrect),$tdclass);
 	if ($target eq 'tex') {
 	    $message='\vskip 2 mm '.$message.' ';
 	} else {
-	    $message="<td class=\"$css_class\">$message</td>";
+	    $message="<td class=\"$tdclass $css_class\">$message</td>";
 	    if ($previousmsg) {
-		$previousmsg="<td class=\"LC_answer_previous\">$previousmsg</td>";
+		$previousmsg="<td class=\"$tdclass LC_answer_previous\">$previousmsg</td>";
 	    }
 	}
     }
@@ -1629,7 +1656,7 @@ sub get_grade_messages {
 		    $trial.="/".$Apache::inputtags::params{'maxtries'};
 		}
 	    }
-	    $trystr = '<td><span class="LC_nobreak">'.&mt($tries_text.' [_1]',$trial).'</span></td>';
+	    $trystr = '<td class="'.$tdclass.'"><span class="LC_nobreak">'.&mt($tries_text.' [_1]',$trial).'</span></td>';
 	}
     }
 
@@ -1637,7 +1664,7 @@ sub get_grade_messages {
 	#last submissions was after due date
 	$latemessage=&mt(' The last submission was after the Due Date ');;
 	if ($target eq 'web') {
-	    $latemessage='<td class="LC_answer_late">'.$latemessage.'</td>';
+	    $latemessage='<td class="'.$tdclass.' LC_answer_late">'.$latemessage.'</td>';
 	}
     }
     return ($previousmsg,$latemessage,$message,$trystr,$showbutton);
@@ -1651,6 +1678,7 @@ sub gradestatus {
     my $trystr='';
     my $button='';
     my $previousmsg='';
+    my $tdclass='';
 
     my $status = $Apache::inputtags::status['-1'];
     &Apache::lonxml::debug("gradestatus has :$status:");
@@ -1667,21 +1695,29 @@ sub gradestatus {
             $showbutton = 0;
         }
 
+        unless (($status eq 'SHOW_ANSWER') || ($status eq 'CANNOT_ANSWER')) {
+            if ($target ne 'tex') {
+                $tdclass = 'LC_status_submit_'.$id;
+            }
+        }
+
 	($previousmsg,$latemessage,$message,$trystr) =
 	    &get_grade_messages($id,"resource.$id",$target,$status,
-				$showbutton);
+				$showbutton,$tdclass);
 	if ($status eq 'CANNOT_ANSWER') {
 	    $showbutton = 0;
 	}
 	if ( $status eq 'SHOW_ANSWER') {
 	    undef($previousmsg);
 	}
-	if ( $showbutton ) { 
+	if ( $showbutton ) {
 	    if ($target ne 'tex') {
 		$button = 
             '<input onmouseup="javascript:setSubmittedPart(\''.$id.'\');this.form.action+=\'#'.&escape($id).'\';"
-                    type="submit" name="submit_'.$id.'"
-                    value="'.&mt('Submit Answer').'" />';
+                    type="submit" name="submit_'.$id.'" id="submit_'.$id.'" class="LC_hwk_submit"
+                    value="'.&mt('Submit Answer').'" />&nbsp;'.
+                    '<div id="msg_submit_'.$id.'" style="display:none">'.
+                    &mt('Processing your submission ...').'</div>';
 	    }
 	}
 
@@ -1696,7 +1732,7 @@ sub gradestatus {
 	    $output =
 		'<table><tr><td>'.$button.'</td>'.$output;
 	    if (!$no_previous) {
-		$output.='<td>'.&previous_tries($id,$target).'</td>';
+		$output.='<td class="'.$tdclass.'">'.&previous_tries($id,$target).'</td>';
 	    }
 	    $output.= '</tr></table>';
 	    return $output;
@@ -1711,15 +1747,15 @@ sub previous_tries {
 
     my $count;
     my %count_lookup;
-    my $lastrndseed;
+    my ($lastrndseed,$lasttype);
     my $numstamps = 0;
 
     foreach my $i (1..$Apache::lonhomework::history{'version'}) {
 	my $prefix = $i.":resource.$id";
-        my $is_anon; 
+        my $is_anon;
+        my $curr_type = $Apache::lonhomework::history{"$prefix.type"};    
         if (defined($env{'form.grade_symb'})) {
-            if (($Apache::lonhomework::history{"$prefix.type"} eq 'anonsurvey') || 
-                ($Apache::lonhomework::history{"$prefix.type"} eq 'anonsurveycred')) {
+            if (($curr_type eq 'anonsurvey') || ($curr_type eq 'anonsurveycred')) {
                 $is_anon = 1;
             }
         }
@@ -1758,8 +1794,9 @@ sub previous_tries {
                              {$1 <strong>$txt_correct</strong>. $3}s;
 	    }
             my $trystr = "(".&mt('Try [_1]',$Apache::lonhomework::history{"$prefix.tries"}).")";
-            if (($curr_rndseed || $lastrndseed) && ($i > 1)) {
-                if ($curr_rndseed ne $lastrndseed) {
+            if (($curr_rndseed ne '') &&  ($lastrndseed ne '')) {
+                if (($curr_rndseed ne $lastrndseed) && 
+                    (($curr_type eq 'randomizetry') || ($lasttype eq 'randomizetry'))) {
                     $trystr .= '<br /><span style="color: green; white-space: nowrap; font-style: italic; font-weight: bold; font-size: 80%;">'.&mt('New problem variation this try.').'</span>';
                 }
             } 
@@ -1800,6 +1837,7 @@ sub previous_tries {
 	}
 	$output.=&Apache::loncommon::end_data_table_row()."\n";
         $lastrndseed = $curr_rndseed;
+        $lasttype = $curr_type;
     }
     return if ($output eq '');
     my $headers = '<tr>'.
