@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # The LON-CAPA Homework handler
 #
-# $Id: lonhomework.pm,v 1.344 2014/02/25 19:31:48 raeburn Exp $
+# $Id: lonhomework.pm,v 1.344.2.2 2015/04/20 01:22:58 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -469,7 +469,6 @@ sub check_access {
 	if ( $tries eq '' ) { $tries = '0'; }
 	if ( $maxtries eq '' && 
 	     $env{'request.state'} ne 'construct') { $maxtries = '2'; } 
-	$Apache::lonhomework::results{'resource.'.$id.'.maxtries'}=$maxtries;
 	if ($maxtries && $tries >= $maxtries) { $status = 'CANNOT_ANSWER'; }
 	# if (correct and show prob status) or excused then CANNOT_ANSWER
 	if ( ($Apache::lonhomework::history{"resource.$id.solved"}=~/^correct/)
@@ -541,7 +540,7 @@ sub due_date {
     } else {
 	$date = $due_date;
     }
-    return $date
+    return $date;
 }
 
 sub seconds_to_human_length {
@@ -590,7 +589,7 @@ sub showarray {
 sub showhashsubset {
     my ($hash,$keyre) = @_;
     my $resultkey;
-    foreach $resultkey (sort keys %$hash) {
+    foreach $resultkey (sort(keys(%$hash))) {
 	if ($resultkey !~ /$keyre/) { next; }
 	if (ref($$hash{$resultkey})  eq 'ARRAY' ) {
 	    &Apache::lonxml::debug("$resultkey ---- ".
@@ -1278,7 +1277,7 @@ sub handler {
     my $file=&Apache::lonnet::filelocation("",$request->uri);
 
     #check if we know where we are
-    if ($env{'request.course.fn'} && !&Apache::lonnet::symbread()) { 
+    if ($env{'request.course.fn'} && !&Apache::lonnet::symbread('','',1,1)) { 
 	# if we are browsing we might not be able to know where we are
 	if ($Apache::lonhomework::browse ne 'F' && 
 	    $env{'request.state'} ne "construct") {
