@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Simple Problem Parameter Setting "Editor"
 #
-# $Id: lonsimpleproblemedit.pm,v 1.36 2013/07/15 16:13:12 bisitz Exp $
+# $Id: lonsimpleproblemedit.pm,v 1.37 2014/12/15 00:50:20 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -218,11 +218,11 @@ sub handler {
 		$env{'form.newopt'}=~s/\'/\\\'/g;
                 $curoptions{$env{'form.newopt'}}=$env{'form.newopt'};
 	    }
-            $env{'form.options'}="('".join("','",keys %curoptions)."')";
+            $env{'form.options'}="('".join("','",keys(%curoptions))."')";
 	}
 	$env{'form.hiddenparts'}='!'.$env{'form.questiontype'};
-        foreach (keys %env) {
-	    if ($_=~/^form\.(\w+)$/) {
+        foreach my $envkey (keys(%env)) {
+	    if ($envkey=~/^form\.(\w+)$/) {
                 my $parm=$1;
 		$storecontent{$prefix.$parm}=$env{'form.'.$parm};
                 $storecontent{$prefix.$parm}=~s/^\s+//s;
