@@ -1,7 +1,7 @@
 # LearningOnline Network with CAPA
 # option list style responses
 #
-# $Id: optionresponse.pm,v 1.173.2.2 2011/11/08 19:38:05 raeburn Exp $
+# $Id: optionresponse.pm,v 1.195.2.1 2015/04/21 23:36:43 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -64,7 +64,7 @@ sub start_optionresponse {
 				     $token,'4').
         &Apache::edit::select_arg('Randomize Foil Order:','randomize',
 				  ['yes','no'],$token).
-        &Apache::edit::select_arg(&mt('Display of options when printed'),'TeXlayout',
+        &Apache::edit::select_arg('Display of options when printed','TeXlayout',
 				  [['horizontal','Normal list'],
 				   ['vertical','Listed in a vertical column']],$token).
         &Apache::edit::end_row().&Apache::edit::start_spanning_row();
@@ -108,7 +108,11 @@ sub start_foilgroup {
     my $optionlist="<option></option>\n";
     my $option;
     my @opt;
-    eval '@opt ='. &Apache::lonxml::get_param('options',$parstack,$safeeval);
+    my @raw_options = &Apache::lonxml::get_param('options', $parstack, $safeeval, 0,  0, 1);
+
+
+    eval '@opt ='. &Apache::lonxml::get_param('options',$parstack,$safeeval, 0, 0, 1);
+
     my $count=1;
     foreach $option (@opt) {
       $optionlist.="<option value=\"$count\">$option</option>\n";
@@ -116,17 +120,24 @@ sub start_foilgroup {
     }
     my $insertlist=&Apache::edit::insertlist($target,$token);
     $result.=&Apache::edit::start_table($token);
+    my %lt = &Apache::lonlocal::texthash(
+        sel => 'Select Options',
+        add => 'Add new Option:',
+        del => 'Delete an Option:',
+        maxcheck => 'Enter maximum number students may check (e.g., 3)',
+        mincheck => 'Enter minimum number students need to check (e.g., 1)',
+    );
     $result.= (<<ENDTABLE);
-      <tr><td>Select Options</td>
+      <tr><td>$lt{'sel'}</td>
         <td>
-	  Add new Option: <input type="text" name="$Apache::lonxml::curdepth.options" />
+	  $lt{'add'} <input type="text" name="$Apache::lonxml::curdepth.options" />
         </td>
-        <td>Delete an Option:
+        <td>$lt{'del'}
 	  <select name="$Apache::lonxml::curdepth.deleteopt">$optionlist</select>
         </td>
 ENDTABLE
     $result.= '<td>'.&Apache::edit::checked_arg('Print options:','texoptions',
-					 [ ['nochoice','Don\'t show option list'] ]
+					 [ ['nochoice',"Don't show option list"] ]
 					 ,$token).'</td>';
     $result.= '<td><label>'.&mt('Two-option checkboxes for:').
               '<select name="checkboxvalue_'.$Apache::lonxml::curdepth.'">';
@@ -137,10 +148,38 @@ ENDTABLE
        }
        $result.='>'.$option.'</option>';
     }
-    $result.='</select></label>';
+    $result.='</select></label> ';
+    my %checknumtxt = (
+                        maxcheck => 'Max. boxes permitted to check',
+                        mincheck => 'Min. boxes required to check',
+    );
+    my (%onclicknum,%checknum);
+    foreach my $item ('maxcheck','mincheck') {
+        $onclicknum{$item} = 'onclick="'."updateNumber('$item','$Apache::lonxml::curdepth','check','$lt{$item}');".'"';
+        $checknum{$item} = &Apache::lonxml::get_param($item,$parstack,$safeeval);
+    }
     $result.=&Apache::edit::checked_arg('Checkbox options:','checkboxoptions',
-                                         [ ['nochoice','Don\'t show option list'] ]
-                                         ,$token).'</td>';
+                                         [ ['nochoice',"Don't show option list"] ]
+                                         ,$token).' '.
+             &Apache::edit::checked_arg('','noprompt',
+                                         [ ['nochoice','Omit "Select all that are ... "' ] ]
+                                         ,$token);
+    foreach my $item ('maxcheck','mincheck') {
+        my $currnum = $checknum{$item};
+        unless ($currnum =~ /^\d+$/) {
+            $currnum = 'nochoice';
+        } 
+        $result.= ' <span class="LC_nobreak">'.
+                  &Apache::edit::checked_arg('',$item,[ [$currnum,$checknumtxt{$item}] ],
+                                        $token,$onclicknum{$item},1).' '.
+                  '<span id="'.$item.'_numtext_'.$Apache::lonxml::curdepth.'">';
+        if ($checknum{$item} ne '') {
+            $result .= ':&nbsp;<a href="javascript:updateNumber('."'$item','$Apache::lonxml::curdepth','link','$lt{$item}'".')">'.
+                       $checknum{$item}.'</a>';
+        }
+        $result .= '</span></span>';
+    } 
+    $result.= '</td>';
     $result.= &Apache::edit::end_row();
     $result.= &Apache::edit::start_spanning_row();
     $result.= $insertlist.'<br />';
@@ -148,7 +187,10 @@ ENDTABLE
   if ($target eq 'modified') {
     my @options;
     my $optchanged=0;
-    eval '@options ='.&Apache::lonxml::get_param('options',$parstack,$safeeval);
+
+
+    eval '@options ='.&Apache::lonxml::get_param('options',$parstack,$safeeval, 0, 0, 1);
+
     if ($env{"form.$Apache::lonxml::curdepth.deleteopt"}) {
       my $delopt=$env{"form.$Apache::lonxml::curdepth.deleteopt"};
       &Apache::lonxml::debug("Deleting :$delopt:");
@@ -161,7 +203,23 @@ ENDTABLE
       $optchanged=1;
     }
     my $rebuildtag = &Apache::edit::get_new_args($token,$parstack,$safeeval,
-						 'texoptions','checkboxvalue','checkboxoptions');
+						 'texoptions','checkboxvalue','checkboxoptions',
+                                                 'noprompt','maxcheck','mincheck');
+    if ($rebuildtag) {
+        my $maxcheck = $token->[2]->{'maxcheck'};
+        my $mincheck = $token->[2]->{'mincheck'};
+        if ($mincheck =~ /D/) {
+            $token->[2]->{'mincheck'}='';
+        }
+        if ($maxcheck =~ /D/) {
+            $token->[2]->{'maxcheck'}='';
+        }
+        if (($mincheck ne '') && ($maxcheck ne '')) {
+            if ($mincheck > $maxcheck) {
+                $token->[2]->{'mincheck'}=$maxcheck; 
+            }
+        }
+    }
     if ($optchanged || $rebuildtag ) {
       $result = "<foilgroup options=\"(";
       foreach my $option (@options) {
@@ -173,7 +231,10 @@ ENDTABLE
       $result.=')" ';
       $result .= 'texoptions="'.$token->[2]{'texoptions'}.'" ';
       $result .= 'checkboxoptions="'.$token->[2]{'checkboxoptions'}.'" ';
-      $result .= 'checkboxvalue="'.$token->[2]{'checkboxvalue'}.'"';
+      $result .= 'checkboxvalue="'.$token->[2]{'checkboxvalue'}.'" ';
+      $result .= 'noprompt="'.$token->[2]{'noprompt'}.'" ';
+      $result .= 'maxcheck="'.$token->[2]{'maxcheck'}.'" ';
+      $result .= 'mincheck="'.$token->[2]{'mincheck'}.'"';
       $result .= '>';
     } # else nothing changed so just use the default mechanism
   }
@@ -203,13 +264,19 @@ sub end_foilgroup {
 					     -2,0);
     my $checkboxvalue=&Apache::lonxml::get_param('checkboxvalue',$parstack,$safeeval);
     my $checkboxchoices=(&Apache::lonxml::get_param('checkboxoptions',$parstack,$safeeval) ne 'nochoice');
+    my $maxcheck=&Apache::lonxml::get_param('maxcheck', $parstack, $safeeval);
+    $maxcheck =~ s/\D+//g;
+    my $mincheck=&Apache::lonxml::get_param('mincheck', $parstack, $safeeval);
+    $mincheck =~ s/\D+//g;
+    my $noprompt=&Apache::lonxml::get_param('noprompt', $parstack, $safeeval);
+
     if ($target eq 'tex' && $tex_option_switch eq 'nochoice') {@opt=();}
     &Apache::lonxml::debug("Options are $#opt");
     my $max = &Apache::lonxml::get_param('max',$parstack,$safeeval,'-2');
     my $randomize = &Apache::lonxml::get_param('randomize',$parstack,
 					       $safeeval,'-2');
     if ($target eq 'web' || $target eq 'tex') {
-      $result.=&displayfoils($target,$max,$randomize,$TeXlayout,$checkboxvalue,$checkboxchoices,$tex_option_switch,@opt);
+      $result.=&displayfoils($target,$max,$randomize,$TeXlayout,$checkboxvalue,$checkboxchoices,$tex_option_switch, $noprompt,$maxcheck,$mincheck, @opt);
       $Apache::lonxml::post_evaluate=0;
     } elsif ( $target eq 'answer') {
       $result.=&displayanswers($max,$randomize,@opt);
@@ -221,7 +288,9 @@ sub end_foilgroup {
 	my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
 	push (@{ $Apache::lonhomework::analyze{"$part_id.options"} },@opt);
     } elsif ( $target eq 'grade') {
+
       if ( &Apache::response::submitted()) {
+
 	my @whichopt = &whichfoils($max,$randomize);
 	my $temp=1;my $name;
 	my %responsehash;
@@ -229,11 +298,64 @@ sub end_foilgroup {
 	my $right=0;
 	my $wrong=0;
 	my $ignored=0;
+        my $checked=0;
+        my ($numrows,$bubbles_per_row);
+        if ($Apache::lonhomework::scantronmode) {
+            my $numitems = scalar(@opt);
+            ($numrows,$bubbles_per_row) =
+                &Apache::optionresponse::getnumrows($numitems);
+        }
+        if ($numrows < 1) {
+            $numrows = 1;
+        }
+        my $checkboxopt;
+        if ($checkboxvalue) {
+            $checkboxopt = &check_box_opt($target,$checkboxvalue,@opt);
+            if (($checkboxopt) && ($Apache::lonhomework::type !~ /survey/)) {
+                my $corrnumcheck = 0;
+                if (($maxcheck ne '') || ($mincheck ne '')) {
+                    foreach $name (@whichopt) {
+                        if ($Apache::response::foilgroup{$name.'.value'} eq $checkboxopt) {
+                            $corrnumcheck ++; 
+                        }
+                    }
+                    if (($maxcheck ne '') && ($corrnumcheck > $maxcheck)) {
+                        $maxcheck = $corrnumcheck;
+                    }
+                    if (($mincheck ne '') && ($corrnumcheck < $mincheck)) {
+                        $mincheck = $corrnumcheck;
+                    }
+                }
+            }
+        }
 	foreach $name (@whichopt) {
-	  my $response=&Apache::response::getresponse($temp);
-	  if ($env{'form.submitted'} eq 'scantron' && $response=~/\S/) {
-	      $response = $opt[$response];
-	  }
+	  my $response;
+          if ($env{'form.submitted'} eq 'scantron') {
+              if ($numrows > 1) {
+                  my $num = $temp;
+                  my $totalnum;
+                  for (my $i=0; $i<$numrows; $i++) {
+                      my $item = &Apache::response::getresponse($num);
+                      if ($item =~ /^\d+$/) {
+                          $totalnum = $i*$bubbles_per_row + $item;
+                      }
+                      $num ++;
+                  }
+                  if ($totalnum =~ /^\d+$/) {
+                      $response = $opt[$totalnum];
+                  }
+                  $temp += $numrows;
+              } else {
+                  $response=&Apache::response::getresponse($temp);
+                  if ($response=~/\S/) {
+                      $response = $opt[$response];
+                  }
+                  $temp ++;
+              }
+	  } else {
+              $response=&Apache::response::getresponse($temp);
+              $temp ++;
+          }
 	  if ( $response =~ /[^\s]/) {
 	    $responsehash{$name}=$response;
 	    my $value=$Apache::response::foilgroup{$name.'.value'};
@@ -243,13 +365,18 @@ sub end_foilgroup {
 	    } else {
 		$grade{$name}='0'; $wrong++;
 	    }
+            if ($checkboxopt) {
+                if ($response eq $checkboxopt) {
+                    $checked++;
+                }
+            } 
 	  } else {
 	    $ignored++;
 	  }
-	  $temp++;
 	}
 	my $part=$Apache::inputtags::part;
-	my $id = $Apache::inputtags::response['-1'];
+        my $id = $Apache::inputtags::response['-1'];
+        my $nonlenient=&grading_is_nonlenient($part,$id);
 	my $responsestr=&Apache::lonnet::hash2str(%responsehash);
 	my $gradestr   =&Apache::lonnet::hash2str(%grade);
 	my %previous=&Apache::response::check_for_previous($responsestr,
@@ -261,65 +388,90 @@ sub end_foilgroup {
         if ($Apache::lonhomework::type eq 'randomizetry') {
             $Apache::lonhomework::results{"resource.$part.$id.foilorder"} = &Apache::lonnet::array2str(@whichopt);
         }
-	if (($Apache::lonhomework::type eq 'survey') ||
-            ($Apache::lonhomework::type eq 'surveycred') ||
-            ($Apache::lonhomework::type eq 'anonsurvey') ||
-            ($Apache::lonhomework::type eq 'anonsurveycred')) {
-	    if ($ignored == 0) {
-                my $ad;
-                if ($Apache::lonhomework::type eq 'anonsurveycred') {
-                    $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='ANONYMOUS_CREDIT';
-                } elsif ($Apache::lonhomework::type eq 'anonsurvey') {
-                    $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='ANONYMOUS';
-                } elsif ($Apache::lonhomework::type eq 'surveycred') {
-                    $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='SUBMITTED_CREDIT';
-                } else {
-		    $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='SUBMITTED';
-                }
-		&Apache::response::handle_previous(\%previous,$ad);
-	    } elsif ($wrong==0 && $right==0) {
-	    } else {
-		my $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='MISSING_ANSWER';
-		&Apache::response::handle_previous(\%previous,$ad);
-	    }
-	} elsif (!$Apache::lonhomework::scantronmode) {
-	    my $ad;
-	    if ($wrong==0 && $ignored==0) {
-		$ad='EXACT_ANS';
-	    } elsif ($wrong==0 && $right==0) {
-		#nothing submitted
-	    } else {
-		if ($ignored==0) {
-		    $ad='INCORRECT';
-		} else {
-		    $ad='MISSING_ANSWER';
-		}
-	    }
-	    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
-	    &Apache::response::handle_previous(\%previous,$ad);
-	} else {
-	    my $ad;
-	    if ($wrong==0 && $right==0) {
-		#nothing submitted only assign a score if we 
-		#need to override a previous grade
-		if (defined($Apache::lonhomework::history{"resource.$part.$id.awarddetail"})) {
+        my $skipaward;
+        if ($checkboxvalue) {
+            if ((($maxcheck ne '') && ($checked > $maxcheck)) ||
+                (($mincheck ne '') && ($checked < $mincheck))) {
+                my $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='WRONG_NUMBOXESCHECKED';
+                &Apache::response::handle_previous(\%previous,$ad);
+                $skipaward = 1;
+            }
+        }
+        unless ($skipaward) {
+	    if (($Apache::lonhomework::type eq 'survey') ||
+                ($Apache::lonhomework::type eq 'surveycred') ||
+                ($Apache::lonhomework::type eq 'anonsurvey') ||
+                ($Apache::lonhomework::type eq 'anonsurveycred')) {
+	        if ($ignored == 0) {
+                    my $ad;
+                    if ($Apache::lonhomework::type eq 'anonsurveycred') {
+                        $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='ANONYMOUS_CREDIT';
+                    } elsif ($Apache::lonhomework::type eq 'anonsurvey') {
+                        $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='ANONYMOUS';
+                    } elsif ($Apache::lonhomework::type eq 'surveycred') {
+                        $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='SUBMITTED_CREDIT';
+                    } else {
+		        $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='SUBMITTED';
+                    }
+		    &Apache::response::handle_previous(\%previous,$ad);
+	        } elsif ($wrong==0 && $right==0) {
+	        } else {
+		    my $ad=$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}='MISSING_ANSWER';
+		    &Apache::response::handle_previous(\%previous,$ad);
+	        }
+	    } elsif ($nonlenient) {
+#
+# Non-lenient mode. All right or all wrong
+#
+	        my $ad;
+	        if ($wrong==0 && $ignored==0) {
+		    $ad='EXACT_ANS';
+	        } elsif ($wrong==0 && $right==0) {
+		    #nothing submitted
+	        } else {
+		    if ($ignored==0) {
+		        $ad='INCORRECT';
+		    } else {
+		        $ad='MISSING_ANSWER';
+		    }
+	        }
+	        $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
+	        &Apache::response::handle_previous(\%previous,$ad);
+	     } else {
+#
+# This is lenient mode
+#
+	        my $ad;
+	        if ($wrong==0 && $right==0) {
+		    #nothing submitted only assign a score if we 
+		    #need to override a previous grade
+		    if (defined($Apache::lonhomework::history{"resource.$part.$id.awarddetail"})) {
+		        $ad='ASSIGNED_SCORE';
+		    }
+	        } else {
 		    $ad='ASSIGNED_SCORE';
-		}
-	    } else {
-		$ad='ASSIGNED_SCORE';
-	    }
-	    $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
-	    $Apache::lonhomework::results{"resource.$part.$id.awarded"}=
-		$right/(scalar(@whichopt));
-	    $Apache::lonhomework::results{"resource.$part.$id.numfoils"}=
-		scalar(@whichopt);
-	}
+	        }
+	        $Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=$ad;
+	        $Apache::lonhomework::results{"resource.$part.$id.awarded"}=
+		    $right/(scalar(@whichopt));
+	        $Apache::lonhomework::results{"resource.$part.$id.numfoils"}=
+		    scalar(@whichopt);
+	     }
+       }
       }
     }
     my $part_id     = $Apache::inputtags::part;
     my $response_id = $Apache::inputtags::response[-1];
-    &Apache::lonxml::increment_counter(&getfoilcounts($max),
-				       "$part_id.$response_id");
+    my ($numrows,$bubbles_per_row);
+    if (($target eq 'tex') && ($Apache::lonhomework::type eq 'exam')) {
+        ($numrows,$bubbles_per_row) =
+            &Apache::optionresponse::getnumrows(scalar(@opt));
+    }
+    if ($numrows < 1) {
+        $numrows = 1;
+    }
+    my $increment = &getfoilcounts($max) * $numrows;
+    &Apache::lonxml::increment_counter($increment,"$part_id.$response_id");
     if ($target eq 'analyze') {
 	&Apache::lonhomework::set_bubble_lines();
     }
@@ -338,6 +490,37 @@ sub end_foilgroup {
   return $result;
 }
 
+sub grading_is_nonlenient {
+    my ($part,$id) = @_;
+# Web mode: we are non-lenient unless told otherwise
+    my $defaultparm = 'off';
+    my $nonlenient = 1;
+# Grading a bubblesheet exam: we are grading lenient unless told otherwise
+    if ($Apache::lonhomework::scantronmode) {
+        $defaultparm = 'on';
+        $nonlenient = 0;
+    }
+    my $lenientparm =
+        &Apache::response::get_response_param($part,'lenient',$defaultparm);
+    if ($lenientparm eq 'default') {
+        $lenientparm = $defaultparm;
+    }
+    $lenientparm =
+        &Apache::response::get_response_param($part.'_'.$id,'lenient',$lenientparm);
+    if ($lenientparm eq 'default') {
+        $lenientparm = $defaultparm;
+    }
+    if ($lenientparm=~/^0|off|no$/i) {
+        $nonlenient = 1;
+    } elsif ($lenientparm=~/^1|on|yes$/i) {
+        $nonlenient = 0;
+    }
+    if (!$nonlenient) {
+        $Apache::inputtags::leniency=1;
+    }
+    return $nonlenient;
+}
+
 sub getfoilcounts {
   my ($max)=@_;
   # +1 since instructors will count from 1
@@ -357,8 +540,11 @@ sub whichfoils {
 
 sub displayanswers {
     my ($max,$randomize,@opt)=@_;
-    if (!defined(@{ $Apache::response::foilgroup{'names'} })) {return;}
-    my @names = @{ $Apache::response::foilgroup{'names'} };
+    my @names;
+    if (ref($Apache::response::foilgroup{'names'}) eq 'ARRAY') {
+        @names = @{ $Apache::response::foilgroup{'names'} };
+    }
+    return if (!@names);
     my @whichopt = &whichfoils($max,$randomize);
     my $result;
     if ($Apache::lonhomework::type eq 'exam') {
@@ -391,7 +577,7 @@ sub check_box_opt {
 # Check if we are in checkbox mode: checkboxvalue specified, on web, only two options.
 # If so, return "checked" value
     if ($#opt!=1) { return ''; }
-    if ($target ne 'web') { return ''; }
+    unless (($target eq 'web') || ($target eq 'grade')) { return ''; }
     return $checkboxvalue;
 }
 
@@ -418,9 +604,12 @@ sub check_for_invalid {
 
 sub displayfoils {
   my ($target,$max,$randomize,$TeXlayout,$checkboxvalue,$checkboxchoices,
-      $tex_option_switch,@opt)=@_;
-  if (!defined(@{ $Apache::response::foilgroup{'names'} })) {return;}
-  my @names = @{ $Apache::response::foilgroup{'names'} };
+      $tex_option_switch, $no_tfprompt, $maxcheck, $mincheck, @opt)=@_;
+  my @names; 
+  if (ref($Apache::response::foilgroup{'names'}) eq 'ARRAY') {
+      @names = @{ $Apache::response::foilgroup{'names'} };
+  }
+  unless (@names > 0) { return;}
   my @truelist;
   my @falselist;
   my $result;  
@@ -434,15 +623,21 @@ sub displayfoils {
   my $part=$Apache::inputtags::part;
   my $id=$Apache::inputtags::response[-1];
   my $break;
-  my $solved=$Apache::lonhomework::history{"resource.$part.solved"};
   if ( ($target ne 'tex') &&
        &Apache::response::show_answer() ) {
     my $temp=1;
     foreach $name (@whichopt) {
 	my $text=$Apache::response::foilgroup{$name.'.text'};
         my $lastresp;
-        unless ((($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurvey') || ($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurveycred')) && (defined($env{'form.grade_symb'}))) {
+        if ((($env{'form.grade_username'} eq '') && ($env{'form.grade_domain'} eq '')) ||
+            (($env{'form.grade_username'} eq $env{'user.name'}) &&
+             ($env{'form.grade_domain'} eq $env{'user.domain'}))) {
             $lastresp = $Apache::lonhomework::history{"resource.$part.$id.submission"};
+        } else {
+            unless (($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurvey') ||
+                    ($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurveycred')) {
+                $lastresp = $Apache::lonhomework::history{"resource.$part.$id.submission"};
+            }
         }
 	my %lastresponse=&Apache::lonnet::str2hash($lastresp);
 	my $lastopt=$lastresponse{$name};
@@ -481,14 +676,28 @@ sub displayfoils {
         }
     }
     unless ($newvariation) {
-        %lastresponse=&Apache::lonnet::str2hash($Apache::lonhomework::history{"resource.$part.$id.submission"});
+        if ((($env{'form.grade_username'} eq '') && ($env{'form.grade_domain'} eq '')) ||
+            (($env{'form.grade_username'} eq $env{'user.name'}) &&
+             ($env{'form.grade_domain'} eq $env{'user.domain'}))) {
+                 %lastresponse =
+                     &Apache::lonnet::str2hash($Apache::lonhomework::history{"resource.$part.$id.submission"});
+        } else {
+            unless (($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurvey') ||
+                    ($Apache::lonhomework::history{"resource.$part.type"} eq 'anonsurveycred')) {
+                %lastresponse =
+                    &Apache::lonnet::str2hash($Apache::lonhomework::history{"resource.$part.$id.submission"});
+            }
+        }
     }
     my $internal_counter=$Apache::lonxml::counter;
-    my $checkboxopt=&check_box_opt($target,$checkboxvalue,@opt);
-    if ($checkboxopt) {
+    my $checkboxopt;
+    if ($target eq 'web') {
+        $checkboxopt=&check_box_opt($target,$checkboxvalue,@opt);
+    }
+    if ($checkboxopt && (!$no_tfprompt)) {
        $result.='<br />'.
                 ($checkboxchoices?&mt('Choices: ').'<b>'.$opt[0].','.$opt[1].'</b>. ':'').
-                 &mt('Select all that are <b>[_1]</b>.',$checkboxopt);
+                 &mt('Select all that are [_1].','<b>'.$checkboxopt.'</b>');
     }
     foreach $name (@whichopt) {
       my $text=$Apache::response::foilgroup{$name.'.text'};
@@ -606,9 +815,24 @@ sub displayfoils {
 	  }
 	  if ($Apache::lonhomework::type eq 'exam') {
 	      $result.='\vskip -1 mm\noindent';
-              $result.= '\textbf{'. $internal_counter.'}. \vskip -3mm'.&bubbles(\@alphabet,\@opt).
-		                 ' \strut ';
-	      $internal_counter++;
+              my ($numrows,$bubbles_per_row) = &getnumrows(scalar(@opt)); 
+              if ($numrows == 1) {  
+                  $result .= '\textbf{'.$internal_counter.'}. \vskip -3mm';
+              } else {
+                  my $linetext;
+                  for (my $i=0; $i<$numrows; $i++) {
+                      $linetext .= $internal_counter+$i.', ';
+                  }
+                  $linetext =~ s/,\s$//;
+                  $result .= '\small {\textbf{'.$linetext.'}} '.
+                             '\hskip 2 mm {\footnotesize '.
+                             &mt('(Bubble once in [_1] lines)',$numrows).
+                             '} \vskip 1 mm';
+              }
+              $result.= &bubbles(\@alphabet,\@opt,undef,undef,$numrows,
+                                 $bubbles_per_row,$internal_counter).
+                        ' \strut ';
+	      $internal_counter += $numrows;
 	  }
           if ($target eq 'tex' && $env{'form.pdfFormFields'} eq 'yes'
               && $Apache::inputtags::status[-1] eq 'CAN_ANSWER'
@@ -689,7 +913,8 @@ sub webbubbles {
 
 
 sub bubbles {
-    my ($ralphabet,$ropt,$response, $max_width) = @_;
+    my ($ralphabet,$ropt,$response,$max_width,$numrows,$bubbles_per_row,
+        $internal_counter) = @_;
     my @alphabet = @$ralphabet;
     my @opt = @$ropt;
     my ($result,$head,$line) =('','','');
@@ -710,28 +935,37 @@ sub bubbles {
     }
     &Apache::lonxml::debug("Final maxwidth: $textwidth");
     for (my $ind=0;$ind<=$number_of_bubbles;$ind++) {
+        my $item;
+        if ($numrows > 1) {
+            my $num = $internal_counter+int($ind/$bubbles_per_row);
+            my $idx = int($ind % $bubbles_per_row);
+            $item = $num.$alphabet[$idx];
+        } else {
+            $item = $alphabet[$ind];
+        }
 	my $leftmargin;
 	$opt[$ind]=&Apache::lonxml::latex_special_symbols($opt[$ind]);
 	if ($response eq 'rankresponse') {$opt[$ind]='Rank '.$opt[$ind];}
 	if ($ind==0) {$leftmargin=6;} else {$leftmargin=10;}
 
-	$current_length += (length($opt[$ind])+length($alphabet[$ind])+4)*2;
+	$current_length += (length($opt[$ind])+length($item)+5)*2;
+        
 	if ($current_length<($textwidth-$leftmargin) and $ind!=$number_of_bubbles) {
-	    $line.='\hskip 4 mm {\small \textbf{'.$alphabet[$ind].'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';
+            
+	    $line.='\hskip 4 mm {\small \textbf{'.$item.'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';
 	    $head.='lr';
 	} else {
 	    $line=~s/\&\s*$//;
 	    $result.='\vskip -1 mm\noindent\setlength{\tabcolsep}{2 mm}\renewcommand{\arraystretch}{1.25}\begin{tabular}{'.$head.'}'.$line.'\\\\\end{tabular}\vskip 0 mm';
-	    $line = '\hskip 4 mm {\small \textbf{'.$alphabet[$ind].'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';;
+	    $line = '\hskip 4 mm {\small \textbf{'.$item.'}}$\bigcirc$\hskip -1 mm & \hskip -3 mm {\small '.$opt[$ind].'} & ';
 	    $head ='lr';
-	    $current_length = (length($opt[$ind])+length($alphabet[$ind]))*2;
+	    $current_length = (length($opt[$ind])+length($item))*2;
 	}
 
     }
     return $result;
 }
 
-
 sub start_conceptgroup {
   my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
   $Apache::optionresponse::conceptgroup=1;
@@ -876,6 +1110,24 @@ sub insert_foil {
 sub insert_drawoptionlist {
     return '<drawoptionlist />';
 }
+
+sub getnumrows {
+    my ($numitems) = @_;
+    my $bubbles_per_row;
+    my $default_numbubbles = 10;
+    if (($env{'form.bubbles_per_row'} =~ /^\d+$/) &&
+        ($env{'form.bubbles_per_row'} > 0)) {
+        $bubbles_per_row = $env{'form.bubbles_per_row'};
+    } else {
+        $bubbles_per_row = $default_numbubbles;
+    }
+    my $numrows = int ($numitems/$bubbles_per_row);
+    if (($numitems % $bubbles_per_row) != 0) {
+        $numrows ++;
+    }
+    return ($numrows,$bubbles_per_row);
+}
+
 1;
 __END__
  
