@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Utilities for classifying courses based on institutional code
 #
-# $Id: courseclassifier.pm,v 1.18 2014/03/17 02:33:32 raeburn Exp $
+# $Id: courseclassifier.pm,v 1.23 2015/05/22 16:00:55 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -148,7 +148,7 @@ sub build_code_selections {
     for (my $i=1; $i<@{$codetitles}; $i++) {
         %{$idarrays{$$codetitles[$i]}} = ();
     }
-    foreach my $cid (sort keys %{$codes}) {
+    foreach my $cid (sort(keys(%{$codes}))) {
         &recurse_list($cid,$codetitles,$codes,0,\%idarrays);
     }
     for (my $num=0; $num<@{$codetitles}; $num++) {
@@ -169,7 +169,7 @@ sub build_code_selections {
         } elsif ($num == 1) {
             %{$$idlist{$$codetitles[1]}} = ();
             %{$$idlist_titles{$$codetitles[1]}} = ();
-            foreach my $key_a (keys %{$idarrays{$$codetitles[1]}}) {
+            foreach my $key_a (keys(%{$idarrays{$$codetitles[1]}})) {
                 my @sorted_a = ();
                 my @sorted_a_titles = ();
                 &sort_cats($num,$cat_order,$codetitles,\@{$idarrays{$$codetitles[1]}{$key_a}},\@sorted_a);
@@ -187,10 +187,10 @@ sub build_code_selections {
         } elsif ($num == 2) {
             %{$$idlist{$$codetitles[2]}} = ();
             %{$$idlist_titles{$$codetitles[2]}} = ();
-            foreach my $key_a (keys %{$idarrays{$$codetitles[2]}}) {
+            foreach my $key_a (keys(%{$idarrays{$$codetitles[2]}})) {
                 %{$$idlist{$$codetitles[2]}{$key_a}} = ();
                 %{$$idlist_titles{$$codetitles[2]}{$key_a}} = ();
-                foreach my $key_b (keys %{$idarrays{$$codetitles[2]}{$key_a}}) {
+                foreach my $key_b (keys(%{$idarrays{$$codetitles[2]}{$key_a}})) {
                     my @sorted_b = ();
                     my @sorted_b_titles = ();
                     &sort_cats($num,$cat_order,$codetitles,\@{$idarrays{$$codetitles[2]}{$key_a}{$key_b}},\@sorted_b);
@@ -208,11 +208,11 @@ sub build_code_selections {
             }
         } elsif ($num == 3) {
             %{$$idlist{$$codetitles[3]}} = ();
-            foreach my $key_a (keys %{$idarrays{$$codetitles[3]}}) {
+            foreach my $key_a (keys(%{$idarrays{$$codetitles[3]}})) {
                 %{$$idlist{$$codetitles[3]}{$key_a}} = ();
-                foreach my $key_b (keys %{$idarrays{$$codetitles[3]}{$key_a}}) {
+                foreach my $key_b (keys(%{$idarrays{$$codetitles[3]}{$key_a}})) {
                     %{$$idlist{$$codetitles[3]}{$key_a}{$key_b}} = ();
-                    foreach my $key_c (keys %{$idarrays{$$codetitles[3]}{$key_a}{$key_b}}) {
+                    foreach my $key_c (keys(%{$idarrays{$$codetitles[3]}{$key_a}{$key_b}})) {
                         my @sorted_c = ();
                         my @sorted_c_titles = ();
                         &sort_cats($num,$cat_order,$codetitles,\@{$idarrays{$$codetitles[3]}{$key_a}{$key_b}{$key_c}},\@sorted_c);
@@ -231,13 +231,13 @@ sub build_code_selections {
             }
         } elsif ($num == 4) {
             %{$$idlist{$$codetitles[4]}} = ();
-            foreach my $key_a (keys %{$idarrays{$$codetitles[4]}}) {
+            foreach my $key_a (keys(%{$idarrays{$$codetitles[4]}})) {
                 %{$$idlist{$$codetitles[4]}{$key_a}} = ();
-                foreach my $key_b (keys %{$idarrays{$$codetitles[4]}{$key_a}}) {
+                foreach my $key_b (keys(%{$idarrays{$$codetitles[4]}{$key_a}})) {
                     %{$$idlist{$$codetitles[4]}{$key_a}{$key_b}} = ();
-                    foreach my $key_c (keys %{$idarrays{$$codetitles[4]}{$key_a}{$key_b}}) {
+                    foreach my $key_c (keys(%{$idarrays{$$codetitles[4]}{$key_a}{$key_b}})) {
                         %{$$idlist{$$codetitles[4]}{$key_a}{$key_b}{$key_c}} = ();
-                        foreach my $key_d (keys %{$idarrays{$$codetitles[4]}{$key_a}{$key_b}{$key_c}}) {
+                        foreach my $key_d (keys(%{$idarrays{$$codetitles[4]}{$key_a}{$key_b}{$key_c}})) {
                             my @sorted_d = ();
                             my @sorted_d_titles = ();
                             &sort_cats($num,$cat_order,$codetitles,$idarrays{$$codetitles[4]}{$key_a}{$key_b}{$key_c}{$key_d},\@sorted_d);
@@ -575,15 +575,18 @@ function setElements() {
 }
 
 sub build_instcode_selectors {
-    my ($numtitles,$lasttitle,$cat_items,$codetitles,$cat_titles,$cat_order) = @_;
+    my ($numtitles,$lasttitle,$cat_items,$codetitles,$cat_titles,$cat_order,$official,$class,$id) = @_;
     my $output;
     my @standardnames = &Apache::loncommon::get_standard_codeitems();
     if ($numtitles > 0) {
         my $style;
-        if ($env{'form.official'} eq 'off') {
+        if ($official eq 'off') {
             $style = ' style="display: none"';
         }
-        $output .= '<div id="instcodes"'.$style.'><table><tr>';
+        if ($id eq '') {
+            $id = 'instcodes';
+        }
+        $output .= '<div id="'.$id.'"'.$style.'><table><tr>';
         for (my $k=0; $k<$lasttitle-1; $k++) {
             my (@items,@unsorted);
             if (ref($cat_items->{$codetitles->[$k]}) eq 'ARRAY') {
@@ -599,8 +602,11 @@ sub build_instcode_selectors {
                 @longitems = @items;
             }
             $output .= '<td align="center">'.$codetitles->[$k].'<br />'."\n".
-                       '<select name="'.$standardnames[$k].'" onchange="courseSet(this)"'.
-                       '>'."\n".'<option value="0">'.&mt('All').'</option>'."\n";
+                       '<select name="'.$standardnames[$k].'" onchange="courseSet(this)"';
+            if ($class) {
+                $output .= ' class="'.$class.'"';
+            }
+            $output .= '>'."\n".'<option value="0">'.&mt('All').'</option>'."\n";
             for (my $i=0; $i<@items; $i++) {
                 if ($longitems[$i] eq '') {
                     $longitems[$i] = $items[$i];
@@ -610,7 +616,11 @@ sub build_instcode_selectors {
             $output .= '</select></td>';
         }
         $output .= '<td align="center">'.$codetitles->[$lasttitle-1].'<br />'."\n".
-                   '<select name="'.$standardnames[$lasttitle-1].'">'."\n".
+                   '<select name="'.$standardnames[$lasttitle-1].'"';
+        if ($class) {
+            $output .= ' class="'.$class.'"';
+        }
+        $output .= '>'."\n".
                    '<option value="0">'.&mt('All')."\n".
                    '</option>'."\n".'</select>'."\n".
                    '</td></tr></table>'."\n";
@@ -644,7 +654,7 @@ sub instcode_selectors {
                    '<input type="radio" name="official" value="off"'.$unofficial.
                    ' onclick="toggleOfficial();" />'.&mt('No').'</label></span><br />'.
                    &build_instcode_selectors($numtitles,$lasttitle,\%cat_items,$codetitles,
-                                             \%cat_titles,\%cat_order)."\n".
+                                             \%cat_titles,\%cat_order,$env{'form.official'})."\n".
                    '<input type="hidden" name="numtitles" value="'.$numtitles.'" />'."\n".
                    '<input type="hidden" name="state" value="listing" />'."\n";
     }
@@ -687,7 +697,8 @@ sub build_javascript {
     my ($by_year,$by_sem,$by_dept,$cat_order,$codetitles) = @_;
     my @unsorted = keys(%{$by_year});
     my @sorted_yrs;
-    &sort_cats('0',$cat_order,$codetitles,\@unsorted,\@sorted_yrs);    my $output = 'var idcse_by_yr_year = new Array("'.join('","',@sorted_yrs).'");'."\n".
+    &sort_cats('0',$cat_order,$codetitles,\@unsorted,\@sorted_yrs);
+    my $output = 'var idcse_by_yr_year = new Array("'.join('","',@sorted_yrs).'");'."\n".
                  'var idcse_by_yr_dept = new Array('.scalar(@sorted_yrs).');'."\n".
                  'var idcse_by_yr_num = new Array('.scalar(@sorted_yrs).');'."\n";
     for (my $i=0; $i<@sorted_yrs; $i++) {
@@ -739,12 +750,23 @@ sub javascript_select_filler {
     my $nocrs = &mt('No courses');
     my $output = <<END;
 
-function courseSet(caller) {
+function courseSet(caller, onload) {
     var longtitles = new Array ("$longtitles_str");
-    var valyr = document.$formname.Year.options[document.$formname.Year.selectedIndex].value
-    var valsem  = document.$formname.Semester.options[document.$formname.Semester.selectedIndex].value
-    var valdept = document.$formname.Department.options[document.$formname.Department.selectedIndex].value
-    var valclass = document.$formname.Number.options[document.$formname.Number.selectedIndex].value
+    var valyr = '';
+    var valsem = '';
+    var valdept = '';
+    var valclass = '';
+    if (onload) {
+        valyr = "$env{'form.Year'}";
+        valsem = "$env{'form.Semester'}";
+        valdept = "$env{'form.Department'}";
+        valclass = "$env{'form.Number'}";
+    } else {
+        valyr = document.$formname.Year.options[document.$formname.Year.selectedIndex].value;
+        valsem  = document.$formname.Semester.options[document.$formname.Semester.selectedIndex].value;
+        valdept = document.$formname.Department.options[document.$formname.Department.selectedIndex].value;
+        valclass = document.$formname.Number.options[document.$formname.Number.selectedIndex].value;
+    }
     var idyears = new Array("$allidlist");
     var idyr = -1;
     var idsem = -1;
@@ -753,9 +775,36 @@ function courseSet(caller) {
 
     $scripttext
 
-    selYear = document.$formname.Year.selectedIndex-1;
-    selSemester = document.$formname.Semester.selectedIndex-1;
-    selDepartment = document.$formname.Department.selectedIndex-1;
+    if (onload) {
+        if (document.$formname.Year.options.length > 0) {
+            for (var i = 0; i< document.$formname.Year.options.length; i++) {
+                if (document.$formname.Year.options[i].value == "$env{'form.Year'}") {
+                    document.$formname.Year.selectedIndex = i;
+                    break;
+                }
+            }
+        }
+        if (document.$formname.Department.options.length > 0) {
+            for (var i = 0; i< document.$formname.Department.options.length; i++) {
+                if (document.$formname.Department.options[i].value == "$env{'form.Department'}") {
+                    document.$formname.Department.selectedIndex = i;
+                    break;
+                }
+            }
+        }
+        if (document.$formname.Semester.options.length > 0) {
+            for (var i = 0; i< document.$formname.Semester.options.length; i++) {
+                if (document.$formname.Semester.options[i].value == "$env{'form.Semester'}") {
+                    document.$formname.Semester.selectedIndex = i;
+                    break;
+                }
+            }
+        }
+    }
+
+    var selYear = document.$formname.Year.selectedIndex-1;
+    var selSemester = document.$formname.Semester.selectedIndex-1;
+    var selDepartment = document.$formname.Department.selectedIndex-1;
     if (selYear == -1) {
         if (selSemester == -1) {
             if (selDepartment > -1) {
@@ -870,7 +919,18 @@ function courseSet(caller) {
         }
         document.$formname.Number.selectedIndex = 0
     }
-    $officialjs
+    if (onload) {
+        if (document.$formname.Number.options.length > 0) {
+            for (var i = 0; i< document.$formname.Number.options.length; i++) {
+                if (document.$formname.Number.options[i].value == "$env{'form.Number'}") {
+                    document.$formname.Number.selectedIndex = i;
+                    break;
+                }
+            }
+        }
+    } else {
+        $officialjs
+    }
 }
 END
     return $output;
