@@ -4,7 +4,7 @@
 # kerberos, or SSO) or an e-mail address. Requests to use an e-mail address as
 # username may be processed automatically, or may be queued for approval.
 #
-# $Id: createaccount.pm,v 1.68 2014/06/19 23:36:54 raeburn Exp $
+# $Id: createaccount.pm,v 1.69 2015/01/09 15:41:49 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -856,8 +856,8 @@ sub process_mailtoken {
 
 sub start_session {
     my ($r,$username,$domain,$uhome,$courseid,$token) = @_;
-
-    if ($r->dir_config('lonBalancer') eq 'yes') {
+    my ($is_balancer) = &Apache::lonnet::check_loadbalancing($username,$domain);
+    if ($is_balancer) {
         Apache::lonauth::success($r, $username, $domain, $uhome,
             'noredirect', undef, {});
 
@@ -871,7 +871,6 @@ sub start_session {
             ($courseid ? "/adm/selfenroll?courseid=$courseid" : '/adm/roles'),
             undef, {}); 
     }
-
     return;
 }
 
