@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Handler to set domain-wide configuration settings
 #
-# $Id: lonconfigsettings.pm,v 1.15.2.2 2010/12/24 22:50:34 raeburn Exp $
+# $Id: lonconfigsettings.pm,v 1.21.4.7 2015/05/22 16:52:16 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -35,15 +35,20 @@ use Apache::lonnet;
 use Apache::loncommon();
 use Apache::lonhtmlcommon();
 use Apache::lonlocal;
+use Apache::courseclassifier();
+use LONCAPA qw(:DEFAULT :match);
 
 sub print_header {
-    my ($r,$phase,$context,$jscript) = @_;
-    my ($pagetitle,$brcrumtitle,$action,$call_category_check);
+    my ($r,$phase,$context,$jscript,$container) = @_;
+    my ($pagetitle,$brcrumtitle,$action,$call_category_check,$instcode_check,
+        $crstype,@actions,@code_order);
+    if ($phase eq 'display') {
+        @actions = &Apache::loncommon::get_env_multiple('form.actions');
+    }
     if ($context eq 'domain') {
         ($pagetitle, $brcrumtitle) = ('View/Modify Domain Settings','View/Modify Domain Settings');
         $action = '/adm/domainprefs';
         if ($phase eq 'display') {
-            my @actions = &Apache::loncommon::get_env_multiple('form.actions');
             if (grep(/^coursecategories$/,@actions)) {
                 $call_category_check = qq|
     if (formname == document.display) {
@@ -55,12 +60,61 @@ sub print_header {
             }
         }
     } else {
-        if (&Apache::loncommon::course_type() eq 'Community') {
+        $crstype = &Apache::loncommon::course_type();
+        if ($crstype eq 'Community') {
             ($pagetitle,$brcrumtitle) = ('Community Configuration','Community Configuration');
         } else {
             ($pagetitle,$brcrumtitle) = ('Course Configuration','Course Configuration');
         }
         $action = '/adm/courseprefs';
+        if ($phase eq 'display') {
+            if (grep(/^courseinfo$/,@actions)) {
+                my %codedefaults;
+                &Apache::lonnet::auto_instcode_defaults($env{'request.role.domain'},\%codedefaults,
+                                                        \@code_order);
+                if (@code_order) {
+                   my $noinstcodestr = &mt('You indicated cloning based on category, but did not select any categories.');
+                   $instcode_check = <<"ENDSCRIPT";
+    if (formname == document.display) {
+        if (formname.cloners_instcode.length) {
+            for (var j=0; j<formname.cloners_instcode.length; j++) {
+                if (formname.cloners_instcode[j].checked) {
+                    if (formname.cloners_instcode[j].value == 1) {
+                        var codes;
+                        if (document.getElementsByClassName) {
+                            codes = document.getElementsByClassName('LC_cloners_instcodes');
+                        } else {
+                            codes = getElementsByClassName(document.body,'LC_cloners_instcodes');
+                        }
+                        if (codes.length) {
+                            var gotcode = 0;
+                            for (var i=0; i<codes.length; i++) {
+                                if (codes[i].selectedIndex != 0) {
+                                     gotcode = 1;
+                                     break;
+                                }
+                            }
+                            if (!gotcode) {
+                                for (var k=0; k<formname.cloners_instcode.length; k++) {
+                                    if (formname.cloners_instcode[k].value == 0) {
+                                        formname.cloners_instcode[k].checked = true;
+                                    }
+                                }
+                                toggleCloners(document.display.cloners_instcode);
+                                alert('$noinstcodestr');
+                                return false;
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+ENDSCRIPT
+                }
+            }
+        }
     }
     my $alert = &mt('You must select at least one functionality type to display.');
     my $js = '
@@ -89,13 +143,15 @@ function changePage(formname,newphase) {
             return;
         }
     }
-    '.$call_category_check.'
+    '.$instcode_check.$call_category_check.'
     formname.submit();
 }'."\n";
     if ($phase eq 'pickactions') {
+        $js .= &Apache::lonhtmlcommon::color_picker();
         $js .=
             &Apache::lonhtmlcommon::set_form_elements({actions => 'checkbox'})."\n";
     } elsif ($phase eq 'display') {
+        $js .= &Apache::lonhtmlcommon::color_picker();
         $js .= &color_pick_js()."\n";
     }
     $js .= &Apache::loncommon::viewport_size_js().'
@@ -116,6 +172,25 @@ $jscript
                     'onload' => "setFormElements(document.pickactions);",
                         );
         $additem = {'add_entries' => \%loaditems,};
+    } elsif ($phase eq 'display') {
+        if ($context eq 'domain') {
+            if (grep(/^coursedefaults$/,@actions)) {
+                my %loaditems = (
+                    '             onload' => "toggleDisplay(document.display,'cloneinstcode');".
+                                             "toggleDisplay(document.display,'credits');".
+                                             "toggleDisplay(document.display,'studentsubmission');",
+                                );
+                $additem = {'add_entries' => \%loaditems,};
+            }
+        } elsif ($context eq 'course') {
+            if (grep(/^courseinfo$/,@actions)) {
+                if (@code_order) {
+                    $additem = {
+                                   add_entries => {'onload' => "courseSet('','load');toggleCloners(document.display.cloners_instcode);"},
+                               };
+                }
+            }
+        }
     }
     $r->print(&Apache::loncommon::start_page($pagetitle,$js,$additem));
     $r->print(&Apache::lonhtmlcommon::breadcrumbs($brcrumtitle));
@@ -132,7 +207,7 @@ $jscript
 }
 
 sub print_footer {
-    my ($r,$phase,$newphase,$button_text,$actions) = @_;
+    my ($r,$phase,$newphase,$button_text,$actions,$container) = @_;
     $button_text = &mt($button_text);
     $r->print('<input type="hidden" name="phase" value="" />');
     if (defined($env{'form.origin'})) {
@@ -147,7 +222,9 @@ sub print_footer {
     }
     my $dest='"javascript:changePage(document.'.$phase.','."'$newphase'".')"';
     if ($phase eq 'process') {
-        $r->print('<p><a href='.$dest.'>'.$button_text.'</a></p>');
+        $r->print(
+            &Apache::lonhtmlcommon::actionbox(
+                ['<a href='.$dest.'>'.$button_text.'</a>']));
     } else {
         my $onclick;
         if ($phase eq 'display') {
@@ -159,13 +236,14 @@ sub print_footer {
                   $button_text.'" onclick='.$onclick.' /></p>');
     }
     if ($phase eq 'process') {
-        $r->print('</form>'.&Apache::loncommon::end_page());
+        $r->print('</form>');
+        $r->print(&Apache::loncommon::end_page());
     }
     return;
 }
 
 sub make_changes {
-    my ($r,$dom,$phase,$context,$prefs_order,$prefs,$values,$confname,$roles,$allitems) = @_;
+    my ($r,$dom,$phase,$context,$prefs_order,$prefs,$values,$confname,$roles,$allitems,$container) = @_;
     my %brcrumtext = &get_crumb_text();
     my @actions = &Apache::loncommon::get_env_multiple('form.actions');
     my ($numchanged,%changes,%disallowed);
@@ -174,8 +252,8 @@ sub make_changes {
         text=>$brcrumtext{$context}},
        {href=>"javascript:changePage(document.$phase,'$phase')",
         text=>"Updated"});
-    &print_header($r,$phase,$context);
-    my $crstype;
+    &print_header($r,$phase,$context,undef,$container);
+    my ($crstype,%lastact);
     if ($context eq 'course') {
         $crstype = &Apache::loncommon::course_type();
     }
@@ -186,7 +264,7 @@ sub make_changes {
                 if ($context eq 'domain') {
                     $r->print('<h3>'.&mt($prefs->{$item}{'text'}).'</h3>'.
                               &Apache::domainprefs::process_changes($r,$dom,
-                                          $confname,$item,$roles,$values));
+                                          $confname,$item,$roles,$values,\%lastact));
                 } else {
                     $changes{$item} = {};
                     &Apache::courseprefs::process_changes($dom,$item,$values,
@@ -229,19 +307,24 @@ sub make_changes {
     if ($context eq 'course') {
         $footer_text = 'Back to display/edit settings'; 
     }
-    &print_footer($r,$phase,'display',$footer_text,\@actions);
+    &print_footer($r,$phase,'display',$footer_text,\@actions,$container);
     $r->print('</p>');
+    return \%lastact;
 }
 
 sub display_settings {
     my ($r,$dom,$phase,$context,$prefs_order,$prefs,$values,$confname,$jscript,
-        $allitems,$crstype) = @_;
+        $allitems,$crstype,$container) = @_;
     my %brcrumtext = &get_crumb_text();
     my @actions = &Apache::loncommon::get_env_multiple('form.actions');
     &Apache::lonhtmlcommon::add_breadcrumb
         ({href=>"javascript:changePage(document.$phase,'display')",
           text=>"Display/Edit Settings"});
-    &print_header($r,$phase,$context,$jscript);
+    my $instcode;
+    if (ref($values) eq 'HASH') {
+        $instcode = $values->{'internal.coursecode'};
+    }
+    &print_header($r,$phase,$context,$jscript,$container,$instcode);
     my $divwidth = 900;
     if ((ref($prefs_order) eq 'ARRAY') && (ref($prefs) eq 'HASH') && (ref($values) eq 'HASH')) { 
         if (@actions > 0) {
@@ -251,16 +334,44 @@ sub display_settings {
                 if (grep(/^\Q$item\E$/,@actions)) {
                     push(@items,$item);
                     if ($context eq 'domain') {
+                        my $settings;
+                        if (ref($values) eq 'HASH') {
+                            $settings = $values->{$item};
+                        }
                         if ($item eq 'usersessions') {
                             $r->print('<script type="text/javascript">'."\n".
                                       '// <![CDATA['."\n".
                                       &Apache::loncommon::check_uncheck_jscript()."\n".
                                       '// ]]>'."\n".
                                       '</script>'."\n");
+                        } elsif ($item eq 'selfcreation') {
+                            if (ref($values) eq 'HASH') {
+                                $settings = $values->{'usercreation'};
+                            }
+                        } elsif ($item eq 'defaults') {
+                            if (ref($values->{'inststatus'}) eq 'HASH') {
+                                if (ref($values->{'defaults'}) eq 'HASH') {
+                                    $settings = {%{$values->{'inststatus'}},%{$values->{'defaults'}}};
+                                } else {
+                                    $settings = $values->{'inststatus'};
+                                }
+                            } else {
+                                my ($othertitle,$usertypes,$types) = &Apache::loncommon::sorted_inst_types($dom);
+                                my $inststatus = {
+                                                   inststatustypes => $usertypes,
+                                                   inststatusorder => $types,
+                                                   inststatusguest => [],
+                                                 };
+                                if (ref($values->{defaults}) eq 'HASH') {
+                                    $settings = {%{$inststatus},%{$values->{'defaults'}}};
+                                } else {
+                                    $settings = $inststatus;
+                                }
+                            }
                         }
                         ($output{$item},$rowtotal{$item}) =
                             &Apache::domainprefs::print_config_box($r,$dom,$confname,
-                                $phase,$item,$prefs->{$item},$values->{$item});
+                                $phase,$item,$prefs->{$item},$settings);
                     } else {
                         ($output{$item},$rowtotal{$item}) =
                             &Apache::courseprefs::print_config_box($r,$dom,$phase,
@@ -274,7 +385,7 @@ sub display_settings {
                 $r->print($output{$items[$i]});
             }
             $r->print('</div>');
-            $r->print(&print_footer($r,$phase,'process','Save Changes',\@actions));
+            $r->print(&print_footer($r,$phase,'process','Save Changes',\@actions,$container));
         } else {
             $r->print('<input type="hidden" name="phase" value="" />'.
                       '<span class="LC_error">'.&mt('No settings chosen').
@@ -287,12 +398,12 @@ sub display_settings {
 }
 
 sub display_choices {
-    my ($r,$phase,$context,$prefs_order,$prefs) = @_;
+    my ($r,$phase,$context,$prefs_order,$prefs,$container) = @_;
     if ($phase eq '') {
         $phase = 'pickactions';
     }
     my %helphash;
-    &print_header($r,$phase,$context);
+    &print_header($r,$phase,$context,undef,$container);
     $r->print('<script type="text/javascript">'."\n".
               '// <![CDATA['."\n".
               &Apache::loncommon::check_uncheck_jscript()."\n".
@@ -336,7 +447,7 @@ sub display_choices {
         }
         $r->print('</div><br clear="all" />');
     }
-    $r->print(&print_footer($r,$phase,'display','Display'));
+    $r->print(&print_footer($r,$phase,'display','Display',undef,$container));
     $r->print('</form>');
     $r->print(&Apache::loncommon::end_page());
     return;
@@ -345,15 +456,11 @@ sub display_choices {
 sub color_pick_js {
     my $pjump_def = &Apache::lonhtmlcommon::pjump_javascript_definition();
     my $output = <<"ENDCOL";
-    function pclose() {
-        parmwin=window.open("/adm/rat/empty.html","LONCAPAparms","height=350,width=350,scrollbars=no,menubar=no");
-        parmwin.close();
-    }
 
     $pjump_def
 
     function psub() {
-        pclose();
+        modalWindow.close();
         if (document.parmform.pres_marker.value!='') {
             if (document.parmform.pres_type.value!='') {
                 eval('document.display.'+
