@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Routines for messaging
 #
-# $Id: lonmsg.pm,v 1.232 2011/03/05 21:32:02 raeburn Exp $
+# $Id: lonmsg.pm,v 1.239 2015/02/13 01:46:03 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -120,7 +120,7 @@ Critical message to a user
 
 New routine that respects "forward" and calls old routine
 
-=item * B<user_crit_msg($user, $domain, $subject, $message, $sendback, $nosentstore,$recipid,$attachmenturl)>: 
+=item * B<user_crit_msg($user, $domain, $subject, $message, $sendback, $nosentstore, $recipid, $attachmenturl, $permresults)>: 
     Sends a critical message $message to the $user at $domain.  If $sendback
     is true,  a receipt will be sent to the current user when $user receives 
     the message.
@@ -148,7 +148,7 @@ New routine that respects "forward" and calls old routine
 
 =item * B<user_normal_msg($user, $domain, $subject, $message, $citation,
        $baseurl, $attachmenturl, $toperm, $sentmessage, $symb, $restitle,
-       $error,$nosentstore,$recipid)>:
+       $error,$nosentstore,$recipid,$permresults)>:
  Sends a message to the  $user at $domain, with subject $subject and message $message.
 
     Additionally it will check if the user has a Forwarding address
@@ -424,6 +424,7 @@ sub sendemail {
     my ($to,$subject,$body,$to_uname,$to_udom,$user_lh)=@_;
     my $senderaddress='';
     my $replytoaddress='';
+    my $msgsent;
     if ($env{'form.can_reply'} eq 'N') {
         my $lonhost = $Apache::lonnet::perlvar{'lonHostID'};
         my $hostname = &Apache::lonnet::hostname($lonhost);
@@ -469,7 +470,9 @@ sub sendemail {
     if (my $fh = $msg->open()) {
 	print $fh $body;
 	$fh->close;
+        $msgsent = 1;
     }
+    return $msgsent;
 }
 
 # ==================================================== Send notification emails
@@ -481,6 +484,7 @@ sub sendnotification {
 	$sender=$env{'user.name'}.':'.$env{'user.domain'};
     }
     my $critical=($crit?' critical':'');
+    my $numsent = 0;
 
     $text=~s/\&lt\;/\</gs;
     $text=~s/\&gt\;/\>/gs;
@@ -526,7 +530,7 @@ sub sendnotification {
 
 to access the full message.',$url);
     my %userenv = &Apache::lonnet::get('environment',['notifywithhtml'],$toudom,$touname);
-    my $subject = &mt_user($user_lh,"'New' $critical message from ").$sender;
+    my $subject = &mt_user($user_lh,"'New'$critical message from [_1]",$sender);
     unless ($subj eq '') {
         $subject = $subj;
     }
@@ -558,7 +562,9 @@ to access the full message.',$url);
                 }
                 $body = $bodybegin.$bodysubj.$sendtext.$bodyend;
             }
-            &sendemail($addr,$subject,$body,$touname,$toudom,$user_lh);
+            if (&sendemail($addr,$subject,$body,$touname,$toudom,$user_lh)) {
+                $numsent ++;
+            }
         }
     } else {
         if ($blocked) {
@@ -567,8 +573,11 @@ to access the full message.',$url);
             my $htmlfree = &make_htmlfree($text);
             $body = $bodybegin.$bodysubj.$htmlfree.$bodyend;
         }
-        &sendemail($to,$subject,$body,$touname,$toudom,$user_lh);
+        if (&sendemail($to,$subject,$body,$touname,$toudom,$user_lh)) {
+            $numsent ++;
+        }
     }
+    return $numsent;
 }
 
 sub make_htmlfree {
@@ -625,9 +634,9 @@ sub retrieve_author_res_msg {
     my ($domain,$author)=($url=~/^($match_domain)\/($match_username)\//);
     my %errormsgs=&Apache::lonnet::dump('nohist_res_msgs',$domain,$author);
     my $msgs='';
-    foreach (keys %errormsgs) {
-	if ($_=~/^\Q$url\E\_\d+$/) {
-	    my %content=&unpackagemsg($errormsgs{$_});
+    foreach my $msg (keys(%errormsgs)) {
+	if ($msg =~ /^\Q$url\E\_\d+$/) {
+	    my %content=&unpackagemsg($errormsgs{$msg});
 	    $msgs.='<p><img src="/adm/lonMisc/bomb.gif" /><b>'.
 		$content{'time'}.'</b>: '.$content{'message'}.
 		'<br /></p>';
@@ -645,9 +654,9 @@ sub del_url_author_res_msg {
     $url=&Apache::lonnet::declutter($url);
     my ($domain,$author)=($url=~/^($match_domain)\/($match_username)\//);
     my @delmsgs=();
-    foreach (&Apache::lonnet::getkeys('nohist_res_msgs',$domain,$author)) {
-	if ($_=~/^\Q$url\E\_\d+$/) {
-	    push (@delmsgs,$_);
+    foreach my $msg (&Apache::lonnet::getkeys('nohist_res_msgs',$domain,$author)) {
+	if ($msg =~ /^\Q$url\E\_\d+$/) {
+	    push (@delmsgs,$msg);
 	}
     }
     return &Apache::lonnet::del('nohist_res_msgs',\@delmsgs,$domain,$author);
@@ -659,9 +668,9 @@ sub clear_author_res_msg {
     $url=&Apache::lonnet::declutter($url);
     my ($domain,$author)=($url=~/^($match_domain)\/($match_username)\//);
     my @delmsgs=();
-    foreach (&Apache::lonnet::getkeys('nohist_res_msgs',$domain,$author)) {
-	if ($_=~/^\Q$url\E/) {
-	    push (@delmsgs,$_);
+    foreach my $msg (&Apache::lonnet::getkeys('nohist_res_msgs',$domain,$author)) {
+	if ($msg =~ /^\Q$url\E/) {
+	    push (@delmsgs,$msg);
 	}
     }
     return &Apache::lonnet::del('nohist_res_msgs',\@delmsgs,$domain,$author);
@@ -672,8 +681,8 @@ sub clear_author_res_msg {
 sub all_url_author_res_msg {
     my ($author,$domain)=@_;
     my %returnhash=();
-    foreach (&Apache::lonnet::getkeys('nohist_res_msgs',$domain,$author)) {
-	$_=~/^(.+)\_\d+/;
+    foreach my $msg (&Apache::lonnet::getkeys('nohist_res_msgs',$domain,$author)) {
+	$msg =~ /^(.+)\_\d+/;
 	$returnhash{$1}=1;
     }
     return %returnhash;
@@ -696,7 +705,7 @@ sub store_instructor_comment {
 
 sub user_crit_msg_raw {
     my ($user,$domain,$subject,$message,$sendback,$toperm,$sentmessage,
-        $nosentstore,$recipid,$attachmenturl)=@_;
+        $nosentstore,$recipid,$attachmenturl,$permresults)=@_;
 # Check if allowed missing
     my ($status,$packed_message);
     my $msgid='undefined';
@@ -727,22 +736,40 @@ sub user_crit_msg_raw {
 
 # Notifications
     my %userenv = &Apache::loncommon::getemails($user,$domain);
-    if ($userenv{'critnotification'}) {
-      &sendnotification($userenv{'critnotification'},$user,$domain,$subject,1,
-			$text,$msgid);
+    my $critnotify = $userenv{'critnotification'};
+    my $permemail = $userenv{'permanentemail'};
+    my $numcrit = 0;
+    my $numperm = 0;
+    my $permlogmsgstatus;
+    if ($critnotify) {
+        $numcrit = &sendnotification($critnotify,$user,$domain,$subject,1,$text,$msgid);
     }
-    if ($toperm && $userenv{'permanentemail'}) {
-      &sendnotification($userenv{'permanentemail'},$user,$domain,$subject,1,
-			$text,$msgid);
+    if ($toperm && $permemail) {
+        if ($critnotify && $numcrit) {
+            if (grep(/^\Q$permemail\E/,split(/,/,$critnotify))) {
+                $numperm = 1;
+            }
+        }
+        unless ($numperm) {
+            $numperm = &sendnotification($permemail,$user,$domain,$subject,1,$text,$msgid);
+        }
+    }
+    if ($toperm) {
+        $permlogmsgstatus = '. Perm. email log status '.
+                            &Apache::lonnet::log($env{'user.domain'},$env{'user.name'},$env{'user.home'},
+                                                 "Perm. e-mail count $numperm for $user at $domain");
+        if (ref($permresults) eq 'HASH') {
+            $permresults->{"$user:$domain"} = $numperm;
+        }
     }
 # Log this
     &Apache::lonnet::logthis(
-      'Sending critical email '.$msgid.
+      'Sending critical '.$msgid.
       ', log status: '.
       &Apache::lonnet::log($env{'user.domain'},$env{'user.name'},
                          $env{'user.home'},
-      'Sending critical '.$msgid.' to '.$user.' at '.$domain.' with status: '
-      .$status));
+      'Sending critical '.$msgid.' to '.$user.' at '.$domain.' with status '
+      .$status).$permlogmsgstatus);
     return $status;
 }
 
@@ -750,7 +777,7 @@ sub user_crit_msg_raw {
 
 sub user_crit_msg {
     my ($user,$domain,$subject,$message,$sendback,$toperm,$sentmessage,
-        $nosentstore,$recipid,$attachmenturl)=@_;
+        $nosentstore,$recipid,$attachmenturl,$permresults)=@_;
     my @status;
     my %userenv = &Apache::lonnet::get('environment',['msgforward'],
                                        $domain,$user);
@@ -761,13 +788,13 @@ sub user_crit_msg {
          push(@status,
 	      &user_crit_msg_raw($forwuser,$forwdomain,$subject,$message,
 				 $sendback,$toperm,$sentmessage,$nosentstore,
-                                 $recipid,$attachmenturl));
+                                 $recipid,$attachmenturl,$permresults));
        }
     } else { 
 	push(@status,
 	     &user_crit_msg_raw($user,$domain,$subject,$message,$sendback,
 				$toperm,$sentmessage,$nosentstore,$recipid,
-                                $attachmenturl));
+                                $attachmenturl,$permresults));
     }
     if (wantarray) {
 	return @status;
@@ -816,7 +843,7 @@ sub user_crit_received {
 sub user_normal_msg_raw {
     my ($user,$domain,$subject,$message,$citation,$baseurl,$attachmenturl,
         $toperm,$currid,$newid,$sentmessage,$crsmsgid,$symb,$restitle,
-        $error,$nosentstore,$recipid)=@_;
+        $error,$nosentstore,$recipid,$permresults)=@_;
 # Check if allowed missing
     my ($status,$packed_message);
     my $msgid='undefined';
@@ -853,19 +880,37 @@ sub user_normal_msg_raw {
        }
 # Notifications
        my %userenv = &Apache::loncommon::getemails($user,$domain);
-       if ($userenv{'notification'}) {
-	   &sendnotification($userenv{'notification'},$user,$domain,$subject,0,
-			     $text,$msgid);
+       my $notify = $userenv{'notification'};
+       my $permemail = $userenv{'permanentemail'};
+       my $numnotify = 0;
+       my $numperm = 0;
+       my $permlogmsgstatus;
+       if ($notify) {
+           $numnotify = &sendnotification($notify,$user,$domain,$subject,0,$text,$msgid);
        }
-       if ($toperm && $userenv{'permanentemail'}) {
-           if ((!$userenv{'notification'}) || ($userenv{'notification'} ne $userenv{'permanentemail'})) {
-	       &sendnotification($userenv{'permanentemail'},$user,$domain,$subject,0,
-	  		         $text,$msgid);
+       if ($toperm && $permemail) {
+           if ($notify && $numnotify) {
+               if (grep(/^\Q$permemail\E/,split(/,/,$notify))) {
+                   $numperm = 1;
+               }
+           }
+           unless ($numperm) {
+               $numperm = &sendnotification($permemail,$user,$domain,$subject,0,
+                                            $text,$msgid);
+           }
+       }
+       if ($toperm) {
+           $permlogmsgstatus = '. Perm. email log status '.
+                         &Apache::lonnet::log($env{'user.domain'},$env{'user.name'},$env{'user.home'},
+                                              "Perm. e-mail count $numperm for $user at $domain");
+           if (ref($permresults) eq 'HASH') {
+               $permresults->{"$user:$domain"} = $numperm;
            }
        }
        &Apache::lonnet::log($env{'user.domain'},$env{'user.name'},
 			    $env{'user.home'},
-			    'Sending '.$msgid.' to '.$user.' at '.$domain.' with status: '.$status);
+			    'Sending '.$msgid.' to '.$user.' at '.$domain.' with status '.$status.
+                            $permlogmsgstatus);
    } else {
        $status='no_host';
    }
@@ -874,25 +919,26 @@ sub user_normal_msg_raw {
 
 sub user_normal_msg {
     my ($user,$domain,$subject,$message,$citation,$baseurl,$attachmenturl,
-	$toperm,$sentmessage,$symb,$restitle,$error,$nosentstore,$recipid)=@_;
+	$toperm,$sentmessage,$symb,$restitle,$error,$nosentstore,$recipid,
+        $permresults)=@_;
     my @status;
     my %userenv = &Apache::lonnet::get('environment',['msgforward'],
                                        $domain,$user);
     my $msgforward=$userenv{'msgforward'};
     if ($msgforward) {
-        foreach (split(/\,/,$msgforward)) {
-	    my ($forwuser,$forwdomain)=split(/\:/,$_);
+        foreach my $fwd (split(/\,/,$msgforward)) {
+	    my ($forwuser,$forwdomain)=split(/\:/,$fwd);
 	    push(@status,
 	        &user_normal_msg_raw($forwuser,$forwdomain,$subject,$message,
 				     $citation,$baseurl,$attachmenturl,$toperm,
 				     undef,undef,$sentmessage,undef,$symb,
-                                     $restitle,$error,$nosentstore,$recipid));
+                                     $restitle,$error,$nosentstore,$recipid,$permresults));
         }
     } else {
 	push(@status,&user_normal_msg_raw($user,$domain,$subject,$message,
 				     $citation,$baseurl,$attachmenturl,$toperm,
 				     undef,undef,$sentmessage,undef,$symb,
-                                     $restitle,$error,$nosentstore,$recipid));
+                                     $restitle,$error,$nosentstore,$recipid,$permresults));
     }
     if (wantarray) {
         return @status;
@@ -901,7 +947,9 @@ sub user_normal_msg {
 }
 
 sub process_sent_mail {
-    my ($msgsubj,$subj_prefix,$numsent,$stamp,$msgname,$msgdom,$msgcount,$context,$pid,$savemsg,$recusers,$recudoms,$baseurl,$attachmenturl,$symb,$error,$senderuname,$senderdom,$recipid) = @_;
+    my ($msgsubj,$subj_prefix,$numsent,$stamp,$msgname,$msgdom,$msgcount,
+        $context,$pid,$savemsg,$recusers,$recudoms,$baseurl,$attachmenturl,
+        $symb,$error,$senderuname,$senderdom,$recipid) = @_;
     my $sentsubj;
     if ($numsent > 1) {
         $sentsubj = $subj_prefix.' ('.$numsent.' sent) '.$msgsubj;
