@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
-# Navigate Maps Handler
+# Navigate Maps Display Handler
 #
-# $Id: lonnavdisplay.pm,v 1.13.4.4 2010/12/30 21:36:50 raeburn Exp $
+# $Id: lonnavdisplay.pm,v 1.22.4.6 2015/03/11 15:43:40 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -30,14 +30,14 @@
 package Apache::lonnavdisplay;
 
 use strict;
-use Apache::Constants qw(:common :http);
+use Apache::Constants qw(:common :http REDIRECT);
 use Apache::lonmenu();
 use Apache::loncommon();
 use Apache::lonnavmaps();
 use Apache::lonhtmlcommon();
 use Apache::lonnet;
 use Apache::lonlocal;
-use Time::HiRes qw( gettimeofday tv_interval );
+use Apache::londocs();
 
 sub handler {
     my $r = shift;
@@ -46,85 +46,40 @@ sub handler {
 
 sub real_handler {
     my $r = shift;
-    #my $t0=[&gettimeofday()];
     # Handle header-only request
     if ($r->header_only) {
-        if ($env{'browser.mathml'}) {
-            &Apache::loncommon::content_type($r,'text/xml');
-        } else {
-            &Apache::loncommon::content_type($r,'text/html');
-        }
-        $r->send_http_header;
-        return OK;
-    }
-
-    # Send header, don't cache this page
-    if ($env{'browser.mathml'}) {
-        &Apache::loncommon::content_type($r,'text/xml');
-    } else {
         &Apache::loncommon::content_type($r,'text/html');
-    }
-    &Apache::loncommon::no_cache($r);
-
-    my %toplinkitems=();
-    &Apache::lonnavmaps::add_linkitem(\%toplinkitems,'blank','',
-				      "Select Action");
-    if ($ENV{QUERY_STRING} eq 'collapseExternal') {
-	&Apache::lonnet::put('environment',{'remotenavmap' => 'off'});
-	&Apache::lonnet::appenv({'environment.remotenavmap' => 'off'});
-	my $menu=&Apache::lonmenu::reopenmenu();
-	my $navstatus=&Apache::lonmenu::get_nav_status();
-	if ($menu) {
-	    $menu=(<<MENU)
-             swmenu=$menu
-             swmenu.clearTimeout(swmenu.menucltim);
-	     $navstatus
-MENU
-        } else {
-	    my $nothing = &Apache::lonhtmlcommon::javascript_nothing();
-	    my $mainwindow='window.open('.$nothing.',"loncapaclient","",false);';
-	    $menu=(<<MENU)
-             swmenu=$mainwindow
-	     $navstatus
-MENU
-	}
         $r->send_http_header;
-	my $js =<<"ENDSUBM";
-	  <script type="text/javascript">
- 	    function submitthis() {
-		    $menu
-		    self.close();
-	    }
-	
-  	  </script>
-ENDSUBM
-        $r->print(&Apache::loncommon::start_page(undef,$js,
-						 {'only_body' => 1,
-						  'bgcolor'   => '#FFFFFF',
-						  'add_entries' => 
-						      {'onload' =>
-							   "submitthis()"}}).
-		  &Apache::loncommon::end_page());
-
         return OK;
     }
-    if ($ENV{QUERY_STRING} =~ /^launchExternal/) {
-	&Apache::lonnet::put('environment',{'remotenavmap' => 'on'});
-	&Apache::lonnet::appenv({'environment.remotenavmap' => 'on'});
- 	my $menu=&Apache::lonmenu::reopenmenu();
-	my $navstatus=&Apache::lonmenu::get_nav_status();
-	if ($menu) {
-	    $r->print(<<MENU);
-             <script type="text/javascript">
-             swmenu=$menu
-             swmenu.clearTimeout(swmenu.menucltim);
-	     $navstatus
-             </script>
-MENU
+
+    # Check for critical messages and redirect if present.
+    my ($redirect,$url) = &Apache::loncommon::critical_redirect(300);
+    if ($redirect) {
+        &Apache::loncommon::content_type($r,'text/html');
+        $r->header_out(Location => $url);
+        return REDIRECT;
+    }
+
+    # Check if course needs to be re-initialized
+    my $loncaparev = $r->dir_config('lonVersion');
+    my ($result,@reinit) = &Apache::loncommon::needs_coursereinit($loncaparev);
+
+    if ($result eq 'switch') {
+        &Apache::loncommon::content_type($r,'text/html');
+        $r->send_http_header;
+        $r->print(&Apache::loncommon::check_release_result(@reinit));
+        return OK;
+    } elsif ($result eq 'update') {
+        my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+        my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+        my ($furl,$ferr) = &Apache::lonuserstate::readmap("$cdom/$cnum");
+        if ($ferr) {
+            my $requrl = $r->uri;
+            $env{'user.error.msg'}="$requrl:bre:0:0:Course not initialized";
+            $env{'user.reinit'} = 1;
+            return HTTP_NOT_ACCEPTABLE;
         }
-   }
-    if ($ENV{QUERY_STRING} eq 'turningOffExternal') {
-	$env{'environment.remotenavmap'}='off';
     }
 
     # Create the nav map
@@ -136,54 +91,33 @@ MENU
         $env{'user.reinit'} = 1;
         return HTTP_NOT_ACCEPTABLE;
     }
+
+    # Send header, don't cache this page
+    &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
 
 # ------------------------------------------------------------ Get query string
-    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['register','sort','showOnlyHomework','postsymb']);
+    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['sort','showOnlyHomework','postsymb','register']);
     
 # ----------------------------------------------------- Force menu registration
-    my $body_only='';
-    my $js;
-    if ($env{'environment.remotenavmap'} eq 'on') {
-	$js='<script type="text/javascript">
-                function collapse() {
-                   this.document.location="/adm/navmaps?collapseExternal";
-                }
-             </script>';
-	$body_only=1;
-    }
-
     # Header
     my $course_type = &Apache::loncommon::course_type();
-    my ($title,$breadcrumb_text,$start_page,$args);
-    if ($course_type eq 'Community') {
-        $title = 'Community Contents';
-        $breadcrumb_text = &mt('Community Contents');
-    } else {
-        $title = 'Course Contents';
-        $breadcrumb_text = &mt('Course Contents');
-    }
+    my $title = $course_type . ' Contents';
+    my ($start_page,$args);
     if ($env{'form.register'}) {
         $args = {'force_register' => $env{'form.register'}};
-        if ($body_only && $env{'environment.remote'} eq 'on') {
-            $args->{only_body} = 1;
-        }
-        $start_page = &Apache::loncommon::start_page($title,$js,$args);
+        $start_page = &Apache::loncommon::start_page($title,undef,$args);
     } else {
         my $brcrum = [{href => '/adm/navmaps',
-                       text => $breadcrumb_text,
+                       text => &mt($course_type . ' Contents'),
                        no_mt => 1},
                      ];
-        unless ($env{'environment.remotenavmap'} eq 'on') {
-            $args = {'bread_crumbs' => $brcrum};
-        }
-        if ($body_only && $env{'environment.remote'} eq 'on') {
-            $args->{only_body} = 1;
-        }
-        $start_page = &Apache::loncommon::start_page($title,$js,$args);
+        $args = {'bread_crumbs' => $brcrum};
+        $start_page = &Apache::loncommon::start_page($title,undef,$args);
     }
     $r->print($start_page.
               '<script type="text/javascript">window.focus();</script>');
+    &startContentScreen($r,'navmaps');
     $r->rflush();
 
     # Check that it's defined
@@ -195,21 +129,32 @@ MENU
         return OK;
     }
 
-    # See if there's only one map in the top-level, if we don't
-    # already have a filter... if so, automatically display it
-    # (older code; should use retrieveResources)
-    if ($ENV{QUERY_STRING} !~ /filter/) {
-        my $iterator = $navmap->getIterator(undef, undef, undef, 0);
-        my $curRes;
-        my $sequenceCount = 0;
-        my $sequenceId;
-        while ($curRes = $iterator->next()) {
-            if (ref($curRes) && $curRes->is_sequence()) {
+    my %toplinkitems=();
+    my @resources = $navmap->retrieveResources();
+    my $sequenceCount = 0;
+    my $problemCount = 0;
+    my $notaprobCount = 0;
+    my $sequenceId;
+    my $notools;
+    foreach my $curRes (@resources) {
+        if (ref($curRes)) {
+            if ($curRes->is_sequence()) {
                 $sequenceCount++;
                 $sequenceId = $curRes->map_pc();
+            } elsif ($curRes->is_problem()) {
+                $problemCount ++;
+            } else {
+                $notaprobCount ++;
             }
         }
-        
+    }
+    if (($sequenceCount == 1) && (!$problemCount) && ($notaprobCount <= 1)) {
+        $notools = 1;
+    }
+
+    # If there's only one map in the top-level and we don't
+    # already have a filter, automatically display it
+    if ($ENV{QUERY_STRING} !~ /filter/) {
         if ($sequenceCount == 1) {
             # The automatic iterator creation in the render call 
             # will pick this up. We know the condition because
@@ -219,27 +164,6 @@ MENU
         }
     }
 
-    if ($ENV{QUERY_STRING} eq 'launchExternal') {
-	$r->print('
-          <form name="returnwin" action="/adm/flip?postdata=navlaunch%3a" 
-                method="post" target="loncapaclient">
-          </form>');
-	$r->print('
-          <script type="text/javascript">
-              this.document.returnwin.submit();
-          </script>');
-    }
-
-    if ($env{'environment.remotenavmap'} ne 'on') {
-	$r->print(&launch_win('link','yes',\%toplinkitems));
-    } 
-    if ($env{'environment.remotenavmap'} eq 'on') {
-	&Apache::lonnavmaps::add_linkitem(\%toplinkitems,'closenav',
-					  'collapse()',
-					  "Close navigation window");
-    } 
-
-
     # Check to see if the student is jumping to next open, do-able problem
     if ($ENV{QUERY_STRING} =~ /^jumpToFirstHomework/) {
         # Find the next homework problem that they can do.
@@ -286,9 +210,11 @@ MENU
         if ($env{'form.register'}) {
             $link .= '&amp;register='.$env{'form.register'}; 
         }
-	&Apache::lonnavmaps::add_linkitem(\%toplinkitems,'firsthomework',
-					  'location.href="'.$link.'"',
-					  "Show my first due problem");
+        unless ($notools) {
+	    &Apache::lonnavmaps::add_linkitem(\%toplinkitems,'firsthomework',
+					      'location.href="'.$link.'"',
+					      "Show my first due problem");
+        }
     }
 
     my $suppressEmptySequences = 0;
@@ -309,7 +235,7 @@ MENU
         }
 	&Apache::lonnavmaps::add_linkitem(\%toplinkitems,'everything',
 					  'location.href="'.$link.'"',
-					  "Show everything");
+					  'Show everything');
         $r->print("<span class=\"LC_info\">".&mt("Uncompleted Problems")."</span>");
         $env{'form.filter'} = '';
         $env{'form.condition'} = 1;
@@ -319,25 +245,31 @@ MENU
         if ($env{'form.register'}) {
             $link .= '&amp;register='.$env{'form.register'};
         }
-	&Apache::lonnavmaps::add_linkitem(\%toplinkitems,'uncompleted',
-					  'location.href="'.$link.'"',
-					  "Show only uncompleted problems");
+        unless ($notools) {
+	    &Apache::lonnavmaps::add_linkitem(\%toplinkitems,'uncompleted',
+					      'location.href="'.$link.'"',
+					      'Show only uncompleted problems');
+        }
     }
 
     my %selected=($env{'form.sort'} => ' selected="selected"');
-    my $sort_html=('<form name="sortForm" action="">
+    my $sort_html;
+    unless ($notools) {
+        $sort_html=(
+              '<form name="sortForm" action="">
                  <span class="LC_nobreak">
                     <input type="hidden" name="showOnlyHomework" value="'.$env{'form.showOnlyHomework'}.'" />
                     '.&mt('Sort by:').'
                     <select name="sort" onchange="document.sortForm.submit()">
                        <option value="default"'.$selected{'default'}.'>'.&mt('Default').'</option>
                        <option value="title"'.$selected{'title'}.'>'.&mt('Title').'</option>
-                       <option value="duedate"'.$selected{'duedate'}.'>'.&mt('Duedate').'</option>
+                       <option value="duedate"'.$selected{'duedate'}.'>'.&mt('Due Date').'</option>
                        <option value="discussion"'.$selected{'discussion'}.'>'.&mt('Has New Discussion').'</option>
-                     </select>
+                    </select>
                     <input type="hidden" name="register" value="'.$env{'form.register'}.'" />
                  </span>
                </form>');
+    }
     # renderer call
     my $renderArgs = { 'cols' => [0,1,2,3],
 		       'sort' => $env{'form.sort'},
@@ -350,7 +282,9 @@ MENU
 		       'sort_html'=> $sort_html,
                        'r' => $r,
                        'caller' => 'navmapsdisplay',
-                       'linkitems' => \%toplinkitems};
+                       'linkitems' => \%toplinkitems,
+                       'notools' => $notools};
+                      
     my $render = &Apache::lonnavmaps::render($renderArgs);
 
     # If no resources were printed, print a reassuring message so the
@@ -362,42 +296,36 @@ MENU
             $r->print("<p><span class=\"LC_info\">".&mt("This course is empty.")."</span></p>");
         }
     }
-    #my $td=&tv_interval($t0);
-    #$r->print("<br />$td");
-
+    &endContentScreen($r);
     $r->print(&Apache::loncommon::end_page());
     $r->rflush();
 
     return OK;
 }
 
-sub launch_win {
-    my ($mode,$script,$toplinkitems,$firsttime)=@_;
-    my $result;
-    if ($script ne 'no') {
-	$result.='<script type="text/javascript">';
+sub startContentScreen {
+    my ($r,$mode)=@_;
+
+    $r->print("\n".'<ul class="LC_TabContentBigger" id="mainnav">'."\n");
+    $r->print('<li'.(($mode eq 'navmaps')?' class="active"':'').'><a href="/adm/navmaps"><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.&mt('Main Content').'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</b></a></li>'."\n");
+    my $allowed = &Apache::lonnet::allowed('mdc',$env{'request.course.id'});
+    my ($suppcount,$errors);
+    unless ($allowed) {
+        my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+        my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+        ($suppcount,$errors) = &Apache::lonnet::get_numsuppfiles($cnum,$cdom);
     }
-    if ($firsttime) {
-	$result.='function launch_navmapwin() {
-                 newWindow=open(\'/adm/navmaps?launchExternalRoles\',\'loncapanav\',\'width=400,height=600,scrollbars=1\');
-               }';
-    } else {
-	$result.='function launch_navmapwin() {
-                 newWindow=open(\'/adm/navmaps?launchExternal\',\'loncapanav\',\'width=400,height=600,scrollbars=1\');
-               }';
+    if ($allowed || $suppcount) {
+        $r->print('<li '.(($mode eq 'supplemental')?' class="active"':'').'><a href="/adm/supplemental"><b>'.&mt('Supplemental Content').'</b></a></li>');
     }
-    if ($mode eq 'now') {
-	$result.="\nlaunch_navmapwin();\n";
-    }
-    if ($script ne 'no') {
-	$result.='</script>';
-    }
-    if ($mode eq 'link') {
-	&Apache::lonnavmaps::add_linkitem($toplinkitems,'launchnav',
-					  'launch_navmapwin()',
-					  "Launch navigation window");
-    }
-    return $result;
+    $r->print('<li'.(($mode eq 'coursesearch')?' class="active"':'').'><a href="/adm/searchcourse"><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'.&mt('Content Search').'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</b></a></li>'."\n");
+    $r->print("\n".'</ul>'."\n");
+    $r->print('<div class="LC_Box" style="clear:both;margin:0;"><div id="maincoursedoc" style="margin:0 0;padding:0 0;"><div class="LC_ContentBox" id="mainCourseDocuments" style="display: block;">');
+}
+
+sub endContentScreen {
+   my ($r)=@_;
+   $r->print('</div></div></div>');
 }
 
 1;
