@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Sending messages
 #
-# $Id: lonnotify.pm,v 1.38 2010/11/10 14:44:50 bisitz Exp $
+# $Id: lonnotify.pm,v 1.41 2014/12/11 13:15:35 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -220,8 +220,8 @@ sub print_display_option_form {
     $output .= &Apache::lonhtmlcommon::row_title(&mt('Choose sender(s)'));
     my %personnel = &Apache::lonnet::get_domain_roles($cdom,\@roles);
     my @domcc = ();
-    foreach my $server (keys %personnel) {
-        foreach my $user (sort(keys %{$personnel{$server}})) {
+    foreach my $server (keys(%personnel)) {
+        foreach my $user (sort(keys(%{$personnel{$server}}))) {
             my ($trole,$uname,$udom,$runame,$rudom,$rsec) = split(/:/,$user);
             unless (grep/^$uname:$udom$/,@domcc) {
                 my %userinfo = &Apache::lonnet::get('environment',['lastname','firstname'],$udom,$uname);
@@ -286,11 +286,11 @@ ENDSCRIPT
     $output .= &Apache::loncommon::start_data_table();
     if ($msgcount > 0) {
         $output .= &Apache::loncommon::start_data_table_header_row().
-	    '<th><a href="javascript:changeSort(\'date\')">Date</a></th>'.
-	    '<th><a href="javascript:changeSort(\'subject\')">Subject</a></th>'.
-	    '<th><a href="javascript:changeSort(\'sender\')">Sender</a></th>'.
-	    '<th><a href="javascript:changeSort(\'message\')">Message</a></th>'.
-	    '<th><a href="javascript:changeSort(\'recipients\')">Recipients</a></th>'.
+	    '<th><a href="javascript:changeSort(\'date\')">'.&mt('Date').'</a></th>'.
+	    '<th><a href="javascript:changeSort(\'subject\')">'.&mt('Subject').'</a></th>'.
+	    '<th><a href="javascript:changeSort(\'sender\')">'.&mt('Sender').'</a></th>'.
+	    '<th><a href="javascript:changeSort(\'message\')">'.&mt('Message').'</a></th>'.
+	    '<th><a href="javascript:changeSort(\'recipients\')">'.&mt('Recipients').'</a></th>'.
 	    &Apache::loncommon::end_data_table_header_row();
 
         if (($env{'form.sortby'} eq 'date') || ($env{'form.sortby'} eq '') || (!defined($env{'form.sortby'})) || (($env{'form.sortby'} eq 'sender') && (@senders <= 1))) {
@@ -357,7 +357,7 @@ ENDSCRIPT
         }
     } else {
         $output .= &Apache::loncommon::start_data_table_empty_row().
-	    '<td>No mail sent matching supplied criteria</td>'.
+	    '<td>'.&mt('No mail sent matching supplied criteria').'</td>'.
 	    &Apache::loncommon::end_data_table_empty_row();
     }
     $output .= &Apache::loncommon::end_data_table();
@@ -645,7 +645,7 @@ function goBack(target) {
         $output .= &Apache::loncommon::end_data_table();
         if (@unmatched) {
             $output .= '<br /><br />'.&mt('Could not determine e-mail addresses for the following users:').'<ul>';
-            foreach my $username (sort @unmatched) {
+            foreach my $username (sort(@unmatched)) {
                 $output .= '<li>'.$username.'</li>';
             }
             $output .= '</ul>';
@@ -705,7 +705,7 @@ ENDSCRIPT
 	    '<th>'.&mt('Status').'</th>'.
 	    '<th>'.&mt('Subject').'</th>'.
 	    '<th>'.&mt('Message').'</th>'.
-	    '<th>'.&mt('Recipents').'</th>'.
+	    '<th>'.&mt('Recipients').'</th>'.
 	    &Apache::loncommon::end_data_table_header_row();
 	$output .= &Apache::loncommon::start_data_table_row().
 	    '<td valign="middle">'.&mt('Sent').'</td>'.
@@ -720,7 +720,7 @@ ENDSCRIPT
 	    &Apache::loncommon::end_data_table_row().
 	    &Apache::loncommon::end_data_table();
     } else {
-        $output .= 'No mail sent - no recipients identified'; 
+        $output .= &mt('No mail sent - no recipients identified'); 
     }
     $output .= '<br /><a href="/adm/notify">'.&mt('Send another e-mail').'</a>'."\n";
     $output .= '<input type="hidden" name="command" />'."\n".
