@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Pick a course
 #
-# $Id: lonpickcourse.pm,v 1.115 2014/05/30 01:50:05 raeburn Exp $
+# $Id: lonpickcourse.pm,v 1.116 2015/05/21 23:40:09 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -50,9 +50,10 @@ sub handler {
     &Apache::loncommon::get_unprocessed_cgi
         ($ENV{'QUERY_STRING'},['domainfilter','form','cnumelement',
 			       'cdomelement','cnameelement','roleelement',
-                               'multiple','type','setroles','fixeddom','cloner']);
+                               'multiple','type','setroles','fixeddom','cloner',
+                               'crscode','crsdom']);
     my ($type,$title,$jscript,$multelement,$multiple,$roleelement,$typeelement,
-        $lastaction,$autosubmit,$submitopener,$cloneruname,$clonerudom);
+        $lastaction,$autosubmit,$submitopener,$cloneruname,$clonerudom,$crscode,$crsdom);
 
     # Get course type - Course or Community.
     $type = $env{'form.type'};
@@ -82,12 +83,16 @@ sub handler {
     # if called when a DC is creating a course for another user.
     if ($env{'form.form'} eq 'ccrs') {
         ($cloneruname,$clonerudom) = ($env{'form.cloner'} =~ /^($match_username):($match_domain)$/);
+        $crscode = $env{'form.crscode'};
+        $crsdom = $env{'request.role.domain'};
     }
 
     # if called when requesting a course
     if ($env{'form.form'} eq 'requestcrs') {
         $cloneruname = $env{'user.name'};
         $clonerudom =  $env{'user.domain'};
+        $crscode = $env{'form.crscode'};
+        $crsdom = $env{'form.crsdom'};
     }
 
     my $onlyown = 0;
@@ -220,6 +225,12 @@ sub handler {
             if ($coord_cloneable) {
                 $clonetext .= '<input type="hidden" name="cc_clone" value="'.$coord_cloneable.'" />';
             }
+            if ($crscode ne '') {
+                $clonetext .= '<input type="hidden" name="crscode" value="'.$crscode.'" />';
+            }
+            if ($crsdom ne '') {
+                $clonetext .= '<input type="hidden" name="crsdom" value="'.$crsdom.'" />';
+            }
         }
         $r->print(&Apache::loncommon::build_filters($filterlist,$type,$roleelement,$multelement,
                                                     $filter,$action,\$numtitles,undef,$cloneruname,
@@ -247,7 +258,8 @@ sub handler {
             my $srchdom = $filter->{'domainfilter'};
             %courses = &Apache::loncommon::search_courses($srchdom,$type,$filter,$numtitles,
                                                           $cloneruname,$clonerudom,$domcloner,
-                                                          \@codetitles,$env{'form.cc_clone'});
+                                                          \@codetitles,$env{'form.cc_clone'},
+                                                          $crsdom,$crscode);
         } else {
             $r->print('<br />');
             my %coursehash = &Apache::loncommon::findallcourses();
@@ -262,7 +274,7 @@ sub handler {
                            '<b>'.$filter->{'persondomfilter'}.'</b>').'</span>');
         } else {
             &display_matched_courses($r,$type,$multiple,$action,$showroles,$cloneruname,
-                                     $clonerudom,%courses);
+                                     $clonerudom,$crsdom,$crscode,%courses);
         }
     }
     $r->print(&Apache::loncommon::end_page());
@@ -371,7 +383,7 @@ function setDefaultCredits() {
 }
 
 sub display_matched_courses {
-    my ($r,$type,$multiple,$action,$showroles,$cloneruname,$clonerudom,%courses) = @_;
+    my ($r,$type,$multiple,$action,$showroles,$cloneruname,$clonerudom,$crsdom,$crscode,%courses) = @_;
     if ($env{'form.form'} eq 'portform') {
         $action = '/adm/portfolio';
     }
@@ -464,6 +476,7 @@ sub display_matched_courses {
             map {$cc_cloneable{$_} = 1;} split('&',$coord_cloneable);
         }
     }
+    my (%gotdomdefaults,%gotcodedefaults);
     foreach my $description (sort { lc($a) cmp lc($b) } (keys(%by_descrip))) {
         foreach my $course (@{$by_descrip{$description}}) {
             $r->print(&Apache::loncommon::start_data_table_row());
@@ -489,7 +502,7 @@ sub display_matched_courses {
                     }
                     unless ($canclone) {
                         my $cloners = $courses{$course}{'cloners'};
-                        if ($cloners ne '') { 
+                        if ($cloners ne '') {
                             my @cloneable = split(',',$cloners);
                             if (grep(/^\*$/,@cloneable)) {
                                 $canclone = 1;
@@ -500,6 +513,80 @@ sub display_matched_courses {
                             if (grep(/^\Q$cloneruname\E:\Q$clonerudom\E$/,@cloneable)) {
                                 $canclone = 1;
                             }
+                            unless ($canclone) {
+                                if (($instcode) && ($crscode) && ($cdom eq $crsdom)) {
+                                    foreach my $cloner (@cloneable) {
+                                        if (($cloner ne '*') && ($cloner !~ /^\*\:$match_domain$/) &&
+                                            ($cloner !~ /^$match_username\:$match_domain$/) && ($cloner ne '')) {
+                                            if ($cloner =~ /\=/) {
+                                                my (%codedefaults,@code_order);
+                                                if (ref($gotcodedefaults{$cdom}) eq 'HASH') {
+                                                    if (ref($gotcodedefaults{$cdom}{'defaults'}) eq 'HASH') {
+                                                        %codedefaults = %{$gotcodedefaults{$cdom}{'defaults'}};
+                                                    }
+                                                    if (ref($gotcodedefaults{$cdom}{'order'}) eq 'ARRAY') {
+                                                        @code_order = @{$gotcodedefaults{$cdom}{'order'}};
+                                                    }
+                                                } else {
+                                                    &Apache::lonnet::auto_instcode_defaults($cdom,
+                                                                                            \%codedefaults,
+                                                                                            \@code_order);
+                                                    $gotcodedefaults{$cdom}{'defaults'} = \%codedefaults;
+                                                    $gotcodedefaults{$cdom}{'order'} = \@code_order;
+                                                }
+                                                if (@code_order > 0) {
+                                                    if (&Apache::lonnet::check_instcode_cloning(\%codedefaults,\@code_order,
+                                                                                                $cloner,$instcode,$crscode)) {
+                                                        $canclone = 1;
+                                                        last;
+                                                    }
+                                                }
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                        } else {
+                            my %domdefs;
+                            if (ref($gotdomdefaults{$cdom}) eq 'HASH') {
+                                %domdefs = %{$gotdomdefaults{$cdom}};
+                            } else {
+                                %domdefs = &Apache::lonnet::get_domain_defaults($cdom);
+                                $gotdomdefaults{$cdom} = \%domdefs;
+                            }
+                            if ($domdefs{'canclone'}) {
+                                unless ($domdefs{'canclone'} eq 'none') {
+                                    if ($domdefs{'canclone'} eq 'domain') {
+                                        if ($clonerudom eq $cdom) {
+                                            $canclone = 1;
+                                        }
+                                    } elsif (($instcode) && ($crscode) &&
+                                             ($cdom eq $crsdom)) {
+                                        my (%codedefaults,@code_order); 
+                                        if (ref($gotcodedefaults{$cdom}) eq 'HASH') {
+                                            if (ref($gotcodedefaults{$cdom}{'defaults'}) eq 'HASH') {
+                                                %codedefaults = %{$gotcodedefaults{$cdom}{'defaults'}};
+                                            }
+                                            if (ref($gotcodedefaults{$cdom}{'order'}) eq 'ARRAY') {
+                                                @code_order = @{$gotcodedefaults{$cdom}{'order'}};
+                                            }
+                                        } else {
+                                            &Apache::lonnet::auto_instcode_defaults($cdom,
+                                                                                    \%codedefaults,
+                                                                                    \@code_order);
+                                            $gotcodedefaults{$cdom}{'defaults'} = \%codedefaults;
+                                            $gotcodedefaults{$cdom}{'order'} = \@code_order;
+                                        }
+                                        if (@code_order > 0) {
+                                            if (&Apache::lonnet::default_instcode_cloning($cdom,$domdefs{'canclone'},
+                                                                                          $instcode,$crscode,\%codedefaults,
+                                                                                          \@code_order)) {
+                                                $canclone = 1;
+                                            }
+                                        }
+                                    }
+                                }
+                            }
                         }
                     }
                 }
@@ -849,9 +936,9 @@ javascript code for reporting selected sections (as a string of comma separated
 
 =item *
 X<display_matched_courses()>
-B<display_matched_courses($r,$type,$multiple,$action,$showroles,$cloneruname,$clonerudom,%courses)>:
+B<display_matched_courses($r,$type,$multiple,$action,$showroles,$cloneruname,$clonerudom,$crsdom,$crscode,%courses)>:
 
-Input: 7 - request object, course type, multiple (0 or 1), form action, whether to show roles (for course personnel filter), username of new course owner, domain of new course owner, hash of courses.
+Input: 8 - request object, course type, multiple (0 or 1), form action, whether to show roles (for course personnel filter), username of new course owner, domain of new course owner, domain of new course, institutional code of new course, hash of courses.
 
 Output: 0
 
