@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Preferences
 #
-# $Id: lonpreferences.pm,v 1.196.4.15 2014/06/09 22:39:01 raeburn Exp $
+# $Id: lonpreferences.pm,v 1.196.4.16 2015/03/12 02:58:33 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -789,7 +789,7 @@ sub lockwarning {
     my $textbottom=&mt('Changing roles or logging out may result in data corruption.');
     my ($num,%which)=&Apache::lonnet::get_locks();
     my $which='';
-    foreach my $id (keys %which) {
+    foreach my $id (keys(%which)) {
        $which.='<li>'.$which{$id}.'</li>';
     }
     my $change=&mt('Override');
@@ -1220,7 +1220,7 @@ sub verify_and_change_colors {
     );
 
     my $message='';
-    foreach my $item (keys %colortypes) {
+    foreach my $item (keys(%colortypes)) {
         my $color=$env{'form.'.$item};
         if (!($color =~ /^#/)) {
             $color = '#' . $color;
