@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Quick Student Grades Display
 #
-# $Id: lonquickgrades.pm,v 1.49.6.1 2014/05/22 12:44:24 raeburn Exp $
+# $Id: lonquickgrades.pm,v 1.49.6.2 2015/03/13 22:40:35 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -44,14 +44,6 @@ sub handler {
 sub real_handler {
     my $r = shift;
 
-    # Check for critical messages and redirect if present.
-    my ($redirect,$url) = &Apache::loncommon::critical_redirect(300);
-    if ($redirect) {
-        &Apache::loncommon::content_type($r,'text/html');
-        $r->header_out(Location => $url);
-        return REDIRECT;
-    }
-
     &Apache::loncommon::get_unprocessed_cgi($ENV{QUERY_STRING});
 
     # Handle header-only request
@@ -65,6 +57,37 @@ sub real_handler {
         return OK;
     }
 
+    unless (&Apache::lonnet::allowed('mgr')) {
+        # Check for critical messages and redirect if present.
+        my ($redirect,$url) = &Apache::loncommon::critical_redirect(300);
+        if ($redirect) {
+            &Apache::loncommon::content_type($r,'text/html');
+            $r->header_out(Location => $url);
+            return REDIRECT;
+        }
+
+        # Check if course needs to be re-initialized
+        my $loncaparev = $r->dir_config('lonVersion');
+        my ($result,@reinit) = &Apache::loncommon::needs_coursereinit($loncaparev);
+
+        if ($result eq 'switch') {
+            &Apache::loncommon::content_type($r,'text/html');
+            $r->send_http_header;
+            $r->print(&Apache::loncommon::check_release_result(@reinit));
+            return OK;
+        } elsif ($result eq 'update') {
+            my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+            my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+            my ($furl,$ferr) = &Apache::lonuserstate::readmap("$cdom/$cnum");
+            if ($ferr) {
+                my $requrl = $r->uri;
+                $env{'user.error.msg'}="$requrl:bre:0:0:Course not initialized";
+                $env{'user.reinit'} = 1;
+                return HTTP_NOT_ACCEPTABLE;
+            }
+        }
+    }
+
     # Send header, don't cache this page
     &Apache::loncommon::no_cache($r);
     $r->send_http_header;
