@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # RSS Feeder
 #
-# $Id: lonrss.pm,v 1.49 2009/11/04 15:38:27 raeburn Exp $
+# $Id: lonrss.pm,v 1.53 2013/07/15 14:32:44 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -88,7 +88,7 @@ sub changefeeddisplay {
 }
 
 sub advertisefeeds {
-    my ($uname,$udom,$edit)=@_;
+    my ($uname,$udom,$edit,$count,$hidden)=@_;
     my $feeds='';
     my %feednames=&Apache::lonnet::dump('nohist_all_rss_feeds',$udom,$uname);
     my $mode='public';
@@ -100,7 +100,20 @@ sub advertisefeeds {
 	next if ($feed =~/^\s*$/    ||
 		 $feed =~ /^error:/ ||
 		 $feed =~ /^feed_display_option_/);
-
+        if ($feednames{'feed_display_option_'.$feed} eq 'hidden') {
+            if (ref($hidden)) {
+                $$hidden ++;
+            }
+            if (ref($count)) {
+                unless ($edit) {
+                    next;
+                }
+            }
+        } else {
+            if (ref($count)) {
+                $$count ++;
+            }
+        }
 	my $feedurl= $server.'/public/'.$udom.'/'.$uname.'/'.$feed.'.rss';
 	my $htmlurl= $server.'/'.$mode.'/'.$udom.'/'.$uname.'/'.$feed.'_rss.html';
 	if ($feednames{'feed_display_option_'.$feed} eq 'hidden') {
@@ -288,7 +301,9 @@ sub handler {
 	$edit=1;
 	$html=1;
     }
-    if  (($mode eq 'adm') && (&Apache::lonnet::allowed('mdc',$env{'request.course.id'}))) {
+    if  (($mode eq 'adm') && (&Apache::lonnet::allowed('mdc',$env{'request.course.id'}))
+        && ($uname eq $env{'course.'.$env{'request.course.id'}.'.num'} &&
+            $udom eq $env{'course.'.$env{'request.course.id'}.'.domain'})) {
 	$edit=1;
 	$html=1;
     }
@@ -396,7 +411,7 @@ sub handler {
 # Add a new feed
         if (($html) && ($edit)) {
 	    $r->print('<h4>' . &mt('New RSS Feed or Blog'). '</h4>');
-	    $r->print('<form method="post" name="makenewfeed">');
+	    $r->print('<form method="post" name="makenewfeed" action="">');
             $r->print(&mt('Name').": <input type='text' size='40' name='namenewblog' />");
 	    $r->print('<input type="submit" value="'.&mt('New Feed').'" />');
 	    $r->print('</form>');
@@ -498,7 +513,7 @@ sub handler {
 		      ($html?'<hr /><h3>':'<title>').
 		      &mt('LON-CAPA Feed "[_1]" for [_2]',$displayfeedname,$name).
 		      ($displayoption eq 'hidden'?' ('.&mt('Hidden').')':'').
-		      ($html?'</h3>'.($edit?'<form method="post" name="lonhomework" enctype="multipart/form-data"><br />'.
+		      ($html?'</h3>'.($edit?'<form method="post" name="lonhomework" enctype="multipart/form-data" action=""><br />'.
 				      &mt('Name of this Feed').
 				      ': <input type="text" size="50" name="newblogname" value="'.
 				      $displayfeedname.'" />':'').'<ul>':'</title>'));
@@ -537,20 +552,20 @@ sub handler {
 <li>
 <label><input name='$id\_modified' type='checkbox' value="modified" /> $lt{'store'}</label>
 &nbsp;&nbsp;
-<label><input name='$id\_status' type="radio" value="public" $status{'public'} onClick="changed(this.form,'$id');" /> $lt{'public'}</label>
+<label><input name='$id\_status' type="radio" value="public" $status{'public'} onclick="changed(this.form,'$id');" /> $lt{'public'}</label>
 &nbsp;&nbsp;
-<label><input name='$id\_status' type="radio" value="private" $status{'private'} onClick="changed(this.form,'$id');" /> $lt{'private'}</label>
+<label><input name='$id\_status' type="radio" value="private" $status{'private'} onclick="changed(this.form,'$id');" /> $lt{'private'}</label>
 &nbsp;&nbsp;
-<label><input name='$id\_status' type="radio" value="hidden" $status{'hidden'} onClick="changed(this.form,'$id');" /> $lt{'hidden'}</label>
+<label><input name='$id\_status' type="radio" value="hidden" $status{'hidden'} onclick="changed(this.form,'$id');" /> $lt{'hidden'}</label>
 &nbsp;&nbsp;
-<label><input name='$id\_status' type="radio" value="deleted" onClick="changed(this.form,'$id');" /> $lt{'delete'}</label>
+<label><input name='$id\_status' type="radio" value="deleted" onclick="changed(this.form,'$id');" /> $lt{'delete'}</label>
 <br />
 $lt{'title'}:
-<input name='$id\_title' type='text' size='60' value='$newsfeed{$id.'_title'}' onChange="changed(this.form,'$id');" /><br />
+<input name='$id\_title' type='text' size='60' value='$newsfeed{$id.'_title'}' onchange="changed(this.form,'$id');" /><br />
 $lt{'description'}:<br />
-<textarea name='$id\_description' rows="6" cols="80" onChange="changed(this.form,'$id');">$newsfeed{$id.'_description'}</textarea><br />
+<textarea name='$id\_description' rows="6" cols="80" onchange="changed(this.form,'$id');">$newsfeed{$id.'_description'}</textarea><br />
 $lt{'link'}:
-<input name='$id\_link' type='text' size='60' value='$newsfeed{$id.'_link'}' onChange="changed(this.form,'$id');" /><br />
+<input name='$id\_link' type='text' size='60' value='$newsfeed{$id.'_link'}' onchange="changed(this.form,'$id');" /><br />
 $lt{'enc'} -
 $uploadlink
 <hr /></li>
@@ -599,9 +614,10 @@ ENDEDIT
 			      ($displayoption eq 'hidden'?'<input type="submit" name="advertisethisblog" value="'.&mt('Advertise this Feed').'" />':
 			       '<input type="submit" name="hidethisblog" value="'.&mt('Hide this Feed').'" />'));
 		}
+                $r->print('</form>');
 	    }
 	} # was a real display feedname
-	$r->print(($html?'</form>'.&Apache::loncommon::end_page():'</channel></rss>'."\n"));
+	$r->print(($html?&Apache::loncommon::end_page():'</channel></rss>'."\n"));
     } # a real user
     return OK;
 } # end handler
