@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Simple Page Editor
 #
-# $Id: lonsimplepage.pm,v 1.98 2011/05/03 21:38:40 www Exp $
+# $Id: lonsimplepage.pm,v 1.103 2013/09/17 15:04:30 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -33,8 +33,7 @@ use Apache::Constants qw(:common);
 use Apache::loncommon;
 use Apache::lontemplate;
 use Apache::lonnet;
-use Apache::lontexconvert;
-use Apache::lonfeedback;
+use Apache::lonhtmlgateway;
 use Apache::lonlocal;
 use Apache::lonprintout;
 use Apache::lonxml;
@@ -44,14 +43,18 @@ use HTML::Entities();
 use LONCAPA;
 
 sub get_db_name {
-    my ($url) = @_;
-    my ($udom,$uname,$marker)=(split(m{/},$url))[2,3,4];
-    my $dom = $env{'course.'.$env{'request.course.id'}.'.domain'};
-    my $crs = $env{'course.'.$env{'request.course.id'}.'.num'};
-
+    my ($url,$marker,$cdom,$cnum) = @_;
+    my ($udom,$uname,$timemark)=(split(m{/},$url))[2,3,4];
+    if ($marker eq '') {
+        $marker = $timemark;
+    }
+    if (($cdom eq '') || ($cnum eq '')) {
+        $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+        $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+    }
     my $db_name;
 
-    if ($dom && $crs && ($udom eq $dom) && ($uname eq $crs)) {
+    if ($cdom && $cnum && ($udom eq $cdom) && ($uname eq $cnum)) {
         $marker =~ s/\W//g;
         $db_name = 'grppage_'.$marker;
     } else {
@@ -59,7 +62,6 @@ sub get_db_name {
         $db_name = 'smppage_'.$marker;
     }
     return if (!defined($marker));
-
     return $db_name;
 }
 
@@ -84,16 +86,17 @@ sub handler {
         return OK;
     }
 
-    my $db_name = &get_db_name($r->uri);
+    my $marker = (split(m{/},$r->uri))[4];
+    my $db_name = &get_db_name($r->uri,$marker);
 
     my $dom = $env{'course.'.$env{'request.course.id'}.'.domain'};
     my $crs = $env{'course.'.$env{'request.course.id'}.'.num'};
-    my ($group,$group_desc,$group_home_view,$group_home_edit,
-        $group_view_perm,$group_edit_perm);
+    my ($group,$group_desc);
+
     my %curr_group = ();
     my %groupinfo = ();
     if ($db_name =~ /^grppage_/) {
-        $group = (split(m{/},$r->uri))[4];
+        $group = $marker;
         $group =~ s/\W//g;
         my %curr_groups = &Apache::longroup::coursegroups($dom,$crs,$group);
         if (!%curr_groups) {
@@ -122,13 +125,14 @@ sub handler {
 
 # ------------------------------------------------------------ Get query string
     &Apache::loncommon::get_unprocessed_cgi
-                        ($ENV{'QUERY_STRING'},['forcestudent','forceedit',
+                        ($ENV{'QUERY_STRING'},['forceedit','todocs',
                                                'register','ref']);
 # --------------------------------------------------------------- Force Student
-    my $forcestudent='';
-    if ($env{'form.forcestudent'} || $target eq 'tex' ) { $forcestudent='student'; };
-    my $forceedit='';
-    if ($env{'form.forceedit'}) { $forceedit='edit'; }
+    my ($forceedit,$forcestudent);
+    $forceedit = $env{'form.forceedit'};
+    if (!$forceedit) {
+        $forcestudent=1;
+    }
 
     my $refarg;
     if ($env{'form.ref'}) {
@@ -138,23 +142,16 @@ sub handler {
     my %syllabus=&Apache::lonnet::dump($db_name,$dom,$crs);
 
 # --------------------------------------- There is such a user, get environment
-
-    if ($target ne 'tex') {
-        my $title = 'Simple Course Page';
-        if ($group ne '') {
-            $title = 'Simple Group Page';
+    my ($registered,$group_view_perm,$group_edit_perm,$group_home_view,
+        $group_home_edit,$has_group_access);
+    my $brcrum = [];
+    if ($group eq '') {
+        $registered = $env{'form.register'};
+    } else {
+        unless ($env{'form.ref'} eq 'grouplist') {
+            $registered = $env{'form.register'};
         }
-        my $start_page =
-        &Apache::loncommon::start_page($title,undef,
-                       {'function'       => $forcestudent,
-                        'domain'         => $dom,
-                        'force_register' =>
-                        $env{'form.register'},});
-        $r->print($start_page);
-    }
-
-    if ($group ne '') {
-        my $group_view_perm =
+        $group_view_perm =
                &Apache::lonnet::allowed('vcg',$env{'request.course.id'}.
                ($env{'request.course.sec'}?'/'.$env{'request.course.sec'}:''));
         $group_edit_perm =
@@ -166,10 +163,28 @@ sub handler {
                                          $env{'request.course.id'}.'/'.$group);
         if ($group_view_perm || $group_edit_perm || $group_home_view ||
             $group_home_edit || &Apache::longroup::check_group_access($group)) {
+            $has_group_access = 1;
             if (($env{'form.ref'} eq 'grouplist') && ($target ne 'tex') &&
-                (!$env{'form.register'})) {
-                $r->print(&grouppage_breadcrumbs($dom,$crs,$group,$group_desc));
+                (!$registered)) {
+                $brcrum = &grouppage_breadcrumbs($dom,$crs,$group,$group_desc);
             }
+        }
+    }
+
+    if ($target ne 'tex') {
+        my $title = ($group eq '')? 'Simple Course Page':'Simple Group Page';
+        my $start_page =
+        &Apache::loncommon::start_page($title,undef,
+                       {'domain'         => $dom,
+                        'group'          => $group,
+                        'bread_crumbs'   => $brcrum,
+                        'force_register' => $registered,
+                       }); 
+        $r->print($start_page);
+    }
+
+    if ($group ne '') {
+        if ($has_group_access) {
             if ((!$group_home_edit) && (!$group_home_view) &&
                 (!$group_view_perm) && (!$group_edit_perm)) {
                 &display_group_links($r,$target,$group,'view',$refarg,%groupinfo);
@@ -210,10 +225,6 @@ sub handler {
     } else {
         $allowed=&Apache::lonnet::allowed('mdc',$env{'request.course.id'});
     }
-    my $privileged=$allowed;
-    if (($syllabus{'uploaded.lastmodified'}) && (!$forceedit)) {
-        $forcestudent='student';
-    }
 
     if ($forcestudent or $target eq 'tex') { $allowed=0; }
 
@@ -225,10 +236,12 @@ sub handler {
             }
             if ($group ne '') {
                 $syllabus{'uploaded.photourl'}=&Apache::lonnet::userfileupload(
-                                              'uploaddoc','coursedoc',"grouppage/$group");
+                                              'uploaddoc','coursedoc',
+                                              "grouppage/$group");
             } else {
                 $syllabus{'uploaded.photourl'}=
-                &Apache::lonnet::userfileupload('uploaddoc','coursedoc','simplepage');
+                &Apache::lonnet::userfileupload('uploaddoc','coursedoc',
+                                                "simplepage/$marker");
             }
         }
         $syllabus{'uploaded.lastmodified'}=time;
@@ -245,60 +258,14 @@ sub handler {
         foreach my $syl_field (keys(%syllabusfields)) {
             my $field=$env{'form.'.$syl_field};
             chomp($field);
-            $field=~s/\s+$//s;
-            $field=~s/^\s+//s;
-            $field=~s/\<br\s*\/*\>$//s;
-            $field=&Apache::lonfeedback::clear_out_html($field,1);
+            my $gateway = Apache::lonhtmlgateway->new();
+            $field = $gateway->process_incoming_html($field,1);
             $syllabus{$syl_field}=$field;
         }
         $syllabus{'uploaded.lastmodified'}=time;
         &Apache::lonnet::put($db_name,\%syllabus,$dom,$crs);
     }
 
-#---Print Function
-    if ($target ne 'tex') {
-        if ($allowed || $privileged) {
-            my $functions=&Apache::lonhtmlcommon::start_funclist();
-            my $url = $r->uri;
-            if ($group ne '') {
-                my $navmap=Apache::lonnavmaps::navmap->new();
-                if (ref($navmap)) {
-                    $url = &Apache::longroup::get_group_link($dom,$crs,$group,$navmap);
-                }
-            }
-            if ($allowed) {
-                $url .= (($url=~/\?/)?'&amp;':'?').'forcestudent=1';
-                $functions.=&Apache::lonhtmlcommon::add_item_funclist(
-                                '<a href="'.$url.'">'
-                               .&mt('Show Student View').'</a>'
-                               .&Apache::loncommon::help_open_topic(
-                                    'Uploaded_Templates_PublicView'));
-            } elsif ($privileged) {
-                my $edittext = &mt('Edit');
-                $url .= (($url=~/\?/)?'&amp;':'?').'forceedit=edit'.$refarg;
-                if ($group ne '') {
-                    $edittext = &mt('Edit Group Homepage');
-                }
-                $functions.=&Apache::lonhtmlcommon::add_item_funclist(
-                                '<a href="'.$url.'">'
-                               .$edittext.'</a>');
-                if ($group ne '') {
-                    if ($group_edit_perm) {
-                        $functions.=&Apache::lonhtmlcommon::add_item_funclist(
-                                        '<a href="/adm/coursegroups'
-                                       .'?action=modify&amp;refpage=grouplist'
-                                       .'&amp;state=pick_task&amp;groupname='
-                                       .$group.'">'
-                                       .&mt('Edit Group Settings')
-                                       .'</a>');
-                       }
-                    }
-            }
-            $functions.=&Apache::lonhtmlcommon::end_funclist();
-            $r->print(&Apache::loncommon::head_subbox($functions));
-        }
-    }
-
 #---Print help Text
     if($target ne 'tex'){
         if ($allowed) {
@@ -323,7 +290,8 @@ sub handler {
         if ($allowed) {
             if ($env{'form.grade_target'} ne 'tex') {
                 #editbox for title
-                $r->print('<form method="post" action="" enctype="multipart/form-data">');
+                $r->print('<form method="post" action="" enctype="multipart/form-data">'."\n".
+                          '<input type="hidden" register="'.$registered.'" />');
 				&Apache::lontemplate::print_start_template($r,&mt('Title'),'LC_Box');
 				$r->print($titletext);
 				$r->print("<br /><div>");
@@ -493,17 +461,13 @@ sub display_group_links {
 sub grouppage_breadcrumbs {
     my ($cdom,$cnum,$group,$description) = @_;
     &Apache::lonhtmlcommon::clear_breadcrumbs();
-    &Apache::lonhtmlcommon::add_breadcrumb
-        ({href=>"/adm/coursegroups",
-          text=>"Groups",
-          title=>"Display Groups"},
-        {href=>"/adm/$cdom/$cnum/$group/smppg?ref=grouplist",
-          text=>&mt('Group:')." $description",
-          title=>"Go to group's home page"},
-        );
-    my $output .= &Apache::lonhtmlcommon::breadcrumbs(&mt('Group page - [_1]',
-                                                           $description));
-    return $output;
+    return [{href=>"/adm/coursegroups",
+             text=>"Groups",
+             title=>"Display Groups"},
+            {href=>"/adm/$cdom/$cnum/$group/smppg?ref=grouplist",
+             text=>&mt('Group:')." $description",
+             title=>"Go to group's home page"},
+           ];
 }
 
 1;
