@@ -1,6 +1,6 @@
 # The LearningOnline Network with CAPA
 #
-# $Id: lonstatistics.pm,v 1.156.2.4 2014/02/26 01:19:55 raeburn Exp $
+# $Id: lonstatistics.pm,v 1.156.2.5 2015/03/12 03:08:29 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -867,7 +867,7 @@ ENDSTYLE
         #
         # Clean out the caches
         if (exists($env{'form.ClearCache'})) {
-            &Apache::loncoursedata::delete_caches($env{'requres.course.id'});
+            &Apache::loncoursedata::delete_caches($env{'request.course.id'});
         }
         #
         my $GoToPage = $env{'form.reportSelected'};
