@@ -1,5 +1,5 @@
 #
-# $Id: lonwhatsnew.pm,v 1.101.2.1 2011/05/12 16:53:02 raeburn Exp $
+# $Id: lonwhatsnew.pm,v 1.105.2.10 2015/03/12 03:28:51 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -72,6 +72,7 @@ sub handler {
 			    ['mdc','versionchanges',0],
                             ['vcl','newroles',1],
                             ['vcl','oldroles',1],
+                            ['whn','crslogin',1],
 			    ) {
 	my ($perm,$key,$check_section) = @{ $perm_check };
 	my $scope = $env{'request.course.id'};
@@ -143,6 +144,13 @@ sub handler {
         $r->print(&Apache::lonhtmlcommon::breadcrumbs
             ("What's New?",#'Course_Action_Items_Intervals'
              ));
+    } elsif ($command eq 'chgcrslogininterval' && $checkallowed{'crslogin'}) {
+        &Apache::lonhtmlcommon::add_breadcrumb
+            ({href=>'/adm/whatsnew?command=chgcrslogininterval&amp;refpage='.$refpage,
+              text=>"Change interval"});
+        $r->print(&Apache::lonhtmlcommon::breadcrumbs
+            ("What's New?",#'Course_Action_Items_Intervals'
+             ));
     } else {
         $r->print(&Apache::lonhtmlcommon::breadcrumbs
             ("What's New?",#'Course_Action_Items_Display'
@@ -188,10 +196,17 @@ sub display_main_box {
                    604800 => 'roles which expired since last week',
                     86400 => 'roles which expired since yesterday',
                    );
+    my %crslogins = (
+                       -1 => 'last logins for anyone who has ever logged in',
+                  2592000 => 'last logins for users in last 30 days',
+                   604800 => 'last logins for users in last 7 days',
+                    86400 => 'last logins for users in last 24 hours',
+                   );
     my %interval_titles = (
         versions => \%versions,
         newroles => \%newroles,
         oldroles => \%oldroles,
+        crslogin => \%crslogins,
     );
     my %initpage = &Apache::lonlocal::texthash (
                      firstres => "first resource in the $lctype",
@@ -220,6 +235,9 @@ sub display_main_box {
     } elsif (($command eq 'chgoldroleinterval')
              && $checkallowed->{'oldroles'}) {
         &display_interval_config($r,$refpage,\%interval_titles,'oldroles');
+    } elsif (($command eq 'chgcrslogininterval')
+             && $checkallowed->{'crslogin'}) {
+        &display_interval_config($r,$refpage,\%interval_titles,'crslogin');
     } else {
         &display_actions_box($r,$command,$refpage,\%threshold_titles,
                         \%interval_titles,\%initpage,$cdom,$crs,$checkallowed);
@@ -245,9 +263,11 @@ sub display_header {
     
     my $scripttag;
     unless ($command eq 'chgthreshold' || $command eq 'chginterval' || 
-            $command eq 'chgoldroleinterval' || $command eq 'chgnewroleinterval') {
+            $command eq 'chgoldroleinterval' ||
+            $command eq 'chgnewroleinterval' || $command eq 'chgcrslogininterval') {
        $scripttag = <<"END";
 <script type="text/javascript">
+// <![CDATA[
 function change_display(caller,change) {
     caller.value = change;
     document.visible.submit();
@@ -264,6 +284,57 @@ END
         }
         $scripttag.='document.visible.submit();
 }
+
+function thresholdreset() {
+    document.visible.command.value="reset";
+    document.visible.submit();
+}
+
+function togglelogins() {
+    var total = document.visible.logincount.value;
+    var sumrow = document.visible.loginrow.value;
+    if (total == 0) {
+        return;
+    }
+    var showlogindetails = 0;
+    for (var i=0; i<document.visible.logindetails.length; i++) {
+        if (document.visible.logindetails[i].checked) {
+            showlogindetails = document.visible.logindetails[i].value;
+        }
+    }
+    var detval = "none";
+    var sumval = "";
+    if (showlogindetails == 1) {
+        detval = "";
+        sumval = "none";
+    }
+    for (var j=0; j<total; j++) {
+        var counter = j+1;
+        var itemid = "logindet_"+counter;
+        personele = document.getElementById(itemid);
+        if (personele != null) {
+            personele.style.display = detval;
+        }
+    }
+    var detheaderele = document.getElementById("logintitledet");
+    if (detheaderele != null) {
+        detheaderele.style.display = detval;
+    }
+    for (var k=0; k<sumrow; k++) {
+        var counter = k+1;
+        var itemid = "loginsum_"+counter;
+        logincatele = document.getElementById(itemid);
+        if (logincatele != null) {
+            logincatele.style.display = sumval;
+        }
+    }
+    var sumheaderele = document.getElementById("logintitlesum");
+    if (sumheaderele != null) {
+        sumheaderele.style.display = sumval;
+    }
+    return;
+}
+// ]]>
 </script>
 ';
     }
@@ -315,6 +386,8 @@ sub display_actions_box {
     my %expired;
     my $activecount;
     my %activated;
+    my %loggedin;
+    my $logincount;
     my %res_title = ();
     my %show = ();
     my $needitems = 0;
@@ -445,21 +518,46 @@ sub display_actions_box {
         $headings{'newroles'} = &mt('Roles for which access to '.$lctype.' has become available since yesterday');
     }
 
-    my $now = time;
+    $timediff{'crslogin'} = $display_settings{$cid.':crslogininterval'};
+    unless (defined($timediff{'crslogin'})) { $timediff{'crslogin'} = 604800; }
+    $interval{'crslogin'} = $interval_titles->{'crslogin'}->{$timediff{'crslogin'}};
+
+    if ($timediff{'crslogin'} == -1) {
+        $headings{'crslogin'} = &mt('Last login for anyone who has ever logged in');
+    } elsif ($timediff{'crslogin'} == 2592000) {
+        $headings{'crslogin'} = &mt('Last login for users in last 30 days');
+    } elsif ($timediff{'crslogin'} == 604800) {
+        $headings{'crslogin'} = &mt('Last login for users in last 7 days');
+    } elsif ($timediff{'crslogin'} == 86400) {
+        $headings{'crslogin'} = &mt('Last login for users in last 24 hours');
+    }
+
+    my ($now,$starttime,$activatedstart,$expiredstart,$crsloginstart);
+    $now = time;
+
     if ($timediff{'versions'} == -1) {
-        $timediff{'versions'} = time;
+        $starttime = 0;
+    } else {
+        $starttime = $now - $timediff{'versions'};
     }
-    my $starttime = $now - $timediff{'versions'};
 
     if ($timediff{'newroles'} == -1) {
-        $timediff{'newroles'} = time;
+        $activatedstart = 0;
+    } else {
+        $activatedstart = $now - $timediff{'newroles'};
     }
-    my $activatedstart = $now - $timediff{'newroles'};
 
     if ($timediff{'oldroles'} == -1) {
-        $timediff{'oldroles'} = time;
+        $expiredstart = 0;
+    } else {
+        $expiredstart = $now - $timediff{'oldroles'};
+    }
+
+    if ($timediff{'crslogin'} == -1) {
+        $crsloginstart = 0;
+    } else {
+        $crsloginstart = $now - $timediff{'crslogin'};
     }
-    my $expiredstart = $now - $timediff{'oldroles'};
 
     my $countunread = $display_settings{$cid.':countunread'};
     unless (defined($countunread)) {
@@ -474,7 +572,7 @@ sub display_actions_box {
 	    $threshold{'av_attempts'},$threshold{'degdiff'},
 	    '<br />',$threshold{'numstudents'});
 
-    my @actionorder = ('handgrading','haserrors','abovethreshold','versionchanges','coursediscussion','coursenormalmail','coursecritmail','newroles','oldroles');
+    my @actionorder = ('handgrading','haserrors','abovethreshold','versionchanges','coursediscussion','coursenormalmail','coursecritmail','newroles','oldroles','crslogin');
 
     foreach my $key (keys(%{$checkallowed})) {
 	if ($key =~ /_section$/) { next; }
@@ -500,6 +598,10 @@ sub display_actions_box {
     if ($needitems) {
         $itemserror = &getitems(\%unread,\%ungraded,\%bombed,\%triggered,\%changed,\@newdiscussions,\@tograde,\@bombs,\@warnings,\%threshold,$cdom,$crs,\%res_title,\%show,$starttime,$countunread);
     }
+    my $classlist;
+    if ($show{'oldroles'} || $show{'newroles'} || $show{'crslogin'}) {
+        $classlist = &Apache::loncoursedata::get_classlist();
+    }
     if ($show{'coursenormalmail'}) {
         $msgcount = &getnormalmail(\@newmsgs);
     }
@@ -507,10 +609,13 @@ sub display_actions_box {
         $critmsgcount = &getcritmail(\@critmsgs);
     }
     if ($show{'oldroles'}) {
-        $expirecount = &getexpired(\%expired,$expiredstart,'previous');
+        $expirecount = &getexpired(\%expired,$expiredstart,'previous',$classlist);
     }
     if ($show{'newroles'}) {
-        $activecount = &getactivated(\%activated,$activatedstart,'active');
+        $activecount = &getactivated(\%activated,$activatedstart,'active',$classlist);
+    }
+    if ($show{'crslogin'}) {
+        $logincount = &getloggedin($cdom,$crs,\%loggedin,$crsloginstart);
     }
     $r->print(qq|<a href="javascript:changeAll('hide');">$lt{'hial'}</a>
      &nbsp;&nbsp;<a href="javascript:changeAll('show');">$lt{'shal'}</a>
@@ -522,7 +627,7 @@ sub display_actions_box {
         }
     }
 
-    $r->print('<input type="hidden" name="refpage" value="'.$refpage.'" /></form><table class="LC_double_column"><tr><td class="LC_left_col">');
+    $r->print('<input type="hidden" name="refpage" value="'.$refpage.'" /><table class="LC_double_column"><tr><td class="LC_left_col">');
 
     my $displayed = 0;
     my $totalboxes = 0;
@@ -540,7 +645,7 @@ sub display_actions_box {
             if ($displayed == $halfway) {
                 $r->print('</td><td>&nbsp;</td><td class="LC_right_col" >');
             }
-            &display_launcher($r,$actionitem,$refpage,$checkallowed,\%show,\%headings,\%res_title,\@tograde,\%ungraded,\@bombs,\%bombed,\%changed,\@warnings,\%triggered,\@newdiscussions,\%unread,$msgcount,\@newmsgs,$critmsgcount,\@critmsgs,\%interval,$countunread,\%expired,$expirecount,\%activated,$activecount,$crstype,$itemserror);
+            &display_launcher($r,$actionitem,$refpage,$checkallowed,\%show,\%headings,\%res_title,\@tograde,\%ungraded,\@bombs,\%bombed,\%changed,\@warnings,\%triggered,\@newdiscussions,\%unread,$msgcount,\@newmsgs,$critmsgcount,\@critmsgs,\%interval,$countunread,\%expired,$expirecount,\%activated,$activecount,$crstype,$itemserror,\%loggedin,$logincount,$classlist);
             $displayed ++; 
         }
     }
@@ -548,6 +653,7 @@ sub display_actions_box {
       </td>
     </tr>
    </table>
+   </form>
 ');
 }
 
@@ -586,7 +692,7 @@ sub display_threshold_config {
     foreach my $type (@thresholditems) {
         my $parameter = $env{'request.course.id'}.':threshold_'.$type;
 # onchange is javascript to automatically check the 'Set' button.
-        my $onchange = 'onFocus="javascript:window.document.forms'.
+        my $onchange = 'onfocus="javascript:window.document.forms'.
               "['thresholdform'].elements['".$parameter."_setparmval']".
               '.checked=true;"';
         $r->print(&Apache::loncommon::start_data_table_row()."\n".
@@ -628,6 +734,8 @@ sub display_interval_config {
         $r->print('<br />'.&mt('Choose the time window to use to display roles for which access to the '.$lctype.' expired.').'<br />');
     } elsif ($context eq 'newroles') {
         $r->print('<br />'.&mt('Choose the time window to use to display roles for which access to the '.$lctype.' became available.').'<br />');
+    } elsif ($context eq 'crslogin') {
+        $r->print('<br />'.&mt('Choose the time window to use to display the last login by a user in the '.$lctype).'<br />');
     } else {
         $r->print('<br />'.&mt('Choose the time window to use to display resources in the '.$lctype.' with version changes.').'<br />');
     }
@@ -691,7 +799,7 @@ function toggle_countunread(choice) {
              .'<br />'
              .&mt("This can increase the time taken to gather data for the [_1]What's New Page[_2] by a few seconds.",'<i>','</i>')
              .'&nbsp;&nbsp;'
-             .&mt('Currently set to [_1].','<b>'.$current.'</b>.')
+             .&mt('Currently set to [_1].','<b>'.$current.'</b>')
     );
     $r->print('<br /><br />
 <form method="post" name="discussionswitch" action="/adm/whatsnew">
@@ -793,10 +901,10 @@ sub display_launcher {
         $tograde,$ungraded,$bombs,$bombed,$changed,$warnings,$triggered,
         $newdiscussions,$unread,$msgcount,$newmsgs,$critmsgcount,$critmsgs,
         $interval,$countunread,$expired,$expirecount,$activated,$activecount,
-        $crstype,$itemserror) = @_;
+        $crstype,$itemserror,$loggedin,$logincount,$classlist) = @_;
 
     if ($$checkallowed{$action}) {
-        &start_box($r,$show,$headings,$action,$refpage,$action);
+        &start_box($r,$show,$headings,$action,$refpage);
         if ($$show{$action}) {
             if ($action eq 'handgrading') {    # UNGRADED ITEMS
                 &display_handgrade($r,$tograde,$ungraded,$itemserror);
@@ -816,10 +924,13 @@ sub display_launcher {
                 &display_coursecritmail($r,$critmsgcount,$critmsgs);
             } elsif ($action eq 'newroles') { # ACTIVATED ROLES
                 &display_rolechanges($r,$activecount,$activated,$interval->{'newroles'},
-                                     $crstype);
+                                     $crstype,$classlist);
             } elsif ($action eq 'oldroles') { # EXPIRED ROLES
                 &display_rolechanges($r,$expirecount,$expired,$interval->{'oldroles'},
-                                     $crstype);
+                                     $crstype,$classlist);
+            } elsif ($action eq 'crslogin') { #LAST LOGIN
+                &display_crslogins($r,$logincount,$loggedin,$interval->{'crslogin'},
+                                   $crstype,$classlist);
             }
         }
         &end_box($r);
@@ -999,6 +1110,7 @@ sub check_thresholds {
             $av_attempts = $attempts/$users;
             $av_attempts = sprintf("%.2f",$av_attempts);
         }
+        &Apache::lonnet::statslog($symb,$part,$users,$av_attempts,$degdiff);
         if ((($degdiff ne '' && $degdiff >= $$threshold{'degdiff'}) || ($av_attempts ne '' && $av_attempts >= $$threshold{'av_attempts'})) && ($users >= $$threshold{'numstudents'})) {
             $stats{$part}{degdiff} = $degdiff;
             $stats{$part}{attempts} = $av_attempts;
@@ -1052,7 +1164,7 @@ sub get_curr_thresholds {
     my $thresholdcount = 0;
     my ($tmp) = %thresholdsettings;
     unless ($tmp =~ /^(con_lost|error|no_such_host)/i) {
-        foreach my $item (keys %{$threshold}) { 
+        foreach my $item (keys(%{$threshold})) { 
             if (exists($thresholdsettings{$cid.':threshold_'.$item})) {
                 $$threshold{$item} = 
                              $thresholdsettings{$cid.':threshold_'.$item};
@@ -1067,7 +1179,7 @@ sub get_curr_thresholds {
                                               $cdom,$crs,'internal.threshold');
     my ($temp) = %coursesettings;
     unless ($temp =~ /^(con_lost|error|no_such_host)/i) {  
-        foreach my $item (keys %{$threshold}) {
+        foreach my $item (keys(%{$threshold})) {
             unless (exists($thresholdsettings{$cid.':threshold_'.$item})) {
                 if (exists($coursesettings{'internal.threshold_'.$item})) {
                     $$threshold{$item} = 
@@ -1134,9 +1246,13 @@ sub process_reset {
 sub process_update {
     my ($uname,$udom,$threshold_titles) = @_;
     my $setoutput = '<b>'.&mt('Changes to threshold(s) for problem tracking:').'</b><br/><br />';
-    foreach (keys %env) {
-        next if ($_!~/^form\.(.+)\_setparmval$/);
-        my $name  = $1;
+    foreach my $key (keys(%env)) {
+        my $name;
+        if ($key =~/^form\.(.+)\_setparmval$/) {
+            $name  = $1;
+        } else {
+            next;
+        }
         my $value = $env{'form.'.$name.'_value'};
         if ($name && defined($value) && ($value ne '')) {
             my $put_result = &Apache::lonnet::put('nohist_whatsnew',
@@ -1219,13 +1335,13 @@ sub getcritmail {
 }
 
 sub getexpired {
-    my ($rolechgs,$rolechgtime,$status) = @_;
-    my $expirecount = &getrolechanges($rolechgs,$rolechgtime,$status);
+    my ($rolechgs,$rolechgtime,$status,$classlist) = @_;
+    my $expirecount = &getrolechanges($rolechgs,$rolechgtime,$status,$classlist);
     return $expirecount;
 }
 
 sub getactivated {
-    my ($rolechgs,$rolechgtime,$status) = @_;
+    my ($rolechgs,$rolechgtime,$status,$classlist) = @_;
     my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
     my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
     my $now = time();
@@ -1237,28 +1353,26 @@ sub getactivated {
     my (%stucounted,%advcounted);
     my $activatedcount = 0;
     if (keys(%changes) > 0) {
-        foreach my $chg (sort { $b <=> $a } (keys(%changes))) {
+        foreach my $chg (keys(%changes)) {
             if (ref($changes{$chg}) eq 'HASH') {
-                my $timestamp = $changes{$chg}{'exe_time'};
-                if ($timestamp) {
-                    if ($rolechgtime > 0) {
-                        if ($timestamp < $rolechgtime) {
-                            last;
-                        }
-                    }
+                next if ($changes{$chg}{'delflag'});
+                if ($rolechgtime > 0) {
+                    next if ($changes{$chg}{'exe_time'} < $rolechgtime);
+                }
+                if ($changes{$chg}{'exe_time'}) {
+                    my $timestamp = $changes{$chg}{'exe_time'};
                     if (ref($changes{$chg}{'logentry'}) eq 'HASH') {
-                        next if ($changes{$chg}{'delflag'});
-                        my $start = $changes{$chg}{'logentry'}{'start'};
                         my $end = $changes{$chg}{'logentry'}{'end'};
-                        my $section = $changes{$chg}{'logentry'}{'section'};
-                        my $role = $changes{$chg}{'logentry'}{'role'};
-                        my $uname = $changes{$chg}{'uname'};
-                        my $udom = $changes{$chg}{'udom'};
                         next if ($end && $end <= $now);
+                        my $start = $changes{$chg}{'logentry'}{'start'};
+                        next if ($start >= $timestamp);
+                        my $section = $changes{$chg}{'logentry'}{'section'};
                         if (($viewablesec ne '') && ($section ne '')) {
                             next if ($viewablesec ne $section);
                         }
-                        next if ($start >= $timestamp);
+                        my $role = $changes{$chg}{'logentry'}{'role'};
+                        my $uname = $changes{$chg}{'uname'};
+                        my $udom = $changes{$chg}{'udom'};
                         if ($role eq 'st') {
                             $stucounted{$uname.':'.$udom.':'.$section} = $start.':'.$end;
                         } else {
@@ -1278,12 +1392,13 @@ sub getactivated {
             }
         }
     }
-    $activatedcount += &getrolechanges($rolechgs,$rolechgtime,$status,\%stucounted,\%advcounted);
+    $activatedcount += &getrolechanges($rolechgs,$rolechgtime,$status,$classlist,\%stucounted,
+                                       \%advcounted);
     return $activatedcount;
 }
 
 sub getrolechanges {
-    my ($rolechgs,$rolechgtime,$status,$stucountref,$advcountref) = @_;
+    my ($rolechgs,$rolechgtime,$status,$classlist,$stucountref,$advcountref) = @_;
     my (%stucounted,%advcounted);
     if (ref($stucountref) eq 'HASH') {
         %stucounted = %{$stucountref};
@@ -1301,63 +1416,64 @@ sub getrolechanges {
     my ($permission,$allowed) =
         &Apache::lonuserutils::get_permission($context);
     my $viewablesec = &Apache::lonuserutils::viewable_section($permission);
-    my $classlist = &Apache::loncoursedata::get_classlist();
     my $secidx = &Apache::loncoursedata::CL_SECTION();
     my $startidx = &Apache::loncoursedata::CL_START();
     my $endidx = &Apache::loncoursedata::CL_END();
     my $rolechgcount = 0;
-    foreach my $key (keys(%{$classlist})) {
-        my ($userstatus,$eventtime);   
-        my $student = $classlist->{$key};
-        if (ref($student) eq 'ARRAY') {
-            my $start = $student->[$startidx];
-            my $end = $student->[$endidx];
-            my $sec = $student->[$secidx]; 
-            my ($stuname,$studom) = split(/:/,$key);
-            if ($status eq 'active') {
-                if (exists($stucounted{$key.':'.$sec})) {
-                    next;
+    if (ref($classlist) eq 'HASH') {
+        foreach my $key (keys(%{$classlist})) {
+            my ($userstatus,$eventtime);   
+            my $student = $classlist->{$key};
+            if (ref($student) eq 'ARRAY') {
+                my $start = $student->[$startidx];
+                my $end = $student->[$endidx];
+                my $sec = $student->[$secidx]; 
+                my ($stuname,$studom) = split(/:/,$key);
+                if ($status eq 'active') {
+                    if (exists($stucounted{$key.':'.$sec})) {
+                        next;
+                    }
                 }
-            }
-            if (($end == 0) || ($end > $start)) {
-                if ($start <= $now) {
-                    if ($end && $end < $now) {
-                        if ($rolechgtime > 0) {
-                            if ($end > $rolechgtime) {
+                if (($end == 0) || ($end > $start)) {
+                    if ($start <= $now) {
+                        if ($end && $end < $now) {
+                            if ($rolechgtime > 0) {
+                                if ($end > $rolechgtime) {
+                                    $userstatus = 'previous';
+                                }
+                            } else {
                                 $userstatus = 'previous';
                             }
                         } else {
-                            $userstatus = 'previous';
-                        }
-                    } else {
-                        if ($rolechgtime > 0) {
-                            if ($start >= $rolechgtime) {
+                            if ($rolechgtime > 0) {
+                                if ($start >= $rolechgtime) {
+                                    $userstatus = 'active';
+                                }
+                            } else {
                                 $userstatus = 'active';
                             }
-                        } else {
-                            $userstatus = 'active';
                         }
                     }
                 }
+                next if ($userstatus ne $status);
+                if ($status eq 'active') {
+                    $eventtime = $start;
+                } else {
+                    $eventtime = $end;
+                }
+                if (($viewablesec ne '') && ($sec ne '')) {
+                    next if ($viewablesec ne $sec);
+                }
+                my %chginfo = (
+                                'section' => $sec,
+                                'uname'   => $stuname,
+                                'udom'    => $studom,
+                                'role'    => 'st', 
+                                'status'  => $userstatus,
+                              );
+                $rolechgcount ++;
+                push(@{$rolechgs->{$eventtime}},\%chginfo);
             }
-            next if ($userstatus ne $status);
-            if ($status eq 'active') {
-                $eventtime = $start;
-            } else {
-                $eventtime = $end;
-            }
-            if (($viewablesec ne '') && ($sec ne '')) {
-                next if ($viewablesec ne $sec);
-            }
-            my %chginfo = (
-                            'section' => $sec,
-                            'uname'   => $stuname,
-                            'udom'    => $studom,
-                            'role'    => 'st', 
-                            'status'  => $userstatus,
-                          );
-            $rolechgcount ++;
-            push (@{$rolechgs->{$eventtime}},\%chginfo);
         }
     }
     my %advrolehash = &Apache::lonnet::get_my_roles($cnum,$cdom,undef,
@@ -1418,6 +1534,39 @@ sub getrolechanges {
     return $rolechgcount;
 }
 
+sub getloggedin {
+    my ($cdom,$crs,$lastlogins,$starttime) = @_;
+    my $context = 'course';
+    my ($permission,$allowed) =
+        &Apache::lonuserutils::get_permission($context);
+    my $viewablesec = &Apache::lonuserutils::viewable_section($permission);
+    my %crslogins=&Apache::lonnet::dump('nohist_crslastlogin',$cdom,$crs);
+    my $logincount = 0;
+    my ($tmp) = keys(%crslogins);
+    unless ($tmp =~ /^(con_lost|error|no_such_host)/i) {
+        if (keys(%crslogins) > 0) {
+            foreach my $key (keys(%crslogins)) {
+                my ($uname,$udom,$section,$role) = split(/:/,$key);
+                my $eventtime = $crslogins{$key};
+                if ($eventtime > $starttime) {
+                    if (($viewablesec ne '') && ($section ne '')) {
+                        next if ($viewablesec ne $section);
+                    }
+                    my %chginfo = (
+                                   'section' => $section,
+                                   'uname'   => $uname,
+                                   'udom'    => $udom,
+                                   'role'    => $role,
+                    );
+                    $logincount ++;
+                    push (@{$lastlogins->{$eventtime}},\%chginfo);
+                }
+            }
+        }
+    }
+    return $logincount;
+}
+
 sub checkversions {
     my ($cdom,$crs,$navmap,$changed,$starttime) = @_;
     my %changes=&Apache::lonnet::dump('versionupdate',$cdom,$crs);
@@ -1477,9 +1626,9 @@ sub display_handgrade {
             my $linkurl=&Apache::lonnet::clutter($url);
             $linkurl .= '?symb='.&escape($res);
             if ($$ungraded{$res}{'enclink'}) {
-                $linkurl = 
+                $linkurl =
                     $$ungraded{$res}{'enclink'}.'?symb='.$$ungraded{$res}{'encsymb'};
-            } 
+            }
             $r->print('<tr'.$css_class.'><td><a href="'.$linkurl.'">'.$$ungraded{$res}{title}.'</a></td><td class="LC_right_item">'.$$ungraded{$res}{count}.'</td></tr>');
         }
     } elsif ($itemserror) {
@@ -1532,12 +1681,9 @@ sub display_abovethreshold {
     );
     if (@{$warnings} > 0) {
         @{$warnings} = sort { &cmp_title($a,$b,$res_title) } @{$warnings};
-        $r->print('<form name="reset_tracking" method="post" action="/adm/whatsnew">'.
-                ' <input type="hidden" name="command" value="reset" />'."\n".
-                ' <input type="hidden" name="refpage" value="'.$refpage.'" />'.
-                "\n");
         $r->print('<tr class="LC_info_row">'.
-		  '<td class="LC_left_item">'.$lt{'reso'}.'</td>'.
+		  '<td class="LC_left_item">'.$lt{'reso'}.
+                  '<input type="hidden" name="command" value="" /></td>'.
 		  '<td>'.$lt{'part'}.'</td><td>'.$lt{'nust'}.'</td>'.
 		  '<td>'.$lt{'avat'}.'</td><td>'.$lt{'dedi'}.'</td>'.
 		  '<td>'.$lt{'lare'}.'</td><td  class="LC_right_item">'.
@@ -1567,13 +1713,13 @@ sub display_abovethreshold {
             if (ref($$triggered{$res}{text}) eq 'ARRAY') {
                 if (@{$$triggered{$res}{text}} > 1) {
                     for (my $i=1; $i<@{$$triggered{$res}{text}}; $i++) {
-                        $r->print('<tr class="'.$css_class.'">'.
+                        $r->print('<tr '.$css_class.'>'.
                                   $$triggered{$res}{text}[$i].'</tr>');
                     }
                 }
             }
         }
-        $r->print('<tr class="LC_info_row"><td colspan="7" class="LC_right_item"><br /><input type="submit" name="counters" value="'.$lt{'rese'}.'" /></td></tr></form>');
+        $r->print('<tr class="LC_info_row"><td colspan="7" class="LC_right_item"><br /><input type="button" name="counters" value="'.$lt{'rese'}.'" onclick="javascript:thresholdreset();" /></td></tr>');
     } elsif ($itemserror) {
         $r->print('<tr class="LC_empty_row"><td>'.$itemserror.'</td></tr>');
     } else {
@@ -1619,7 +1765,7 @@ sub display_versionchanges {
 }
 
 sub display_rolechanges {
-    my ($r,$chgcount,$changed,$interval,$crstype) = @_;
+    my ($r,$chgcount,$changed,$interval,$crstype,$classlist) = @_;
     my $now = time();
     my %lt = &Apache::lonlocal::texthash(
         'user'  => 'User',
@@ -1638,6 +1784,7 @@ sub display_rolechanges {
         if (ref($changed) eq 'HASH') {
             my @changes = sort { $b <=> $a } (keys(%{$changed}));
             my $changenum = 0;
+            my $fullnameidx = &Apache::loncoursedata::CL_FULLNAME();
             foreach my $item (@changes) {
                 if (ref($changed->{$item}) eq 'ARRAY') {
                     foreach my $chg (@{$changed->{$item}}) {
@@ -1655,8 +1802,14 @@ sub display_rolechanges {
                             my $udom = $chg->{'udom'};
                             $changenum ++;
                             my $css_class = $changenum%2?' class="LC_odd_row"':'';
+                            my $fullname;
+                            if (ref($classlist->{$uname.':'.$udom}) eq 'ARRAY') {
+                                $fullname = $classlist->{$uname.':'.$udom}->[$fullnameidx];
+                            } else {
+                                $fullname = &Apache::loncommon::plainname($uname,$udom,'lastname');
+                            }
                             my $link = 
-                                &Apache::loncommon::aboutmewrapper(&Apache::loncommon::plainname($uname,$udom),$uname,$udom);
+                                &Apache::loncommon::aboutmewrapper($fullname,$uname,$udom);
                             $r->print('<tr'.$css_class.'>'.
                                       '<td>'.&Apache::lonlocal::locallocaltime($item).'</td>'.
                                       '<td>'.$link.'</td>'.
@@ -1674,6 +1827,99 @@ sub display_rolechanges {
     }
     return;
 }
+
+sub display_crslogins {
+    my ($r,$logincount,$loggedin,$interval,$crstype,$classlist) = @_;
+    return unless (ref($classlist) eq 'HASH');
+    my %lt = &Apache::lonlocal::texthash(
+        'user'   => 'User',
+        'role'   => 'Role',
+        'sec'    => 'Section',
+        'number' => 'Total number of logins',
+    );
+    if ($logincount) {
+
+        my $hdr = '<tr class="LC_info_row" style="display:none" id="logintitledet">'.
+                  '<td class="LC_left_item">'.$lt{'user'}.'</td>'.
+                  '<td class="LC_left_item">'.$lt{'role'}.'</td>'.
+                  '<td class="LC_left_item">'.$lt{'sec'}.'</td></tr>'.
+                  '<tr class="LC_info_row" id="logintitlesum">'.
+                  '<td class="LC_left_item">'.$lt{'number'}.'</td>'.
+                  '<td class="LC_left_item">'.$lt{'role'}.'</td>'.
+                  '<td class="LC_left_item">'.$lt{'sec'};
+        my (%bylastname,%counts);
+        if (ref($loggedin) eq 'HASH') {
+            my @logins = sort { $b <=> $a } (keys(%{$loggedin}));
+            my $numlogin = 0;
+            my $fullnameidx = &Apache::loncoursedata::CL_FULLNAME();
+            foreach my $item (@logins) {
+                if (ref($loggedin->{$item}) eq 'ARRAY') {
+                    foreach my $user (@{$loggedin->{$item}}) {
+                        if (ref($user) eq 'HASH') {
+                            my $section;
+                            my $role =
+                                &Apache::lonnet::plaintext($user->{'role'},$crstype);
+                            my $status = &mt($user->{'status'});
+                            if ($user->{'section'} eq '') {
+                                $section = &mt('none');
+                            } else {
+                                $section = $user->{'section'};
+                            }
+                            $counts{$user->{'role'}}{$section} ++;
+                            my $uname = $user->{'uname'};
+                            my $udom = $user->{'udom'};
+                            my $fullname;
+                            if (ref($classlist->{$uname.':'.$udom}) eq 'ARRAY') {
+                                $fullname = $classlist->{$uname.':'.$udom}->[$fullnameidx];
+                            } else {
+                                $fullname = &Apache::loncommon::plainname($uname,$udom,'lastname');
+                            }
+                            my $link =
+                                &Apache::loncommon::aboutmewrapper($fullname,$uname,$udom);
+                            push(@{$bylastname{$fullname}},
+                                                          '<td>'.$link.'</td>'.
+                                                          '<td>'.$role.'</td>'.
+                                                          '<td>'.$section.'</td>');
+                        }
+                    }
+                }
+            }
+            my $table;
+            foreach my $person (sort(keys(%bylastname))) {
+                if (ref($bylastname{$person}) eq 'ARRAY') {
+                    foreach my $item (@{$bylastname{$person}}) {
+                        $numlogin ++;
+                        my $css_class = $numlogin%2?' class="LC_odd_row"':'';
+                        $table .= '<tr'.$css_class.' style="display:none;"  id="logindet_'.$numlogin.'">'.$item.'</tr>';
+                    }
+                }
+            }
+            my $numrow = 0;
+            foreach my $role (sort(keys(%counts))) {
+                my $showrole = &Apache::lonnet::plaintext($role,$crstype);
+                if (ref($counts{$role}) eq 'HASH') {
+                    foreach my $sec (sort { $b <=> $a } (keys(%{$counts{$role}}))) {
+                        $numrow ++;
+                        my $css_class = $numrow%2?' class="LC_odd_row"':'';
+                        $table .= '<tr '.$css_class.' id="loginsum_'.$numrow.'">'.
+                                  '<td>'.$counts{$role}{$sec}.'</td>'.
+                                  '<td>'.$showrole.'</td>'.
+                                  '<td>'.$sec.'</td></tr>';
+                   }
+                }
+            }
+            $r->print($hdr.'<input type="hidden" name="logincount" value="'.$numlogin.
+                      '" /><input type="hidden" name="loginrow" value="'.$numrow.
+                      '" /></td></tr>'.$table);
+        }
+    } else {
+        $r->print('<tr class="LC_empty_row"><td>'.
+                  &mt('There are no '.$interval).
+                  '<input type="hidden" name="logincount" value="'.$logincount.
+                  '." /></td></tr>');
+    }
+    return;
+}
  
 sub display_coursediscussion {
     my ($r,$newdiscussions,$unread,$countunread,$res_title,$itemserror) = @_;
@@ -1704,7 +1950,7 @@ sub display_coursediscussion {
             my $forum_title = $$unread{$ressymb}{'title'};
             my $type = 'Resource';
             my $feedurl=&Apache::lonfeedback::get_feedurl($ressymb);
-            my $disclink = $feedurl.'?symb='.$$unread{$ressymb}{symb};
+            my $disclink = $feedurl.'?symb='.&escape($$unread{$ressymb}{symb});
             if ($feedurl =~ /bulletinboard/) {
                 $type = 'Discussion Board';
             }
@@ -1835,6 +2081,8 @@ sub store_interval_setting {
             $interval_settings{$cid.':oldroleinterval'} = $env{'form.interval'};
         } elsif ($context eq 'newroles') {
             $interval_settings{$cid.':newroleinterval'} = $env{'form.interval'};
+        } elsif ($context eq 'crslogin') {
+            $interval_settings{$cid.':crslogininterval'} = $env{'form.interval'};
         } else {
             $interval_settings{$cid.':interval'} = $env{'form.interval'};
         }
@@ -1989,7 +2237,15 @@ sub start_box {
                 <td class="LC_subheader" colspan="2"><a href="/adm/whatsnew?command=chgoldroleinterval&amp;refpage='.$refpage.'">'.$lt{'chin'}.'</a></td>
               </tr>');
          }
-     }
+    } elsif (($caller eq 'crslogin') && ($$show{$caller})) {
+        if ($$show{$caller}) {
+            $r->print('
+              <tr>
+                <td class="LC_subheader" align="left"><span class="LC_nobreak"><label><input type="radio" name="logindetails" value="0" checked="checked" onclick="javascript:togglelogins();" />&nbsp;'.&mt('Summary').'</label><label><input type="radio" name="logindetails" value="1" onclick="javascript:togglelogins();" />&nbsp;'.&mt('Details').'</label></span></td><td class="LC_subheader"><a href="/adm/whatsnew?command=chgcrslogininterval&amp;refpage='.$refpage.'">'.$lt{'chin'}.'</a></td>
+              </tr>');
+        }
+    }
+
     $r->print('
               <tr>
                <td colspan="2">
