@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Allow access to password changing via a token sent to user's e-mail. 
 #
-# $Id: resetpw.pm,v 1.23.2.2 2010/12/24 21:21:51 raeburn Exp $
+# $Id: resetpw.pm,v 1.36 2013/08/17 00:34:29 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -66,9 +66,13 @@ sub handler {
         return OK;
     }
     my $contact_name = &mt('LON-CAPA helpdesk');
-    my $contact_email =  $r->dir_config('lonSupportEMail');
+    my $origmail =  $r->dir_config('lonSupportEMail');
     my $server = $r->dir_config('lonHostID');
     my $defdom = &Apache::lonnet::default_login_domain();
+    my $contacts =
+        &Apache::loncommon::build_recipient_list(undef,'helpdeskmail',
+                                                 $defdom,$origmail);
+    my ($contact_email) = split(',',$contacts);
     my $handle = &Apache::lonnet::check_for_valid_session($r);
     my $lonidsdir=$r->dir_config('lonIDsDir');
     if ($handle ne '') {
@@ -87,13 +91,10 @@ sub handler {
     my $udom = $env{'form.udom'};
     my $token = $env{'form.token'};
     my $brcrum = [];
-    my $bread_crumbs_component = 'Forgotten Password';
     if ($token) {
         push (@{$brcrum},
             {href => '/adm/resetpw',
              text => 'Update Password'});
-
-        $bread_crumbs_component = 'Reset Password';
     } else {
         push (@{$brcrum},
             {href => '/adm/resetpw',
@@ -104,11 +105,9 @@ sub handler {
                  text => 'Result'});
         }
     }
-    my $args = {bread_crumbs           => $brcrum,
-                bread_crumbs_component => $bread_crumbs_component,
-               'no_inline_link'        => 1};
+    my $args = {bread_crumbs => $brcrum};
     $r->print(&Apache::loncommon::start_page('Reset password','',$args));
-    $r->print('<h3>'.&mt('Reset forgotten LON-CAPA password').'</h3>');
+    $r->print('<h2>'.&mt('Reset forgotten LON-CAPA password').'</h2>');
     my $output;
     if ($token) {
         $output = &reset_passwd($r,$token,$contact_name,$contact_email);
@@ -174,8 +173,8 @@ sub handler {
 sub get_uname {
     my ($defdom) = @_;
     my %lt = &Apache::lonlocal::texthash(
-                                         unam => 'username',
-                                         udom => 'domain',
+                                         unam => 'LON-CAPA username',
+                                         udom => 'LON-CAPA domain',
                                          uemail => 'E-mail address in LON-CAPA',
                                          proc => 'Proceed');
 
@@ -186,11 +185,11 @@ sub get_uname {
            .'<li>'.&mt('Your LON-CAPA account must be of a type for which LON-CAPA can reset a password.')
            .'</ul>';
     $msg .= '<form name="forgotpw" method="post" action="/adm/resetpw">'.
-            &Apache::lonhtmlcommon::start_pick_box().
-            &Apache::lonhtmlcommon::row_title('LON-CAPA '.$lt{'unam'}).
+            &Apache::lonhtmlcommon::start_pick_box(). 
+            &Apache::lonhtmlcommon::row_title($lt{'unam'}).
             '<input type="text" name="uname" size="20" />'.
             &Apache::lonhtmlcommon::row_closure(1).
-            &Apache::lonhtmlcommon::row_title('LON-CAPA '.$lt{'udom'}).
+            &Apache::lonhtmlcommon::row_title($lt{'udom'}).
             &Apache::loncommon::select_dom_form($defdom,'udom').
             &Apache::lonhtmlcommon::row_closure(1).
             &Apache::lonhtmlcommon::row_title($lt{'uemail'}).
@@ -203,7 +202,10 @@ sub get_uname {
 sub send_token {
     my ($uname,$udom,$email,$server,$domdesc,$contact_name,
         $contact_email) = @_;
-    my $msg = &mt('Thank you for your request to reset the password for your LON-CAPA account.').'<br /><br />';
+    my $msg =
+        '<p class="LC_info">'
+       .&mt('Thank you for your request to reset the password for your LON-CAPA account.')
+       .'</p>';
 
     my $now = time;
     my $temppasswd = &create_passwd();
@@ -223,12 +225,25 @@ sub send_token {
         my $result = &send_mail($domdesc,$email,$mailmsg,$contact_name,
                                 $contact_email);
         if ($result eq 'ok') {
-            $msg .= &mt("An e-mail sent to the e-mail address associated with your LON-CAPA account includes the web address for the link you should use to complete the reset process.<br /><br />The link included in the message will be valid for the next <b>two</b> hours.");
+            $msg .=
+                &mt('An e-mail sent to the e-mail address associated with your LON-CAPA account includes the web address for the link you should use to complete the reset process.')
+               .'<br /><br />'
+               .&mt('The link included in the message will be valid for the next [_1]two[_2] hours.','<b>','</b>');
         } else {
-            $msg .= &mt("An error occurred when sending a message to the e-mail address associated with your LON-CAPA account. Please contact the [_1] ([_2]) for assistance.",$contact_name,$contact_email);
+            $msg .=
+                '<p class="LC_error">'
+               .&mt('An error occurred when sending a message to the e-mail address'
+                   .' associated with your LON-CAPA account.')
+               .'</p>'
+               .&display_actions($contact_email,$domdesc);
         }
     } else {
-        $msg .= &mt("An error occurred creating a token required for the password reset process. Please contact the [_1] ([_2]) for assistance.",$contact_name,$contact_email);
+        $msg .=
+            '<p class="LC_error">'
+           .&mt('An error occurred creating a token required for the'
+               .' password reset process.')
+           .'</p>'
+           .&display_actions($contact_email,$domdesc);
     }
     return $msg;
 }
@@ -257,8 +272,12 @@ sub invalid_state {
     my ($error,$domdesc,$contact_name,$contact_email) = @_;
     my $msg;
     if ($error eq 'invalid') {
-        $msg = '<p class="LC_warning">'.&mt('The username you provided was not verified as a valid username in the LON-CAPA system for the [_1] domain.',$domdesc)
-              .'</p>'.&mt('Please [_1]go back[_2] and try again.','<a href="javascript:history.go(-1)"><u>','</u></a>');
+        $msg =
+            '<p class="LC_warning">'
+            .&mt('The username you provided was not verified as a valid username'
+                .' in the LON-CAPA system for the [_1] domain.','<i>'.$domdesc.'</i>')
+              .'</p>';
+        $msg .= &display_actions($contact_email,$domdesc);
     } else {
         if ($error eq 'baduseremail') {
             $msg = &mt('The e-mail address you provided does not appear to be a valid address.');
@@ -269,14 +288,8 @@ sub invalid_state {
         } elsif ($error eq 'authentication') {
             $msg = &mt('The username you provided uses an authentication type which can not be reset directly via LON-CAPA.');
         }
-        $msg = '<p class="LC_warning">'.$msg.'</p>';
-        if ($contact_email ne '') {
-            my $escuri = &HTML::Entities::encode('/adm/resetpw','&<>"');
-            $msg .= '<br /> '.&mt('You may wish to contact the [_1]LON-CAPA helpdesk[_2] for the [_3] domain.'
-                                 ,'<a href="/adm/helpdesk?origurl='.$escuri.'">','</a>',$domdesc);
-        } else {
-            $msg .= '<br /> '.&mt('You may wish to send an e-mail to the server administrator: [_1] for the [_2] domain.',$Apache::lonnet::perlvar{'AdminEmail'},$domdesc);
-        }
+        $msg = '<p class="LC_warning">'.$msg.'</p>'
+              .&display_actions($contact_email,$domdesc);
     }
     return $msg;
 }
@@ -313,13 +326,24 @@ sub reset_passwd {
                     my $mailmsg = &mt('The password for your LON-CAPA account in the [_1] domain was changed [_2] from IP address: [_3].  If you did not perform this change or authorize it, please contact the [_4] ([_5]).',$domdesc,$now,$ENV{'REMOTE_ADDR'},$contact_name,$contact_email)."\n";
                     my $result = &send_mail($domdesc,$data{'email'},$mailmsg,
                                             $contact_name,$contact_email);
+                    my $confirm_msg;
                     if ($result eq 'ok') {
-                        $msg .= &mt('An e-mail confirming setting of the password for your LON-CAPA account has been sent to [_1].',$data{'email'});
+                        $confirm_msg =
+                            &Apache::lonhtmlcommon::confirm_success(
+                                &mt('An e-mail confirming setting of the password'
+                                   .' for your LON-CAPA account has been sent to [_1].'
+                                    ,'<span class="LC_filename">'.$data{'email'}.'</span>'));
                     } else {
-                        $msg .= &mt('An error occurred when sending e-mail to [_1] confirming setting of your new password.',$data{'email'});
+                        $confirm_msg =
+                            &Apache::lonhtmlcommon::confirm_success(
+                                &mt('An error occurred when sending e-mail to [_1]'
+                                   .' confirming setting of your new password.'
+                                    ,'<span class="LC_filename">'.$data{'email'}.'</span>'),1);
                     }
-                    $msg .= '<br /><br />'
-                           .'<a href="/adm/login">'.&mt('Go to the login page').'</a>.';
+                    $msg .=
+                        &Apache::loncommon::confirmwrapper($confirm_msg)
+                       .&Apache::lonhtmlcommon::actionbox([
+                            '<a href="/adm/login">'.&mt('Go to the login page').'</a>']);
                 } elsif ($change_failed eq 'invalid_client') {
                     my $homeserver = &Apache::lonnet::homeserver($data{'username'},$data{'domain'});
                     if ($homeserver eq 'no_host') {
@@ -334,7 +358,13 @@ sub reset_passwd {
                            $opentag = '<a href="'.$url.'">';
                            $closetag = '</a>';
                         }
-                        $msg .= &mt('A problem occurred when attempting to reset the password for your account. Please try again from your [_1]home server[_2].',$opentag,$closetag);
+                        $msg .=
+                            '<p class="LC_warning">'
+                           .&mt('A problem occurred when attempting to reset'
+                               .' the password for your account.'
+                               .' Please try again from your [_1]home server[_2].'
+                                ,$opentag,$closetag)
+                           .'</p>';
                     }
                 } else {
                     $msg .= &generic_failure_msg($contact_name,$contact_email);
@@ -345,20 +375,31 @@ sub reset_passwd {
                 &Apache::lonpreferences::passwordchanger($r,'','reset_by_email',$token);
             }
         } else {
-            $msg = &mt('Sorry, the token generated when you requested a password reset has expired. Please submit a [_1]new request[_2], and follow the link to the web page included in the new e-mail that will be sent to you, to allow you to enter a new password.'
-                      ,'<a href="/adm/resetpw">','</a>');
+            $msg =
+                '<p class="LC_warning">'
+               .&mt('Sorry, the token generated when you requested a password reset has expired. Please submit a [_1]new request[_2], and follow the link to the web page included in the new e-mail that will be sent to you, to allow you to enter a new password.'
+                    ,'<a href="/adm/resetpw">','</a>')
+               .'</p>';
         }
     } else {
-        $msg .= &mt('Sorry, the URL generated when you requested reset of your password contained incomplete information. Please submit a [_1]new request[_2] for a password reset, and use the new URL that will be sent to your e-mail account to complete the process.'
-                   ,'<a href="/adm/resetpw">','</a>');
+        $msg .=
+            '<p class="LC_warning">'
+           .&mt('Sorry, the URL generated when you requested reset of your password contained incomplete information. Please submit a [_1]new request[_2] for a password reset, and use the new URL that will be sent to your e-mail account to complete the process.'
+                ,'<a href="/adm/resetpw">','</a>')
+           .'</p>';
     }
     return $msg;
 }
 
 sub generic_failure_msg {
     my ($contact_name,$contact_email) = @_;
-    return &mt('A problem occurred when attempting to reset the password for your account. Please contact the [_1] - ([_2]) for assistance.',
-              $contact_name,'<a href="mailto:'.$contact_email.'">'.$contact_email.'</a>');
+    return
+        '<p class="LC_error">'
+       .&mt('A problem occurred when attempting to reset the password for your account.')
+       .'<br />'
+       .&mt('Please contact the [_1] ([_2]) for assistance.',
+              $contact_name,'<a href="mailto:'.$contact_email.'">'.$contact_email.'</a>')
+       .'</p>';
 }
 
 sub create_passwd {
@@ -381,4 +422,28 @@ sub create_passwd {
     return ($passwd);
 }
 
+sub display_actions {
+    my ($contact_email, $domdesc) = @_;
+    my @msg = (&mt('[_1]Go back[_2] and try again',
+                   '<a href="javascript:history.go(-1)">','</a>'));
+    my $msg2 = '';
+    if ($contact_email ne '') {
+        my $escuri = &HTML::Entities::encode('/adm/resetpw','&<>"');
+        push(@msg, &mt('Contact the [_1]LON-CAPA helpdesk[_2] for the institution: [_3]',
+                       '<a href="/adm/helpdesk?origurl='.$escuri.'">',
+                       '</a>','<i>'.$domdesc.'</i>'));
+    } else {
+        $msg2 =
+            '<p>'
+           .&mt('You may wish to send an e-mail to the'
+           .' server administrator: [_1] for the [_2] domain.',
+                '<i>'.$Apache::lonnet::perlvar{'AdmEMail'}.'</i>',
+                '<i>'.$domdesc.'</i>')
+           .'</p>';
+    }
+
+    return &Apache::lonhtmlcommon::actionbox(\@msg).$msg2;
+
+}
+
 1;
