@@ -1,5 +1,5 @@
 #
-# $Id: assesscalc.pm,v 1.58 2007/01/23 01:36:26 albertel Exp $
+# $Id: assesscalc.pm,v 1.60 2014/02/11 19:11:30 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -156,7 +156,7 @@ sub initialize {
     if (! defined($navmap)) {
         $navmap = Apache::lonnavmaps::navmap->new();
     }
-    if (!defined($navmap)) {
+    if ((!defined($navmap)) || (!ref($navmap))) {
         &Apache::lonnet::logthis('assesscalc:Can not open Coursemap');
     }
     &Apache::loncoursedata::clear_internal_caches();
@@ -488,19 +488,19 @@ END
     $r->print($tableheader);
     #
     # Print out template row
-    $r->print('<tr><td>Template</td><td>&nbsp;</td>'.
+    $r->print('<tr><td>'.&mt('Template').'</td><td>&nbsp;</td>'.
 	      $self->html_template_row($num_uneditable,$importcolor).
               "</tr>\n");
     #
     # Print out summary/export row
-    $r->print('<tr><td>Export</td><td>0</td>'.
+    $r->print('<tr><td>'.&mt('Export').'</td><td>0</td>'.
 	      $self->html_export_row($exportcolor)."</tr>\n");
     #
     # Prepare to output rows
-    $tableheader =<<"END";
-<table border="2">
-<tr><th>row</th><th>Item</th>
-END
+    $tableheader =
+        '<table border="2">'.
+        '<tr><th>'.&mt('Row').'</th>'.
+        '<th>'.&mt('Item').'</th>';
     foreach (split(//,'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz')){
 	if ($label_num<$num_uneditable) { 
             $tableheader.='<th bgcolor="'.$importcolor.'">';
