@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Localization routines
 #
-# $Id: lonlocal.pm,v 1.63 2011/08/03 18:25:16 raeburn Exp $
+# $Id: lonlocal.pm,v 1.65 2014/12/11 01:47:25 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -81,7 +81,7 @@ Inside might be something like this
      if ($status eq 'WON') {
         $message='You have won.';
      } elsif ($status eq 'LOST') {
-        $message='You are a total looser.';
+        $message='You are a total loser.';
      }
      return $message;
  }
@@ -103,7 +103,7 @@ The first two examples are easy:
      if ($status eq 'WON') {
         $message='You have won.';
      } elsif ($status eq 'LOST') {
-        $message='You are a total looser.';
+        $message='You are a total loser.';
      }
      return &mt($message);
  }
@@ -135,7 +135,7 @@ like this:
     'You have won.'
  => 'Sie haben gewonnen.',
 
-    'You are a total looser.'
+    'You are a total loser.'
  => 'Sie sind der totale Verlierer.',
 
     'Rules'
@@ -325,7 +325,7 @@ sub current_locale {
 
 sub texthash {
     my %hash=@_;
-    foreach (keys %hash) {
+    foreach (keys(%hash)) {
 	$hash{$_}=&mt($hash{$_});
     }
     return %hash;
