@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # URL translation for encrypted filenames
 #
-# $Id: lonenc.pm,v 1.24 2008/12/19 14:29:13 raeburn Exp $
+# $Id: lonenc.pm,v 1.25 2011/09/30 15:58:05 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -168,8 +168,10 @@ sub encrypt_ref {
 	    if ($href !~ /^https?\:/) {
 		# IE really wants an extension
 		my ($extension) = ($href =~ m/(\.[^.]*)$/);
-		$href = &Apache::lonenc::encrypted($href,$force_enc);
-		$href .= $extension;
+		my $newhref = &Apache::lonenc::encrypted($href,$force_enc);
+		unless ($newhref eq $href) {
+		    $href = $newhref.$extension;
+		}
 	    }
 	    $token->[2]->{$name}=$href;
 	}
