@@ -2,7 +2,7 @@
 #
 # The LearningOnline Network
 #
-# $Id: parse_activity_log.pl,v 1.24 2006/04/08 07:10:10 albertel Exp $
+# $Id: parse_activity_log.pl,v 1.25 2014/11/24 02:36:34 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -481,6 +481,8 @@ sub process_courselog {
         # %3aPOST%3a(name)%3d(value)%3a(name)%3d(value)
         # or
         # %3aCSTORE%3a(name)%3d(value)%26(name)%3d(value)
+        # or
+        # %3aPUTSTORE%3a(name)%3d(value)%26(name)%3d(value)
         #
         # get delimiter between timestamped entries to be &&&
         $log=~s/\%26(\d{9,10})\%3a/\&\&\&$1\%3a/g;
@@ -511,7 +513,7 @@ sub process_courselog {
             if (! defined($action) || $action eq '') {
                 $action = 'VIEW';
             }
-            if ($action !~ /^(LOGIN|VIEW|POST|CSTORE|STORE)$/) {
+            if ($action !~ /^(LOGIN|VIEW|POST|CSTORE|STORE|PUTSTORE)$/) {
                 $warningflag .= 'action';
                 print $error_fh 'full log entry:'.$log.$/;
                 print $error_fh 'error on chunk (saving)'.$/;
