@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Authoring Space Directory Lister
 #
-# $Id: lonpubdir.pm,v 1.158 2014/06/29 03:33:23 raeburn Exp $
+# $Id: lonpubdir.pm,v 1.160 2014/08/05 19:32:23 musolffc Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -115,17 +115,6 @@ sub handler {
     my $numdir = 0;
     my $numres = 0;
   
-    # Retrieving value for "sortby" and "sortorder" from QUERY_STRING
-    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
-        ['sortby','sortorder']);
-
-    # Sort by name as default, not reversed
-    if (! exists($env{'form.sortby'})) { $env{'form.sortby'} = 'filename' }
-    if (! exists($env{'form.sortorder'})) { $env{'form.sortorder'} = '' }
-
-    my $sortby = $env{'form.sortby'};
-    my $sortorder = $env{'form.sortorder'};
-
     if ((@files == 0) && ($thisdisfn =~ m{^/$match_domain/$match_username})) {
         if ($thisdisfn =~ m{^/$match_domain/$match_username$}) {
             $r->print('<p class="LC_info">'.&mt('This Authoring Space is currently empty.').'</p>');
@@ -136,43 +125,85 @@ sub handler {
         return OK;
     }
 
-    # Start off the directory table.
+    # Retrieving value for "sortby" and "sortorder" from QUERY_STRING
+    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
+        ['sortby','sortorder']);
+
+    # Sort by name as default, not reversed
+    if (! exists($env{'form.sortby'})) { $env{'form.sortby'} = 'filename' }
+    if (! exists($env{'form.sortorder'})) { $env{'form.sortorder'} = '' }
+    my $sortby = $env{'form.sortby'};
+    my $sortorder = $env{'form.sortorder'};
+
+    # Order in which columns are displayed from left to right
+    my @order = ('filetype','actions','filename','title',
+                    'pubstatus','cmtime','size');
+
+    # Up and down arrows to indicate sort order
+    my @arrows = ('&nbsp;&#9650;','&nbsp;&#9660;','');
+
+    # Default sort order and column title
+    my %columns = (
+        filetype =>     {
+                            order => 'ascending',
+                            text  => &mt('Type'),
+                        },
+        actions =>      {
+                            # Not sortable
+                            text  => &mt('Actions'),
+                        },
+        filename =>     {
+                            order => 'ascending',
+                            text  => &mt('Name'),
+                        },
+        title =>        {
+                            order => 'ascending',
+                            text  => &mt('Title'),
+                        },
+        pubstatus =>    {
+                            order => 'ascending',
+                            text  => &mt('Status'),
+                            colspan => '2',
+                        },
+        cmtime =>       {
+                            order => 'descending',
+                            text  => &mt('Last Modified'),
+                        },
+        size =>         {
+                            order => 'ascending',
+                            text  => &mt('Size').' (kB)',
+                        },
+    ); 
+
+    # Print column headers
+    my $output = '';
+    foreach my $key (@order) {
+        my $idx;
+        # Append an up or down arrow to sorted column
+        if ($sortby eq $key) {
+            $idx = ($columns{$key}{order} eq 'ascending') ? 0:1;
+            if ($sortorder eq 'rev') { $idx ++; }
+            $idx = $idx%2;
+        } else { $idx = 2; } # No arrow if column is not sorted
+        $output .= (($columns{$key}{order}) ?
+            '<th'.($columns{$key}{colspan} ? ' colspan="'.$columns{$key}{colspan}.'"' : '')
+            .'><a href="'.$linkdir.'/?sortby='.$key.'&sortorder='
+            .((($sortby eq $key) && ($sortorder ne 'rev')) ? 'rev' : '').'">'
+            .$columns{$key}{text}.$arrows[$idx].'</a></th>' :
+            '<th>'.$columns{$key}{text}.'</th>');
+    }
     $r->print(&Apache::loncommon::start_data_table()
-        .&Apache::loncommon::start_data_table_header_row()
-        .'<th><a href="'.$linkdir.'/?sortby=filetype&sortorder='
-            .((($sortby eq "filetype") && ($sortorder ne 'rev')) ? 'rev' : '') 
-            .'">'.&mt('Type')
-            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
-        .'<th>'.&mt('Actions').'</th>'
-        .'<th><a href="'.$linkdir.'/?sortby=filename&sortorder='
-            .((($sortby eq "filename") && ($sortorder ne 'rev')) ? 'rev' : '') 
-            .'">'.&mt('Name')
-            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
-        .'<th><a href="'.$linkdir.'/?sortby=title&sortorder='
-            .((($sortby eq "title") && ($sortorder ne 'rev')) ? 'rev' : '') 
-            .'">'.&mt('Title')
-            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
-        .'<th colspan="2"><a href="'.$linkdir.'/?sortby=pubstatus&sortorder='
-            .((($sortby eq "pubstatus") && ($sortorder ne 'rev')) ? 'rev' : '') 
-            .'">'.&mt('Status')
-            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
-        .'<th><a href="'.$linkdir.'/?sortby=cmtime&sortorder='
-            .((($sortby eq "cmtime") && ($sortorder ne 'rev')) ? 'rev' : '') 
-            .'">'.&mt('Last Modified')
-            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
-        .'<th><a href="'.$linkdir.'/?sortby=size&sortorder='
-            .((($sortby eq "size") && ($sortorder ne 'rev')) ? 'rev' : '') 
-            .'">'.&mt('Size').' (kB)'
-            .'<span class="LC_fontsize_small"> &#9660;</span></a></th>'
+        .&Apache::loncommon::start_data_table_header_row() . $output
         .&Apache::loncommon::end_data_table_header_row()
     );
 
     my $dirptr=16384;		# Mask indicating a directory in stat.cmode.
     my $filehash = {};
     foreach my $filename (@files) {
-        # Skip .DS_Store and hidden files
+        # Skip .DS_Store, .DAV and hidden files
         my ($extension) = ($filename=~/\.(\w+)$/);
         next if (($filename eq '.DS_Store')
+                || ($filename eq '.DAV')
                 || (&Apache::loncommon::fileembstyle($extension) eq 'hdn')
                 || ($filename =~ /^\._/));
 
