@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Handler to show and edit custom distribution rights
 #
-# $Id: lonrights.pm,v 1.30 2010/06/06 02:40:44 raeburn Exp $
+# $Id: lonrights.pm,v 1.35 2013/06/04 22:20:16 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -58,8 +58,8 @@ sub handler {
   # Breadcrumbs
   &Apache::lonhtmlcommon::clear_breadcrumbs();
   &Apache::lonhtmlcommon::add_breadcrumb({
-      'text'  => 'Construction Space',
-      'href'  => &Apache::loncommon::authorspace(),
+      'text'  => 'Authoring Space',
+      'href'  => &Apache::loncommon::authorspace($r->uri),
       });
   &Apache::lonhtmlcommon::add_breadcrumb({
       'text'  => 'Custom Rights Editor',
@@ -78,7 +78,7 @@ sub handler {
   my $uri=$r->uri;
   my $fn=&Apache::lonnet::filelocation('',$uri);
   my $contents='';
-  my $constructmode=($uri=~/^\/\~/);
+  my $constructmode=($uri=~ m{^/priv/});
 
 # ============================================================ Modify and store
   if ($constructmode && $env{'form.store'}) {
@@ -180,7 +180,7 @@ sub handler {
           $contents=join('',<$fh>);
           $fh->close();
       }
-      $r->print('<form name="rules" method="post">');
+      $r->print('<form name="rules" method="post" action="">');
   }
   unless ($contents=~/\<accessrule/s) {
       $contents='<accessrule effect="deny" />';
@@ -196,6 +196,10 @@ sub handler {
 				     'co' => 'Course / User',
 				     'se' => 'Section',
 				     'ro' => 'Role');
+  my %iconimg = ('allow' => '<img src="/adm/lonIcons/navmap.correct.gif"'
+                           .' alt="'.&mt('allow').'" title="'.&mt('allow').'" />',
+                 'deny' => '<img src="/adm/lonIcons/navmap.wrong.gif"'
+                           .' alt="'.&mt('deny').'" title="'.&mt('deny').'" />');
 # ---------------------------------------------------------- Start table output
   $r->print(&Apache::loncommon::start_data_table().
 	    &Apache::loncommon::start_data_table_header_row().
@@ -217,14 +221,13 @@ sub handler {
       $r->print('&nbsp;');
   }
   $r->print('</td><td>');
+  $r->print($iconimg{$token->[2]->{'effect'}});
   if ($constructmode) {
       my %lt = &Apache::lonlocal::texthash
                                    ('allow' => 'allow',
                                     'deny'  => 'deny');
       $r->print(&Apache::loncommon::select_form
                                  ($token->[2]->{'effect'},'effect_0',\%lt));
-  } else {
-      $r->print($token->[2]->{'effect'});
   }
   $r->print('</td><td colspan="5">'.&mt('Default'));
   if (($token->[2]->{'realm'}) || ($token->[2]->{'role'})) {
@@ -242,8 +245,8 @@ sub handler {
 	  $r->print('<span class="LC_nobreak">'.$rulecounter.'. ');
           if ($constructmode) {
              my %lt = &Apache::lonlocal::texthash
-                          ('' => '',
-                           'delete' => 'Delete this rule',
+                          (''            => '',
+                           'delete'      => 'Delete this rule',
                            'insertabove' => 'Insert rule above',
                            'insertbelow' => 'Insert rule below',
                            'moveup'      => 'Move rule up',
@@ -256,6 +259,7 @@ sub handler {
           $r->print('</span></td>'.
 		    '<td rowspan="2">');
 # effect
+          $r->print($iconimg{$token->[2]->{'effect'}});
           if ($constructmode) {
              my %lt = &Apache::lonlocal::texthash
                              ('allow' => 'allow',
@@ -263,8 +267,6 @@ sub handler {
              $r->print(&Apache::loncommon::select_form
                                  ($token->[2]->{'effect'},
                                   'effect_'.$rulecounter,\%lt));
-          } else {
-             $r->print($token->[2]->{'effect'});
           }
 
 # type
@@ -277,7 +279,7 @@ sub handler {
               $r->print(&Apache::loncommon::select_form($type,
 							'type_'.$rulecounter,\%lt));
           } else {
-              $r->print($type);
+              $r->print(&mt($type));
           }
 
 # ---- realm
