@@ -1,5 +1,5 @@
 # Handler for parsing text upload problem descriptions into .problems
-# $Id: testbankimport.pm,v 1.30 2011/03/05 22:25:09 raeburn Exp $
+# $Id: testbankimport.pm,v 1.44 2014/02/11 19:11:18 bisitz Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -28,7 +28,6 @@ package Apache::testbankimport;
 
 use strict;
 use Apache::Constants qw(:common :http :methods);
-use Apache::loncacc;
 use Apache::loncommon();
 use Apache::lonnet;
 use HTML::Entities();
@@ -90,7 +89,7 @@ function createWin() {
     newWindow.document.write("<input type='hidden' name='callingmode' value='testbank' />")
     newWindow.document.write("<input type='hidden' name='inhibitmenu' value='yes' />")
     newWindow.document.write("$webpath<input type='text' name='newfilename' value='' />")
-    newWindow.document.write("<input type='button' value='$lt{'go'}' onClick='document.fileaction.submit();' /></form>")
+    newWindow.document.write("<input type='button' value='$lt{'go'}' onclick='document.fileaction.submit();' /></form>")
     newWindow.document.write('$end_page')
     newWindow.document.close()
     newWindow.focus()
@@ -290,18 +289,20 @@ function colSet(caller) {
     else {
         if ((poolForm.elements[caller*5+3].options[poolForm.elements[caller*5+3].selectedIndex].value == "MC") || (poolForm.elements[caller*5+3].options[poolForm.elements[caller*5+3].selectedIndex].value == "MA") || (poolForm.elements[caller*5+3].options[poolForm.elements[caller*5+3].selectedIndex].value == "Ord")) {
             poolForm.elements[caller*5+4].options[0] = new Option("Select","-1",true,true)
-            poolForm.elements[caller*5+4].options[1] = new Option("a.","lcperiod",false,false)
-            poolForm.elements[caller*5+4].options[2] = new Option("A.","ucperiod",false,false)
-            poolForm.elements[caller*5+4].options[3] = new Option("(a)","lcparen",false,false)
-            poolForm.elements[caller*5+4].options[4] = new Option("(A)","ucparen",false,false)
-            poolForm.elements[caller*5+4].options[5] = new Option("a)","lconeparen",false,false)
-            poolForm.elements[caller*5+4].options[6] = new Option("A)","uconeparen",false,false)
-            poolForm.elements[caller*5+4].options[7] = new Option("a.)","lcdotparen",false,false)
-            poolForm.elements[caller*5+4].options[8] = new Option("A.)","ucdotparen",false,false)
-            poolForm.elements[caller*5+4].options[9] = new Option("(i)","romparen",false,false)
-            poolForm.elements[caller*5+4].options[10] = new Option("i)","romoneparen",false,false)
-            poolForm.elements[caller*5+4].options[11] = new Option("i.)","romdotparen",false,false)
-            poolForm.elements[caller*5+4].options[12] = new Option("i.","romperiod",false,false)
+            poolForm.elements[caller*5+4].options[1] = new Option("a ","lcspace",false,false)
+            poolForm.elements[caller*5+4].options[2] = new Option("A ","ucspace",false,false) 
+            poolForm.elements[caller*5+4].options[3] = new Option("a.","lcperiod",false,false)
+            poolForm.elements[caller*5+4].options[4] = new Option("A.","ucperiod",false,false)
+            poolForm.elements[caller*5+4].options[5] = new Option("(a)","lcparen",false,false)
+            poolForm.elements[caller*5+4].options[6] = new Option("(A)","ucparen",false,false)
+            poolForm.elements[caller*5+4].options[7] = new Option("a)","lconeparen",false,false)
+            poolForm.elements[caller*5+4].options[8] = new Option("A)","uconeparen",false,false)
+            poolForm.elements[caller*5+4].options[9] = new Option("a.)","lcdotparen",false,false)
+            poolForm.elements[caller*5+4].options[10] = new Option("A.)","ucdotparen",false,false)
+            poolForm.elements[caller*5+4].options[11] = new Option("(i)","romparen",false,false)
+            poolForm.elements[caller*5+4].options[12] = new Option("i)","romoneparen",false,false)
+            poolForm.elements[caller*5+4].options[13] = new Option("i.)","romdotparen",false,false)
+            poolForm.elements[caller*5+4].options[14] = new Option("i.","romperiod",false,false)
             poolForm.elements[caller*5+4].selectedIndex = 0
         }
         else {
@@ -445,7 +446,7 @@ function verify() {
     if ($numitems > 0) {
         my $maxnum = $numitems - 1;
         my %lt = &Apache::lonlocal::texthash(
-                                              fnmb => 'File names must be unique',
+                                              fnmb => 'Filenames must be unique',
                                               isum => 'is used more than once',
                                             );
         $$jsref .= qq|
@@ -485,12 +486,12 @@ function backPage() {
 
 # ---------------------------------------------------------------- Display Zero
 sub display_zero {
-    my ($r,$uname,$fn,$page,$webpath) = @_;
+    my ($r,$fn,$page,$webpath) = @_;
     my $go_default = 'NextPage'; 
     if ($fn eq '') {
-        $r->print('<b>'.&mt('Incomplete file upload').'</b> '.&mt('Return to the [_1]construction space menu[_2] to upload a file','<a href="'.$webpath.'">','</a>'));
+        $r->print('<b>'.&mt('Incomplete file upload').'</b> '.&mt('Return to the [_1]Authoring Space menu[_2] to upload a file','<a href="'.$webpath.'">','</a>'));
     }
-    $r->print(&mt('The <b>Testbank Upload</b> utility can be used by LON-CAPA authors to generate LON-CAPA problem files from a testbank file of questions/answers.').'<br />'.
+    $r->print(&mt('The [_1]Testbank Upload[_2] utility can be used by LON-CAPA authors to generate LON-CAPA problem files from a testbank file of questions/answers.','<b>','</b>').'<br />'.
               &mt('The following question types can be converted:').'
               <ul>
                 <li>'.&mt('multiple choice').'</li>
@@ -511,15 +512,15 @@ sub display_zero {
         </ol><form name="info" method="post" action="/adm/testbank">'.
         &Apache::lonhtmlcommon::topic_bar(1,&mt('Optional: create a sub-directory in which the testbank questions will be saved')).
         &mt('By default, LON-CAPA problems generated from the testbank file will be stored in the current directory.').' '.&mt('To store them in a new sub-directory:'). 
-       ' <input type="button" name="createdir" value="'.&mt('Create sub-directory').'" onClick="javascript:createWin()" />'.
-       &page_footer($env{'form.newdir'},$uname,$fn,$page,$webpath).'
+       ' <input type="button" name="createdir" value="'.&mt('Create sub-directory').'" onclick="javascript:createWin()" />'.
+       &page_footer($env{'form.newdir'},$fn,$page,$webpath).'
        </form>');
 }
 
 # ---------------------------------------------------------------- Display One
 
 sub display_one {
-    my ($r,$uname,$fn,$page,$textref,$header) = @_;
+    my ($r,$fn,$page,$textref,$header) = @_;
     my %topics;
     $topics{2} = &mt('Select the format of the question number - e.g., 1,  1., 1), (1 or (1) - ').'&nbsp;
                <select name="qnumformat">
@@ -537,9 +538,9 @@ sub display_one {
               &Apache::lonhtmlcommon::topic_bar(2,$topics{2}).'<p>'.
               &mt('A number in the specified format should appear at the start of each question.').'<br />'.
               &mt('For multiple choice questions, the question number must begin the line that contains the question text; foils (starting (a), (i) etc.) should occur on subsequent lines.').'<br />'."\n".
-              &mt('Correct answers should be numbered in the same way as the questions and should appear after <b>all</b> the questions (including question text and possible foils for all questions).').'<br />'."\n".
+              &mt('Correct answers should be numbered in the same way as the questions and should appear after [_1]all[_2] the questions (including question text and possible foils for all questions).','<b>','</b>').'<br />'."\n".
               &mt('Each numbered question must have a corresponding numbered answer, although the answer itself may be blank for essay questions.').'<br /><br />'."\n".
-              &mt('For example, you would select <b>1.</b> if your testbank file contained the following questions:').'<br /><blockquote>'.
+              &mt('For example, you would select [_1]1.[_2] if your testbank file contained the following questions:','<b>','</b>').'<br /><blockquote>'.
 '<pre>
  1. '.&mt('The capital of the USA is ...').'
  (a) Washington D.C.
@@ -558,7 +559,7 @@ sub display_one {
 </pre>'.
              '</blockquote></p>'.
              &Apache::lonhtmlcommon::topic_bar(3,$topics{3}).'<p>'.
-             &mt('For example, you would enter <b>6</b> if your testbank file contained the following sequence of questions:').'</p><blockquote>'.
+             &mt('For example, you would enter [_1]6[_2] if your testbank file contained the following sequence of questions:','<b>','</b>').'</p><blockquote>'.
              &mt('10 multiple choice questions').'<br />'.
              &mt('5 essay questions').'<br />'.
              &mt('5 fill-in-the-blank questions').'<br />'.
@@ -566,7 +567,7 @@ sub display_one {
              &mt('4 multiple choice questions').'<br />'.
              &mt('3 essay questions').'</blockquote></p><p>'.
              &mt('You will indicate the question type and the question number range for each of the blocks on the next page.').'</p><br />'.
-             &page_footer($env{'form.newdir'},$uname,$fn,$page).'
+             &page_footer($env{'form.newdir'},$fn,$page).'
  </form>');
     return;
 }
@@ -574,11 +575,11 @@ sub display_one {
 # ---------------------------------------------------------------- Display Two
 
 sub display_two {
-    my ($r,$uname,$fn,$page,$textref,$header,$qcount) = @_;
+    my ($r,$fn,$page,$textref,$header,$qcount) = @_;
     my $blocks = $env{'form.blocks'};
     my $qnumformat = $env{'form.qnumformat'};
     my @types = ("MC","MA","TF","Ess","FIB","Ord");
-    my %typenames = (
+    my %typenames = &Apache::lonlocal::texthash(
              MC => "Multiple Choice",
              TF => "True/False",
              MA => "Multiple Answer",
@@ -602,15 +603,15 @@ sub display_two {
     my $steptitle = &mt('Information about question types and formats in each block.');
     $r->print('<h3>'.&mt('Classification of blocks').'</h3>'.
               '<form method="post" name="display" action="/adm/testbank"><p>'.
-              &mt('You indicated that <b>all</b> questions (and the corresponding answer(s) for each question) begin with a number in the following format: [_1].','<b>'.$qnumtypes{$qnumformat}.'</b>').'</p><p>'.
-              &mt('A total of <b>[quant,_1,question]</b> and <b>[quant,_2,answer]</b> were found in the file you uploaded.',$qcount,$qcount).' '.
-              &mt('If this total does not match the number you expect, examine your original testbank file to verify that each question <i>and</i> each answer begins with a number in the specified format.').' '.
+              &mt('You indicated that [_1]all[_2] questions (and the corresponding answer(s) for each question) begin with a number in the following format: [_3].','<b>','</b>','<b>'.$qnumtypes{$qnumformat}.'</b>').'</p><p>'.
+              &mt('A total of [_1][quant,_3,question][_2] and [_1][quant,_4,answer][_2] were found in the file you uploaded.','<b>','</b>',$qcount,$qcount).' '.
+              &mt('If this total does not match the number you expect, examine your original testbank file to verify that each question [_1]and[_2] each answer begins with a number in the specified format.','<i>','</i>').' '.
               &mt('If necessary use an editor to edit your testbank file of questions, and click "Previous Page" on this page and the "Exit Now" on the preceding page, so you can upload your file again.').'</p><p>'.
-              &mt('You also indicated that the <b>[quant,_1,question]</b> can be divided into <b>[quant,_2,block]</b> of questions of a particular question type.',$qcount,$blocks).'</p><p>'.
+              &mt('You also indicated that the [_1][quant,_3,question][_2] can be divided into [_1][quant,_4,block][_2] of questions of a particular question type.','<b>','</b>',$qcount,$blocks).'</p><p>'.
               &mt('Provide additional information below, about the types of questions you have uploaded, and, if applicable, the format of answers and "foils" for specific types of questions.').'</p>'.
               &show_uploaded_data($textref,$header).
               &Apache::lonhtmlcommon::topic_bar(4,$steptitle).'<p>'.
-              &mt('For <i>each</i> of the [_1] question blocks, specify the question numbers of the first and last questions in the block (e.g., 1 and 10), and the question type of the questions in the block.','<b>'.$blocks.'</b>').' '.
+              &mt('For [_1]each[_2] of the [_3] question blocks, specify the question numbers of the first and last questions in the block (e.g., 1 and 10), and the question type of the questions in the block.','<i>','</i>','<b>'.$blocks.'</b>').' '.
               &mt('If required, provide additional information about foil formats and answer formats for the question types you select.').'</p><p>'.
               &Apache::loncommon::start_data_table().
               &Apache::loncommon::start_data_table_header_row().
@@ -628,7 +629,7 @@ sub display_two {
                  '<td valign="top">&nbsp;<input type="text" name="start_'.$i.'" value="'.$bl1st.'" size="5" />&nbsp;</td>'."\n".
                  '<td valign="top">&nbsp;<input type="text" name="end_'.$i.'" value="'.$bl1end.'" size="5" />&nbsp;</td>'."\n".
                  '<td valign="top">
-   <select name="qtype_'.$i.'" onChange="colSet('.$i.')">
+   <select name="qtype_'.$i.'" onchange="colSet('.$i.')">
     <option value="-1" selected="selected">'.&mt('Select').'</option>'."\n");
         foreach my $qtype (@types) {
             $r->print('<option value="'.$qtype.'">'.$typenames{$qtype}.'</option>'."\n");
@@ -648,24 +649,24 @@ sub display_two {
                      &Apache::loncommon::end_data_table_row()); 
     }
     $r->print(&Apache::loncommon::end_data_table().'</p><ul><li>'.
-              &mt('For <i>multiple choice</i>, <i>multiple correct answer</i> and <i>ranking</i> type questions, you must use the <b>Foil format</b> column to choose the format of the identifier used for each of the possible answers (e.g., (a), a, a., i, (i) etc.) provided for a given question stem.').'</li><li>'.
-             &mt('For <i>multiple correct answer</i> and <i>fill-in-the-blank</i> questions with more than one correct answer you must use the <b>Answer format</b> column to choose the separator used between the answers, e.g., if the correct answers for question 28. were listed as:[_1] you would choose "comma", or if they were listed as:[_2] you would choose "new line".','<blockquote><pre>28. (a),(d),(e)</pre></blockquote>','<blockquote><pre>
+              &mt('For [_1]multiple choice[_2], [_1]multiple correct answer[_2] and [_1]ranking[_2] type questions, you must use the [_3]Foil format[_4] column to choose the format of the identifier used for each of the possible answers (e.g., (a), a, a., i, (i) etc.) provided for a given question stem.','<i>','</i>','<b>','</b>').'</li><li>'.
+             &mt('For [_1]multiple correct answer[_2] and [_1]fill-in-the-blank[_2] questions with more than one correct answer you must use the [_3]Answer format[_4] column to choose the separator used between the answers, e.g., if the correct answers for question 28. were listed as:[_5] you would choose "comma", or if they were listed as:[_6] you would choose "new line".','<i>','</i>','<b>','</b>','<blockquote><pre>28. (a),(d),(e)</pre></blockquote>','<blockquote><pre>
 28. (a)
     (d)
     (e)
 </pre></blockquote>').'</li><li>'.
-             &mt('For <i>true/false</i> questions you must use the <b>Answer format</b> column to choose how the correct answer - True or False, is displayed in the text file (e.g., T or F, true or false etc.).').'</li><li>'.
-            &mt('For <i>ranking</i> questions you must use the <b>Answer format</b> column to choose the separator used between the (ranked) answers.').'</li></ul>
+             &mt('For [_1]true/false[_2] questions you must use the [_3]Answer format[_4] column to choose how the correct answer - True or False, is displayed in the text file (e.g., T or F, true or false etc.).','<i>','</i>','<b>','</b>').'</li><li>'.
+            &mt('For [_1]ranking[_2] questions you must use the [_3]Answer format[_4] column to choose the separator used between the (ranked) answers.','<i>','</i>','<b>','</b>').'</li></ul>
 <input type="hidden" name="blocks" value="'.$blocks.'" />
 <input type="hidden" name="qnumformat" value="'.$qnumformat.'" />'.
-           &page_footer($env{'form.newdir'},$uname,$fn,$page).'
+           &page_footer($env{'form.newdir'},$fn,$page).'
 </form>');
     return;
 }
 
 # ---------------------------------------------------------------- Display Three
 sub display_three {
-    my ($r,$uname,$fn,$page,$textref,$res,$header,$urlpath,$qcount) = @_;
+    my ($r,$fn,$page,$textref,$res,$header,$webpath,$qcount) = @_;
     my $qnumformat = $env{'form.qnumformat'};
     my $filename = $env{'form.filename'};
     my $source = $env{'form.go'};
@@ -682,7 +683,7 @@ sub display_three {
     my %lt = &Apache::lonlocal::texthash (
                                           crt  => 'Create?',
                                           typ  => 'Type',
-                                          fnam => 'File Name',
+                                          fnam => 'Filename',
                                           ques => 'Question',
                                           answ => 'Answer',
                                           chka => 'check all',
@@ -715,7 +716,7 @@ sub display_three {
         if ($header ne '') {
             $showheader = &HTML::Entities::decode($header);
             if ($res eq 'text/html') {
-                $showheader = &build_image_url($urlpath,$showheader);
+                $showheader = &build_image_url($webpath,$showheader);
             }
         }
     }
@@ -764,7 +765,7 @@ sub display_three {
                                 if (($res eq 'application/rtf') || ($res eq 'text/html')) {
                                     $showqn = &HTML::Entities::decode($showqn);
                                     if ($res eq 'text/html') {
-                                        $showqn = &build_image_url($urlpath,$showqn);
+                                        $showqn = &build_image_url($webpath,$showqn);
                                     }
                                 }
                                 $r->print(&Apache::loncommon::start_data_table_row().
@@ -775,7 +776,12 @@ sub display_three {
                                           '<td valign="top">'.$showqn.'<br /><br />'."\n");
                             } else {
                                 my $foiltag = '';
-                                if ($foilformats[$i] eq "lcperiod") {
+                                if ($foilformats[$i] eq "lcspace") {
+                                    $foiltag = $alphabet->[$k-1].' ';
+                                } elsif ($foilformats[$i] eq "ucspace") {
+                                    $foiltag = $alphabet->[$k-1].' ';
+                                    $foiltag =~ tr/a-z/A-Z/;
+                                } elsif ($foilformats[$i] eq "lcperiod") {
                                     $foiltag = $alphabet->[$k-1].'.'; 
                                 } elsif ($foilformats[$i] eq "lcparen") {
                                     $foiltag = '('.$alphabet->[$k-1].')';
@@ -808,7 +814,7 @@ sub display_three {
                                 if ($res eq 'application/rtf' || $res eq 'text/html') {
                                     $showfoil = &HTML::Entities::decode($showfoil);
                                     if ($res eq 'text/html') {
-                                        $showfoil = &build_image_url($urlpath,$showfoil);
+                                        $showfoil = &build_image_url($webpath,$showfoil);
                                     }
                                 } 
                                 $r->print("$foiltag $showfoil<br />\n");
@@ -859,13 +865,13 @@ sub display_three {
           <input type="hidden" name="ansr_'.$i.'" value="'.$ansrtypes[$i].'" />');
         }
     }
-    $r->print('</p>'.&page_footer($env{'form.newdir'},$uname,$fn,$page).'
+    $r->print('</p>'.&page_footer($env{'form.newdir'},$fn,$page).'
               </form>');
 }
 
 # ---------------------------------------------------------------- Final Display
 sub final_display {
-    my ($r,$uname,$fn,$page,$textref,$res,$header,$css,$js,$webpath,$dirpath,$subdir) = @_;
+    my ($r,$fn,$page,$textref,$res,$header,$css,$js,$webpath,$dirpath,$subdir) = @_;
     my $qnumformat = $env{'form.qnumformat'};
     my $blocks = $env{'form.blocks'};
     my $question_id = '';
@@ -925,7 +931,7 @@ sub final_display {
                         $items->[$k] =~ tr/A-Z/a-z/;
                         $items->[$k] =~ s/<\/?[^>]+>//g;
                         $items->[$k] =~ s/\W//g;
-                        if ($foilformats[$i] eq "lcperiod" || $foilformats[$i] eq "lcparen" || $foilformats[$i] eq "lconeparen" || $foilformats[$i] eq "lcdotparen" || $foilformats[$i] eq "ucparen" || $foilformats[$i] eq "ucperiod" || $foilformats[$i] eq "uconeparen" || $foilformats[$i] eq "ucdotparen") {
+                        if ($foilformats[$i] eq "lcspace" || $foilformats[$i] eq "ucspace" || $foilformats[$i] eq "lcperiod" || $foilformats[$i] eq "lcparen" || $foilformats[$i] eq "lconeparen" || $foilformats[$i] eq "lcdotparen" || $foilformats[$i] eq "ucparen" || $foilformats[$i] eq "ucperiod" || $foilformats[$i] eq "uconeparen" || $foilformats[$i] eq "ucdotparen") {
                             for (my $j=0; $j<@{$alphabet}; $j++) {
                                 if ($alphabet->[$j] eq $items->[$k]) {
                                     push @{$answers{$qnum}}, $j;
@@ -958,7 +964,7 @@ sub final_display {
                             } else {
                                 $correct =~s/\W//g;
                             }
-                            if ($foilformats[$i] eq "lcperiod" || $foilformats[$i] eq "lcparen" || $foilformats[$i] eq "ucparen" || $foilformats[$i] eq "ucperiod") {
+                            if ($foilformats[$i] eq "lcspace" || $foilformats[$i] eq "ucspace" || $foilformats[$i] eq "lcperiod" || $foilformats[$i] eq "lcparen" || $foilformats[$i] eq "ucparen" || $foilformats[$i] eq "ucperiod") {
                                 if (($qtype[$i] eq "Ord") && (@tied > 0)) {
                                     my @ties;
                                     foreach my $tie (@tied) {
@@ -1042,7 +1048,7 @@ sub final_display {
     if (@createprobs == 0) {
         $state = 'unchecked';
         $r->print('<p>'.&mt('No questions were selected for conversion.').'</p>'.
-                  &page_footer($env{'form.newdir'},$uname,$fn,$page,$webpath,$subdir,$state).'</form>');
+                  &page_footer($env{'form.newdir'},$fn,$page,$webpath,$subdir,$state).'</form>');
     } elsif (($destdir ne '') && (-e $destdir)) {
         my (@qn_file,@result,@numid);
         my $qcount = 0;
@@ -1130,11 +1136,11 @@ sub final_display {
             $r->print('<p>'.&mt('The following files already existed, and were not overwritten so these problems generated from the testbank have not been saved:').'<br />'.$existing.'</p>');
             $state = 'existing';
         }
-        $r->print(&page_footer($env{'form.newdir'},$uname,$fn,$page,$webpath,$subdir,$state).'</form>');
+        $r->print(&page_footer($env{'form.newdir'},$fn,$page,$webpath,$subdir,$state).'</form>');
     } else {
         $state = 'nodir';
         $r->print('<p>'.&mt('No destination directory was available so import of questions could not proceed.').'</p>'.
-                  &page_footer($env{'form.newdir'},$uname,$fn,$page,$webpath,$subdir,$state).'</form>');
+                  &page_footer($env{'form.newdir'},$fn,$page,$webpath,$subdir,$state).'</form>');
     }
     return;
 }
@@ -1159,7 +1165,7 @@ sub show_uploaded_data {
 }
 
 sub page_footer {
-    my ($newdir,$uname,$fn,$page,$webpath,$subdir,$state) = @_;
+    my ($newdir,$fn,$page,$webpath,$subdir,$state) = @_;
     my $prevval = &mt('Previous Page');
     my $nextval = &mt('Next Page');
     my $prevclick = 'javascript:backPage();';
@@ -1184,7 +1190,6 @@ sub page_footer {
     }
     my $output = '
        <input type="hidden" name="newdir" value="'.&HTML::Entities::encode($newdir,'<>&"').'" />
-       <input type="hidden" name="uploaduname" value="'.$uname.'" />
        <input type="hidden" name="filename" value="'.$fn.'" />
        <input type="hidden" name="page" value="'.$page.'" />
        <input type="hidden" name="phase" value="three" />
@@ -1277,7 +1282,11 @@ sub file_split {
         if (${$numsref}[$i] > 0) {
             if ((${$qtyperef}[$i] eq "MC") || (${$qtyperef}[$i] eq "MA") || (${$qtyperef}[$i] eq "Ord")) {
                 my $splitstr = '';
-                if (${$foilsref}[$i] eq "lcperiod") {
+                if (${$foilsref}[$i] eq "lcspace") {
+                    $splitstr = '[a-z]\s';
+                } elsif (${$foilsref}[$i] eq "ucspace") {
+                    $splitstr = '[A-Z]\s';
+                } elsif (${$foilsref}[$i] eq "lcperiod") {
                     $splitstr = '[a-z]\.';
                 } elsif (${$foilsref}[$i] eq "lcparen") {
                     $splitstr = '\([a-z]\)';
@@ -1580,6 +1589,7 @@ sub create_ess {
          if (open(PROB,">$destdir$libfile")) {
              print PROB $output;
              close(PROB);
+             $result = 'ok';
          } else {
              $result = 'failed';
          }
@@ -1604,18 +1614,21 @@ sub probfile_name {
 }
 
 sub file_error {
-    my ($r,$uname,$fn,$current_page,$webpath,$res) = @_;
+    my ($r,$fn,$current_page,$webpath,$res) = @_;
     $r->print('<p><form name="display" method="post" action="/adm/testbank">'.&mt('The file you uploaded does not appear to be in the correct format.').
               '</p><p>'.&mt('Extraction of questions is only possible for the following file types:').
-              '<ul><li>'.&mt('plain text').'</li><li>RTF</li><li>HTML</li></ul>'.
+              '<ul>'.
+              '<li>'.&mt('plain text').'</li>'.
+              '<li>'.&mt('RTF').'</li>'.
+              '<li>'.&mt('HTML').'</li></ul>'.
               &mt('The file type identified for the file you uploaded is [_1].','<b>'.$res.'</b>').'</p>');
-    $r->print(&page_footer($env{'form.newdir'},$uname,$fn,$current_page,$webpath,undef,'badfile').
+    $r->print(&page_footer($env{'form.newdir'},$fn,$current_page,$webpath,undef,'badfile').
              '</form>');
     return;
 }
 
 sub parse_datafile {
-    my ($r,$uname,$filename,$pathname,$dirpath,$urlpath,$page_name,$subdir,$timestamp) = @_;
+    my ($r,$filename,$dirpath,$webpath,$page_name,$subdir,$timestamp) = @_;
     my ($badfile,$res,%allfiles,%codebase);
     my $mm = new File::MMagic;
     my ($text,$header,$css,$js);
@@ -1630,7 +1643,7 @@ sub parse_datafile {
                 my $html = '';
                 my $image_uri = $timestamp;
                 if ($page_name eq 'Target') {
-                    $image_uri = $urlpath.'/'.$timestamp;
+                    $image_uri = "$webpath/$timestamp";
                 }
                 my $image_dir;
                 if ($page_name eq 'Blocks') {
@@ -1768,16 +1781,16 @@ sub parse_htmlcontent {
 }
 
 sub build_image_url {
-    my ($urlpath,$item) = @_;
-    $item =~ s/(<img[^>]+src=["']?\s*)(\.?\.?\/?)/$1$urlpath/gsi;
-    return $item; 
+    my ($webpath,$item) = @_;
+    $item =~ s/(<img[^>]+src=["']?\s*)(\.?\.?\/?)/$1$webpath/gsi;
+    return $item;
 }
 
 sub print_header {
     my ($uname,$udom,$javascript,$loadentries,$title,$current_page,$pagesref,
         $namesref) = @_;
-    my $brcrum = [{'href' => &Apache::loncommon::authorspace(),
-                   'text' => 'Construction Space'}];
+    my $brcrum = [{'href' => &Apache::loncommon::authorspace("/priv/$udom/$uname/"),
+                   'text' => 'Authoring Space'}];
     if ($env{'form.phase'} eq 'three') {
         if (ref($pagesref) eq 'ARRAY') {
             for (my $i=0; $i<$current_page; $i++) {
@@ -1814,54 +1827,45 @@ sub print_header {
 # ---------------------------------------------------------------- Main Handler
 sub handler {
     my $r=shift;
-    my $uname;
-    my $udom;
-    my $javascript = '';
-    my $page_name = '';
-    my $current_page = '';
-    my $qcount = '';
-    my $title = 'Upload testbank questions to Construction Space';
 
-    if ($env{'form.uploaduname'}) {
-        $env{'form.filename'}='/priv/'.$env{'form.uploaduname'}.'/'.
-            $env{'form.filename'};
-    }
-    ($uname,$udom)=
-        &Apache::loncacc::constructaccess($env{'form.filename'},
-                                          $r->dir_config('lonDefDomain'));
-    unless (($uname) && ($udom)) {
-        $r->log_reason($uname.':'.$udom.' trying to convert testbank file '.
-                       $env{'form.filename'}.' - not authorized',$r->filename);
-        return HTTP_NOT_ACCEPTABLE;
-    }
+    my $fn=$env{'form.filename'};
 
-    my ($fn,$filename);
-    if ($env{'form.filename'}) {
-        $fn=$env{'form.filename'};
-        $fn=~s/^https?\:\/\/[^\/]+\///;
-        $fn=~s/^\///;
-        $fn=~s{(~|priv/)($LONCAPA::username_re)}{};
-        $fn=~s/\/+/\//g;
-    } else {
+    if ($env{'form.filename1'}) {
+       $fn=$env{'form.filename1'}.$env{'form.filename2'};
+    }
+    $fn=~s{\+}{}g;
+
+    unless ($fn) {
         $r->log_reason($env{'user.name'}.' at '.$env{'user.domain'}.
                        ' unspecified filename for upload', $r->filename);
         return HTTP_NOT_FOUND;
     }
 
+    my ($uname,$udom) = &Apache::lonnet::constructaccess($fn);
+    if (($uname eq '') || ($udom eq '')) {
+        $r->log_reason($uname.':'.$udom.' trying to convert testbank file '.
+                       $fn.' - not authorized',$r->filename);
+        return HTTP_NOT_ACCEPTABLE;
+    }
+
+    my $javascript = '';
+    my $page_name = '';
+    my $current_page = '';
+    my $qcount = '';
+    my $title = 'Upload testbank questions to Authoring Space';
+
 # ----------------------------------------------------------- Start page output
     &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
 
-    my ($filename,$pathname) = &File::Basename::fileparse($fn);
-    my $webpath = '/priv/'.$uname.$pathname;
-    my $urlpath = '/~'.$uname.$pathname;
-    my $dirpath = '/home/'.$uname.'/public_html'.$pathname;
+    my ($filename,$webpath) = &File::Basename::fileparse($fn);
+    my $dirpath = $r->dir_config('lonDocRoot').$webpath;
     my ($res,$subdir,$badfile,$textref,$header,$css,$js,%loadentries,@pages,%names);
 
     if ($env{'form.phase'} eq 'three') {
         $current_page = &display_control();
         @pages = ('Welcome','Blocks','Format','Target','Confirmation');
-        %names = (
+        %names = &Apache::lonlocal::texthash(
                    Welcome      => 'Testbank Format',
                    Blocks       => 'Classification',
                    Format       => 'Selection',
@@ -1877,8 +1881,8 @@ sub handler {
             }
         }
         ($res,$badfile,$textref,$header,$css,$js) = 
-            &parse_datafile($r,$uname,$filename,$pathname,$dirpath,$urlpath,
-                            $page_name,$subdir,$env{'form.timestamp'});
+            &parse_datafile($r,$filename,$dirpath,$webpath,$page_name,
+                            $subdir,$env{'form.timestamp'});
         if ($page_name eq 'Welcome') {
              &jscript_zero($webpath,\$javascript);
         } elsif ($page_name eq 'Blocks') {
@@ -1920,7 +1924,7 @@ sub handler {
             &jscript_zero($webpath,\$js);
             $js = '<script type="text/javascript">'."\n$js\n".'</script>';
             $r->print($js);
-            &display_zero($r,$uname,$fn,$current_page,$webpath);
+            &display_zero($r,$fn,$current_page,$webpath);
         } elsif ($env{'form.phase'} eq 'three') {
             if ($env{'form.action'} eq 'upload_embedded') {
                 my ($result,$flag) = 
@@ -1929,21 +1933,21 @@ sub handler {
                 if ($flag eq 'modify_orightml') {
                     undef($page_name); 
                     $r->print('<form name="testbankForm" method="post" action="/adm/testbank">'.
-                              &page_footer('',$uname,$fn).'</form>');
+                              &page_footer('',$fn).'</form>');
                 }
             }
         }
         if ($badfile) {
-            &file_error($r,$uname,$fn,$current_page,$webpath,$res);
+            &file_error($r,$fn,$current_page,$webpath,$res);
         } else {
-            &display_zero ($r,$uname,$fn,$current_page,$webpath) if $page_name eq 'Welcome';
-            &display_one ($r,$uname,$fn,$current_page,$textref,$header) if $page_name eq 'Blocks';
-            &display_two ($r,$uname,$fn,$current_page,$textref,$header,$qcount) if $page_name eq 'Format';
-            &display_three ($r,$uname,$fn,$current_page,$textref,$res,$header,$urlpath,$qcount) if $page_name eq 'Target';
-            &final_display ($r,$uname,$fn,$current_page,$textref,$res,$header,$css,$js,$webpath,$dirpath,$subdir) if $page_name eq 'Confirmation';
+            &display_zero ($r,$fn,$current_page,$webpath) if $page_name eq 'Welcome';
+            &display_one ($r,$fn,$current_page,$textref,$header) if $page_name eq 'Blocks';
+            &display_two ($r,$fn,$current_page,$textref,$header,$qcount) if $page_name eq 'Format';
+            &display_three ($r,$fn,$current_page,$textref,$res,$header,$webpath,$qcount) if $page_name eq 'Target';
+            &final_display ($r,$fn,$current_page,$textref,$res,$header,$css,$js,$webpath,$dirpath,$subdir) if $page_name eq 'Confirmation';
         }
     } elsif ($env{'form.phase'} eq 'two') {
-        my ($result,$flag) = &Apache::lonupload::phasetwo($r,$fn,$uname,$udom,'testbank');
+        my ($result,$flag) = &Apache::lonupload::phasetwo($r,$fn,'testbank');
         $r->print($result);
         if ($flag eq 'ok') {
             my $current_page = 0;
@@ -1951,13 +1955,13 @@ sub handler {
             &jscript_zero($webpath,\$js);
             $js = '<script type="text/javascript">'."\n$js\n".'</script>';
             $r->print($js);
-            &display_zero($r,$uname,$fn,$current_page,$webpath);
+            &display_zero($r,$fn,$current_page,$webpath);
         } elsif ($flag eq 'embedded') {
             $r->print($js.'<form name="testbankForm" method="post" action="/adm/testbank">'.
-                      &page_footer('',$uname,$fn).'</form>');
+                      &page_footer('',$fn).'</form>');
         }
     } else {
-        &Apache::lonupload::phaseone($r,$fn,$uname,$udom,'testbank');
+        &Apache::lonupload::phaseone($r,$fn,'testbank',$uname,$udom);
     }
     $r->print(&Apache::loncommon::end_page());
     return OK;
