@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # XML Parser Module 
 #
-# $Id: lonxml.pm,v 1.531.2.16 2014/06/21 23:39:27 raeburn Exp $
+# $Id: lonxml.pm,v 1.531.2.19 2015/04/06 17:02:33 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -323,11 +323,11 @@ sub printtokenheader {
 }
 
 sub printalltags {
-  my $temp;
-  foreach $temp (sort keys %Apache::lonxml::alltags) {
-    &Apache::lonxml::debug("$temp -- ".
-		  join(',',@{ $Apache::lonxml::alltags{$temp} }));
-  }
+    my $temp;
+    foreach $temp (sort keys %Apache::lonxml::alltags) {
+        &Apache::lonxml::debug("$temp -- ".
+                               join(',',@{ $Apache::lonxml::alltags{$temp} }));
+    }
 }
 
 sub xmlparse {
@@ -463,6 +463,8 @@ sub inner_xmlparse {
   my $result;
   my $token;
   my $dontpop=0;
+  my $lastdontpop;
+  my $lastendtag;
   my $startredirection = $Apache::lonxml::redirection;
   while ( $#$pars > -1 ) {
     while ($token = $$pars['-1']->get_token) {
@@ -558,17 +560,29 @@ sub inner_xmlparse {
       }
       $result = '';
 
-      if ($token->[0] eq 'E' && !$dontpop) {
-	&end_tag($stack,$parstack,$token);
+      if ($token->[0] eq 'E') {
+          if ($dontpop) {
+              $lastdontpop = $token;
+          } else {
+              $lastendtag = $token->[1];
+              &end_tag($stack,$parstack,$token);
+          }
       }
       $dontpop=0;
-    }	
+    }
     if ($#$pars > -1) {
 	pop @$pars;
 	pop @Apache::lonxml::pwd;
     }
   }
 
+  if (($#$stack == 0) && ($stack->[0] eq 'physnet') && ($target eq 'web') &&
+      ($lastendtag eq 'LONCAPA_INTERNAL_TURN_STYLE_ON')) {
+       if ((ref($lastdontpop) eq 'ARRAY') && ($lastdontpop->[1] eq 'physnet')) {
+           &end_tag($stack,$parstack,$lastdontpop);
+       }
+   }
+
   # if ($target eq 'meta') {
   #   $finaloutput.=&endredirection;
   # }
@@ -908,7 +922,10 @@ sub init_safespace {
   $safeeval->permit("require");
   $safeinit .= ';$external::target="'.$target.'";';
   &Apache::run::run($safeinit,$safeeval);
-  &initialize_rndseed($safeeval);
+  my $rawrndseed = &initialize_rndseed($safeeval);
+  if ($target eq 'grade') {
+      $Apache::lonhomework::rawrndseed = $rawrndseed;
+  }
 }
 
 sub clean_safespace {
@@ -947,6 +964,7 @@ sub initialize_rndseed {
     my $safeinit = '$external::randomseed="'.$rndseed.'";';
     &Apache::lonxml::debug("Setting rndseed to $rndseed");
     &Apache::run::run($safeinit,$safeeval);
+    return $rndseed;
 }
 
 sub default_homework_load {
@@ -1421,7 +1439,7 @@ sub extlink {
     if (!$exact) {
 	$res=&Apache::lonnet::hreflocation($Apache::lonxml::pwd[-1],$res);
     }
-    push(@Apache::lonxml::extlinks,$res)	 
+    push(@Apache::lonxml::extlinks,$res);
 }
 
 sub writeallows {
@@ -1551,13 +1569,15 @@ SIMPLECONTENT
 
 sub verify_html {
     my ($filecontents)=@_;
-    my ($is_html,$is_xml);
+    my ($is_html,$is_xml,$is_physnet);
     if ($filecontents =~/(?:\<|\&lt\;)\?xml[^\<]*\?(?:\>|\&gt\;)/is) {
         $is_xml = 1;
     } elsif ($filecontents =~/(?:\<|\&lt\;)html(?:\s+[^\<]+|\s*)(?:\>|\&gt\;)/is) {
         $is_html = 1;
+    } elsif ($filecontents =~/(?:\<|\&lt\;)physnet[^\<]*(?:\>|\&gt\;)/is) {
+        $is_physnet = 1;
     }
-    unless ($is_xml || $is_html) {
+    unless ($is_xml || $is_html || $is_physnet) {
         return &mt('File does not have [_1] or [_2] starting tag','&lt;html&gt;','&lt;?xml ?&gt;');
     }
     if ($is_html) {
@@ -1845,6 +1865,7 @@ sub handler {
             }
 	}
     }
+    my $inhibit_menu;
     my %mystyle;
     my $result = '';
     my $filecontents=&Apache::lonnet::getfile($file);
@@ -1902,8 +1923,15 @@ ENDNOTFOUND
 	    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
 						    ['rawmode']);
 	    if ($env{'form.rawmode'}) { $result = $filecontents; }
+            if (($env{'request.state'} eq 'construct') &&
+                (($filetype eq 'css') || ($filetype eq 'js')) && ($ENV{'HTTP_REFERER'})) {
+                if ($ENV{'HTTP_REFERER'} =~ m{^https?\://[^\/]+/priv/$LONCAPA::match_domain/$LONCAPA::match_username/[^\?]+\.(x?html?|swf)(|\?)[^\?]*$}) {
+                    $inhibit_menu = 1;
+                }
+            }
             if (($filetype ne 'html') && 
-                (!$env{'form.return_only_error_and_warning_counts'})) {
+                (!$env{'form.return_only_error_and_warning_counts'}) &&
+                (!$inhibit_menu)) {
                 my $nochgview = 1;
                 my $controls = '';
                     if ($env{'request.state'} eq 'construct') {
@@ -1941,7 +1969,7 @@ ENDNOTFOUND
 #
 # Edit action? Insert editing commands
 #
-    unless ($env{'request.state'} eq 'published') {
+    unless (($env{'request.state'} eq 'published') || ($inhibit_menu)) {
 	if ($env{'form.editmode'} && (!($env{'form.viewmode'})) && (!($env{'form.discardview'})))
 	{
             my ($displayfile,$url,$symb,$itemtitle,$action);
@@ -2140,7 +2168,7 @@ sub error {
 	    my $declutter=&Apache::lonnet::declutter($env{'request.filename'});
             my $baseurl = &Apache::lonnet::clutter($declutter);
 	    my @userlist;
-	    foreach (keys %users) {
+	    foreach (keys(%users)) {
 		my ($user,$domain) = split(/:/, $_);
 		push(@userlist,"$user:$domain");
 		my $key=$declutter.'_'.$user.'_'.$domain;
