@@ -1,7 +1,7 @@
 <!--
 The LearningOnline Network
 Resource Assembly Tool
-// $Id: ratcode.js,v 1.13.2.1 2011/11/22 00:22:09 raeburn Exp $
+// $Id: ratcode.js,v 1.13.10.2 2012/12/16 16:34:23 raeburn Exp $
 //
 // Copyright Michigan State University Board of Trustees
 //
@@ -1691,14 +1691,6 @@ function groupopen(url) {
    idx=open("/adm/groupsort?inhibitmenu=yes&mode=rat&recover=1&readfile="+url,"idxout",options);
    idx.focus();
 }
-// ------------------------------------------------------- Open bookmark window
-function bookmarkopen() {
-   var options="scrollbars=1,resizable=1,menubar=0";
-   idxmode=mode;
-   idxflag=1;
-   idx=open("/adm/groupsort?inhibitmenu=yes&mode=rat&bookmarks=1","idxout",options);
-   idx.focus();
-}
 // ------------------------------------------------- Open browser to select map
     var editbrowser;
     function openbrowser(formname,elementname,only,omit) {
