@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Handler to resolve ambiguous file locations
 #
-# $Id: lonambiguous.pm,v 1.23 2008/11/20 15:19:05 jms Exp $
+# $Id: lonambiguous.pm,v 1.26 2014/12/11 01:13:24 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -57,7 +57,7 @@ sub getlost {
     &Apache::loncommon::content_type($r,'text/html');
     $r->send_http_header;
     $r->print(&Apache::loncommon::start_page('Could not handle ambiguous resource reference').
-	      $errmsg.
+	      '<div class="LC_warning">'.$errmsg.'</div>'.
 	      &Apache::loncommon::end_page());
 }
 
@@ -87,7 +87,7 @@ sub handler {
        my $envkey;
        if (tie(%bighash,'GDBM_File',$env{'request.course.fn'}.'.db',
                     &GDBM_READER(),0640)) {
-          foreach $envkey (keys %env) {
+          foreach $envkey (keys(%env)) {
              if ($envkey=~/^form\.(\d+)\.(\d+)$/) {
 # ---------------------------------------------------- Update symb and redirect
 	         my $mapid=$1;
@@ -192,29 +192,45 @@ sub handler {
                $r->send_http_header;
                my $start_page=
 		   &Apache::loncommon::start_page('Pick Instance of Resource');
-               $r->print(<<ENDSTART);
-$start_page
-The resource you had been accessing appears more than once in this course,
-and LON-CAPA has insufficient session information to determine which instance
-of the resource you meant.
-<p>
-Please click on the instance of the resource you intended to access:
-</p>
-<table border="2">
-<tr><th>Title</th><th>Part of ...</th></tr>
-ENDSTART
+               $r->print(
+                   $start_page
+                  .'<p class="LC_warning">'
+                  .&mt('The resource you had been accessing appears more than'
+                      .' once in this course, and LON-CAPA has insufficient'
+                      .' session information to determine which instance'
+                      .' of the resource you meant.')
+                  .'</p>'
+                  .'<p>'
+                  .&mt('Please click on the instance of the resource you'
+                      .' intended to access:')
+                  .'</p>'
+               );
+               $r->print(
+                   &Apache::loncommon::start_data_table()
+                  .&Apache::loncommon::start_data_table_header_row()
+                  .'<th>'.&mt('Title').'</th>'
+                  .'<th>'.&mt('Part of ...').'</th>'
+                  .&Apache::loncommon::end_data_table_header_row()
+                  .&Apache::loncommon::start_data_table_row()
+               );
                map {
-		   my $resurl=$bighash{'src_'.$_};
+                   my $resurl=$bighash{'src_'.$_};
                    my $mapurl=$bighash{'map_id_'.(split(/\./,$_))[0]};
-		   my $symb=&make_symb($_);
-		   $r->print('<tr><td><a href="'.$resurl.'?symb='.$symb.'">'.
-			     &Apache::lonnet::gettitle($symb).
-			     '</a></td><td>'.
-			     &Apache::lonnet::gettitle($mapurl).'&nbsp;'.
-			     '</td></tr>');
+                   my $symb=&make_symb($_);
+                   $r->print(
+                       &Apache::loncommon::start_data_table_row()
+                      .'<td><a href="'.$resurl.'?symb='.$symb.'">'
+                      .&Apache::lonnet::gettitle($symb)
+                      .'</a></td>'
+                      .'<td>'.&Apache::lonnet::gettitle($mapurl).'&nbsp;</td>'
+                      .&Apache::loncommon::end_data_table_row()
+                   );
                } @possibilities;
-               $r->print('</table>'.&Apache::loncommon::end_page());
-	       untie(%bighash);
+               $r->print(
+                   &Apache::loncommon::end_data_table()
+                  .&Apache::loncommon::end_page()
+               );
+               untie(%bighash);
                return OK;
             }
          }
