@@ -2,7 +2,7 @@
 #
 # Page flip handler
 #
-# $Id: lonpageflip.pm,v 1.80.8.6 2014/06/26 18:28:25 raeburn Exp $
+# $Id: lonpageflip.pm,v 1.80.8.8 2015/03/23 12:49:10 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -36,6 +36,7 @@ use LONCAPA;
 use Apache::Constants qw(:common :http REDIRECT);
 use Apache::lonnet;
 use Apache::loncommon();
+use Apache::lonuserstate;
 use Apache::lonlocal;
 use HTML::TokeParser;
 use GDBM_File;
@@ -268,13 +269,57 @@ sub handler {
   my @possibilities=();
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['postdata']);
   if (($env{'form.postdata'})&&($env{'request.course.fn'})) {
-      $env{'form.postdata'}=~/(\w+)\:(.*)/;
-      my $direction=$1;
-      my $currenturl=$2;
+      my ($direction,$currenturl) = ($env{'form.postdata'}=~/(\w+)\:(.*)/);
       if ($currenturl=~m|^/enc/|) {
-	  $currenturl=&Apache::lonenc::unencrypted($currenturl);
+          $currenturl=&Apache::lonenc::unencrypted($currenturl);
       }
       $currenturl=~s/\.\d+\.(\w+)$/\.$1/;
+      $currenturl=~s/^https?\:\/\///;
+      $currenturl=~s/^[^\/]+//;
+      my ($preupdatepos,$last,$reinitcheck);
+      if ($direction eq 'return') {
+          if (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'_symb.db',
+                    &GDBM_READER(),0640)) {
+              $last=$hash{'last_known'};
+              untie(%hash);
+          }
+      }
+      if ($env{'request.course.id'}) {
+          # Check if course needs to be re-initialized
+          my $loncaparev = $r->dir_config('lonVersion');
+          ($reinitcheck,my @reinit) = &Apache::loncommon::needs_coursereinit($loncaparev);
+          if ($reinitcheck eq 'switch') {
+              &Apache::loncommon::content_type($r,'text/html');
+              $r->send_http_header;
+              $r->print(&Apache::loncommon::check_release_result(@reinit));
+              return OK;
+          } elsif ($reinitcheck eq 'update') {
+              my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+              my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+              $preupdatepos = &Apache::lonnet::symbread($currenturl);
+              unless ($direction eq 'return') {
+                  if (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'_symb.db',
+                                &GDBM_READER(),0640)) {
+                      $last=$hash{'last_known'};
+                      untie(%hash);
+                  }
+              }
+              my ($furl,$ferr) = &Apache::lonuserstate::readmap("$cdom/$cnum");
+              if ($ferr) {
+                  my $requrl = $r->uri;
+                  $env{'user.error.msg'}="$requrl:bre:0:0:Course not initialized";
+                  $env{'user.reinit'} = 1;
+                  return HTTP_NOT_ACCEPTABLE;
+              } else {
+                  if ($last) {
+                      my ($murl,$id,$fn)=&Apache::lonnet::decode_symb($last);
+                      unless (&Apache::lonnet::symbverify($last,$fn)) {
+                          undef($last);
+                      }
+                  }
+              }
+          }
+      }
       if ($direction eq 'firstres') {
 	  my $furl=&first_accessible_resource();
 	  &Apache::loncommon::content_type($r,'text/html');
@@ -285,12 +330,6 @@ sub handler {
       }
       if ($direction eq 'return') { 
 # -------------------------------------------------------- Return to last known
-         my $last;
-         if (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'_symb.db',
-                    &GDBM_READER(),0640)) {
-	     $last=$hash{'last_known'};
-             untie(%hash);
-         }
          my $newloc;
          if (($last) && (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'.db',
                         &GDBM_READER(),0640))) {
@@ -299,7 +338,6 @@ sub handler {
 	    $newloc=$hash{'src_'.$id};
 	    if ($newloc) {
 		if ($hash{'encrypted_'.$id}) { $newloc=&Apache::lonenc::encrypted($newloc); }
-			      
 	    } else {
 		$newloc='/adm/navmaps';
 	    }
@@ -313,31 +351,37 @@ sub handler {
 	     
 	 return REDIRECT;
       }
-      $currenturl=~s/^https?\:\/\///;
-      $currenturl=~s/^[^\/]+//;
 #
 # Is the current URL on the map? If not, start with last known URL
 #
+
       unless (&Apache::lonnet::is_on_map($currenturl)) {
-	 my $last;
-         if (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'_symb.db',
-                    &GDBM_READER(),0640)) {
-	     $last=$hash{'last_known'};
+         if ($preupdatepos) {
+             undef($preupdatepos);
+         } elsif (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'_symb.db',
+                            &GDBM_READER(),0640)) {
+             $last=$hash{'last_known'};
              untie(%hash);
          }
+         my $newloc;
          if ($last) {
 	     $currenturl=&Apache::lonnet::clutter((&Apache::lonnet::decode_symb($last))[2]);
 	 } else {
 	     &Apache::loncommon::content_type($r,'text/html');
 	     $r->header_out(Location => 
 			    &Apache::lonnet::absolute_url().
-			    '/adm/noidea.html');
+			    '/adm/navmaps');
 	     return REDIRECT;
          }
       }
 # ------------------------------------------- Do we have any idea where we are?
       my $position;
-      if ($position=Apache::lonnet::symbread($currenturl)) {
+      if ($preupdatepos) {
+          $position = $preupdatepos;
+      } else {
+          $position=Apache::lonnet::symbread($currenturl);
+      }
+      if ($position) {
 # ------------------------------------------------------------------------- Yes
 	  my ($startoutmap,$mapnum,$thisurl)=&Apache::lonnet::decode_symb($position);
           $cachehash{$startoutmap}{$thisurl}=[$thisurl,$mapnum];
@@ -390,7 +434,7 @@ sub handler {
 	      untie(%hash);
 # --------------------------------------------------------- Store position info
               $cachehash{$startoutmap}{'last_direction'}=[$direction,'notasymb'];
-              foreach my $thismap (keys %cachehash) {
+              foreach my $thismap (keys(%cachehash)) {
 		  my $mapnum=$cachehash{$thismap}->{'mapnum'};
 		  delete($cachehash{$thismap}->{'mapnum'});
 		  &Apache::lonnet::symblist($thismap,
@@ -428,7 +472,10 @@ sub handler {
 						     'pick' =>
 						     'Please click on the the resource you intend to access',
 						     'titleheader' => 'Title',
-						     'type' => 'Type');
+						     'type' => 'Type',
+                                                     'update' => 'Content updated',
+                                                     'expupdate' => 'As a result of a recent update to the sequence of materials, it is not possible to complete the page flip.',
+                                                     'gonav' => 'Go to the Contents page to select a resource to display.');
                   if (&Apache::loncommon::course_type() eq 'Community') {
                       $lt{'nav'} = &mt('Community Contents');
                   }
@@ -457,17 +504,38 @@ ENDSTART
                      }
                      $r->print('</table>');
                   } else {
-                      $r->print(
-                          &Apache::loncommon::start_page('No Resource')
-                         .'<h2>'.$lt{'title'}.'</h2>'
-                         .'<p>'.$lt{'explain'}.'</p>');
+                      if ($reinitcheck) {
+                          if (&Apache::loncommon::course_type() eq 'Community') {
+                              $r->print(
+                                  &Apache::loncommon::start_page('Community Contents Updated'));
+                          } else { 
+                              $r->print(
+                                  &Apache::loncommon::start_page('Course Contents Updated'));
+                          }
+                          $r->print('<h2>'.$lt{'update'}.'</h2>'
+                                  .'<p>'.$lt{'expupdate'}.'<br />'
+                                  .$lt{'gonav'}.'</p>');
+                      } else {
+                          $r->print(
+                              &Apache::loncommon::start_page('No Resource')
+                             .'<h2>'.$lt{'title'}.'</h2>'
+                             .'<p>'.$lt{'explain'}.'</p>');
+                      }
 		  }
-                  $r->print(
-                      &Apache::lonhtmlcommon::actionbox(
-                          ['<a href="/adm/flip?postdata=return:">'.$lt{'back'}.'</a></li>',
-                           '<a href="/adm/navmaps">'.$lt{'nav'}.'</a></li>'
-                          ])
-                     .&Apache::loncommon::end_page());
+                  if ((!@possibilities) && ($reinitcheck))  {
+                      $r->print(
+                          &Apache::lonhtmlcommon::actionbox(
+                              ['<a href="/adm/navmaps">'.$lt{'nav'}.'</a></li>'
+                              ]));
+                  } else {
+                      $r->print(
+                          &Apache::lonhtmlcommon::actionbox(
+                              ['<a href="/adm/flip?postdata=return:">'.$lt{'back'}.'</a></li>',
+                               '<a href="/adm/navmaps">'.$lt{'nav'}.'</a></li>'
+                              ]));
+                  }
+                  $r->print(&Apache::loncommon::end_page());
+      
                   return OK;
 	      }
 	  } else {
